/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "main.hpp"
#include "ui.hpp"
#include <iostream>
#include "ai.hpp"
#include "combat.hpp"
#include "adventure.hpp"
#include <stdio.h>
#include "game.hpp"
#include "utils.hpp"
#include "mobileui.hpp"

Cell advmap[MAPHEIGHT][MAPWIDTH];
vector<Mobile> advmobs;
vector<Item> advobjs;
Mission mission;
ToolBar *advmenu = new ToolBar(80, 2, "Stats");
Label *lblexp = new Label(0, 0, "");
Label *lblhp = new Label(0, 0, "");
Label *lblname = new Label(0, 0, "");
Label *lbllevel = new Label(0, 0, "");
Label *lbldungeon = new Label(0, 0, "");
int dungeonlevel = 1;
int oldlevel = 1;

//Lots of code here are borrowed from libtcod sample_cpp.cpp file.

// draw a vertical line
void vline(int x, int y1, int y2) 
{
  int y = y1;
  int dy = (y1 > y2 ? -1 : 1);
  advmap[y][x].ch = ' ';
  if ( y1 == y2 ) 
    {
      return;
    }
  do 
    {
      y += dy;
      advmap[y][x].ch = ' ';
    } 
  while (y != y2);
}


// draw a vertical line up until we reach an empty space
void vline_up(int x, int y) 
{
  while (y >= 0 && advmap[y][x].ch != ' ') 
    {
      advmap[y][x].ch = ' ';
      y--;
  }
}

// draw a vertical line down until we reach an empty space
void vline_down(int x, int y) 
{
  while (y < MAPHEIGHT && advmap[y][x].ch != ' ') 
    {
      advmap[y][x].ch = ' ';
      y++;
    }
}

// draw a horizontal line
void hline(int x1, int y, int x2) 
{
  int x = x1;
  int dx = (x1 > x2 ? -1 : 1);
  advmap[y][x].ch = ' ';
  if ( x1 == x2 )
    { 
      return;
    }
  do 
    {
      x += dx;
      advmap[y][x].ch = ' ';
    } 
  while (x != x2);
}

// draw a horizontal line left until we reach an empty space
void hline_left(int x, int y) 
{
  while (x >= 0 && advmap[y][x].ch != ' ') 
    {
      advmap[y][x].ch = ' ';
      x--;
    }
}

// draw a horizontal line right until we reach an empty space
void hline_right(int x, int y) 
{
  while (x < MAPWIDTH && advmap[y][x].ch != ' ') 
    {
      advmap[y][x].ch = ' ';
      x++;
    }
}

// the class building the dungeon from the bsp nodes
class BspListener : public ITCODBspCallback 
{
public :
  bool visitNode(TCODBsp *node, void *userData) 
  {
    if (node->isLeaf()) 
      {
	// calculate the room size
	int minx = node->x + 1;
	int maxx = node->x + node->w - 1;
	int miny = node->y + 1;
	int maxy = node->y + node->h - 1;
	if (maxx == MAPWIDTH - 1) 
	  {
	    maxx--;
	  }
	if (maxy == MAPHEIGHT - 1) 
	  {
	    maxy--;
	  }
	minx = TCODRandom::getInstance()->getInt(minx, maxx - 11);
	miny = TCODRandom::getInstance()->getInt(miny, maxy - 11);
	maxx = TCODRandom::getInstance()->getInt(minx + 9, maxx);
	maxy = TCODRandom::getInstance()->getInt(miny + 9, maxy);
	// resize the node to fit the room
	node->x = minx;
	node->y = miny;
	node->w = maxx - minx + 1;
	node->h = maxy - miny + 1;
	// dig the room
	for (int x = minx; x <= maxx; x++ ) 
	  {
	    for (int y = miny; y <= maxy; y++ ) 
	      {
		advmap[y][x].ch = ' ';
	      }
	  }
      } 
    else 
      {
	// resize the node to fit its sons
	TCODBsp *left = node->getLeft();
	TCODBsp *right = node->getRight();
	node->x = MIN(left->x, right->x);
	node->y = MIN(left->y, right->y);
	node->w = MAX(left->x + left->w, right->x + right->w) - node->x;
	node->h = MAX(left->y + left->h, right->y + right->h) - node->y;
	// create a corridor between the two lower nodes
	if (node->horizontal) 
	  {
	    // vertical corridor
	    if (left->x + left->w - 1 < right->x || right->x + right->w - 1 < left->x) 
	      {
		// no overlapping zone. we need a Z shaped corridor
		int x1 = TCODRandom::getInstance()->getInt(left->x, left->x + left->w - 1);
		int x2 = TCODRandom::getInstance()->getInt(right->x, right->x + right->w - 1);
		int y = TCODRandom::getInstance()->getInt(left->y + left->h, right->y);
		vline_up(x1, y - 1);
		hline(x1, y, x2);
		vline_down(x2, y + 1);
	      } 
	    else 
	      {
		// straight vertical corridor
		int minx = MAX(left->x, right->x);
		int maxx= MIN(left->x + left->w - 1, right-> x + right->w - 1);
		int x = TCODRandom::getInstance()->getInt(minx, maxx);
		vline_down(x, right->y);
		vline_up(x, right->y - 1);
	      }
	  } 
	else 
	  {
	    // horizontal corridor
	    if (left->y + left->h - 1 < right->y || right->y + right->h - 1 < left->y) 
	      {
		// no overlapping zone. we need a Z shaped corridor
		int y1 = TCODRandom::getInstance()->getInt(left->y, left->y + left->h - 1);
		int y2 = TCODRandom::getInstance()->getInt(right->y, right->y + right->h - 1);
		int x = TCODRandom::getInstance()->getInt(left->x + left->w, right->x);
		hline_left(x - 1, y1);
		vline(x, y1, y2);
		hline_right(x + 1, y2);
	      } 
	    else 
	      {
		// straight horizontal corridor
		int miny = MAX(left->y, right->y);
		int maxy = MIN(left->y + left->h - 1, right->y + right->h - 1);
		int y = TCODRandom::getInstance()->getInt(miny, maxy);
		hline_left(right->x - 1, y);
		hline_right(right->x, y);
	      }
	  }
      }
    return true;
  }
};

/**
 * Render random dungeon.
 */
void render_dungeon() 
{
  for (int y = 0; y < MAPHEIGHT; y++)
    {
      for (int x = 0; x < MAPWIDTH; x++)
	{
	  advmap[y][x].ch = 178;
	}
    }
  fovmap->clear();
  // create the bsp
  TCODBsp *bsp = new TCODBsp(0, 0, MAPWIDTH, MAPHEIGHT);
  // build a new random bsp tree
  bsp->splitRecursive(NULL, 8, 10, 10, 1.5f, 1.5f);
  // create the dungeon from the bsp
  bsp->traverseInvertedLevelOrder(new BspListener(), NULL);
  delete bsp;
  // Workaround libtcod bug.
  for (int y = 240; y < MAPHEIGHT; y++ ) 
    {
      for (int x = 0; x < MAPWIDTH; x++ ) 
	{
	  if (fortmap[y][x].items.size() == 32)
	    {
	      fortmap[y][x].items.clear();
	    }
	}
    }
  // render the level
  bool wall;
  for (int y = 0; y < MAPHEIGHT; y++ ) 
    {
      for (int x = 0; x < MAPWIDTH; x++ ) 
	{
	  if (advmap[y][x].ch == 178)
	    {
	      advmap[y][x].walkable = false;
	      advmap[y][x].transparent = false;
	      advmap[y][x].color = TCODColor::black;
	      advmap[y][x].bgcolor = TCODColor::black;
	    }
	  else
	    {
	      fovmap->setProperties(x, y, true, true);
	      advmap[y][x].walkable = true;
	      advmap[y][x].transparent = true;
	      advmap[y][x].color = TCODColor(0,0,100);
	      advmap[y][x].bgcolor = TCODColor(0,0,100);
	    }
	  advmap[y][x].visited = false;
	  advmap[y][x].items.clear();
	  advmap[y][x].hitems.clear();
	  advmap[y][x].mob = NONE;
	  advmap[y][x].building = NONE;
	}
    }
  //Draw walls
  for (int y = 0; y < MAPHEIGHT; y++ ) 
    {
      for (int x = 0; x < MAPWIDTH; x++ ) 
	{
	  if (advmap[y][x].ch == 178)
	    {
	      wall = false;
	      for (int i = -1; i < 2; i++)
		{
		  for (int j = -1; j < 2; j++)
		    {
		      if ((i == 0) && (j == 0))
			{
			  continue;
			}
		      if (advmap[y + i][x + j].walkable)
			{
			  wall = true;
			  break;
			}
		    }
		}
	      if (wall)
		{
		  advmap[y][x].bgcolor = TCODColor(0,0,100);
		}
	    }
	}
    }
  //Add a player
  advmobs[0].setmob(MAPWIDTH / 2, MAPHEIGHT / 2, 0, true);
  //Add some animals
  TCODList<int> npcs = find_all_proto_mobs(ANIMAL, FORTONLY, advmobs[0].stats[10] + mission.difficult);
  TCODRandom *generator = TCODRandom::getInstance();
  int amount = generator->getInt(10, 100);
  int result, x, y;
  for (int i = 0; i < amount; i++)
    {
      if (npcs.size() > 1)
	{
	  result = generator->getInt(0, npcs.size() - 1);
	}
      else
	{
	  result = 0;
	}
      x = generator->getInt(0, MAPWIDTH - 1);
      y = generator->getInt(0, MAPHEIGHT - 1);
      protomobs[npcs.get(result)].setmob(x, y, NONE);
      advmobs[advmobs.size() - 1].generatestats();
    }
  result = NONE;
  //Add mission target
  if (mission.targettype == MOB)
    {
      result = find_proto_mob(mission.target);
    }
  if ((mission.targettype == MOB) && (mission.length == dungeonlevel))
    {
      x = generator->getInt(0, MAPWIDTH - 1);
      y = generator->getInt(0, MAPHEIGHT - 1);
      protomobs[result].setmob(x, y, NONE);
      advmobs[advmobs.size() - 1].generatestats(advmobs[0].stats[10]);
    }
  //Add some monsters
  npcs = find_all_proto_mobs(MONSTER, FORTONLY, advmobs[0].stats[10] + mission.difficult);
  int index;
  amount = generator->getInt(10, 100);
  for (int i = 0; i < amount; i++)
    {
      while (true)
	{
	  if (npcs.size() > 1)
	    {
	      index = generator->getInt(0, npcs.size() - 1);
	    }
	  else
	    {
	      index = 0;
	      if (result != NONE)
		{
		  if (protomobs[npcs.get(index)].name.compare(protomobs[result].name) == 0)
		    {
		      break;
		    }
		}
	    }
	  if (result != NONE)
	    {
	      if (protomobs[npcs.get(index)].name.compare(protomobs[result].name))
		{
		  x = generator->getInt(0, MAPWIDTH - 1);
		  y = generator->getInt(0, MAPHEIGHT - 1);
		  protomobs[index].setmob(x, y, NONE);
		  advmobs[advmobs.size() - 1].generatestats();
		  break;
		}
	    }
	  else
	    {
	      x = generator->getInt(0, MAPWIDTH - 1);
	      y = generator->getInt(0, MAPHEIGHT - 1);
	      protomobs[index].setmob(x, y, NONE);
	      advmobs[advmobs.size() - 1].generatestats();
	      break;
	    }
	}
    }
  x = advmobs[0].x;
  y = advmobs[0].y;
  int stair1, stair2;
  if (dungeonlevel >= oldlevel)
    {
      stair1 = 0;
      stair2 = 1;
    }
  else
    {
      stair1 = 1;
      stair2 = 0;
    }
  oldlevel = dungeonlevel;
  //Add stairs up
  advmap[y][x].building = stair1;
  //Add stairs down
  bool valid = false;
  if (mission.length > dungeonlevel)
    {
      while (!valid)
	{
	  x = generator->getInt(0, MAPWIDTH - 1);
	  y = generator->getInt(0, MAPHEIGHT - 1);
	  valid = valid_pos(x, y);
	}
      advmap[y][x].building = stair2;
    }
}

/**
 * Save game and quit.
 * @param w Unused.
 * @param data Unused.
 */
void asaveClb(Widget *w, void *data)
{
  setVisibleAdvmenu(false);
  save_game();
  Widget::setForegroundColor(TCODColor::white, TCODColor::red);
}

/**
 * Show player information.
 */
void draw_UI()
{
  char *c = new char[10];
  sprintf(c, "HP: %d/%d", advmobs[0].stats[7], advmobs[0].stats[8]);
  lblhp->setValue(c);
  c = new char[10];
  sprintf(c, "Exp: %d", advmobs[0].stats[11]);
  lblexp->setValue(c);
  c = new char[15];
  sprintf(c, "Level: %d", advmobs[0].stats[10]);
  lbllevel->setValue(c);
  c = new char[20];
  sprintf(c, "Dungeon: %d of %d", dungeonlevel, mission.length);
  lbldungeon->setValue(c);
  lblname->setValue(advmobs[0].name.c_str());
  advmenu->computeSize();
}

/**
 * Draw adventure map
 */
void draw_adventure()
{
  TCODConsole::root->clear();
  screenx = advmobs[0].x - (SCREENWIDTH / 2);
  screeny = advmobs[0].y - (SCREENHEIGHT / 2);
  if (screenx < 0) 
    {
      screenx = 0;
    }
  else if ((screenx + SCREENWIDTH) >= MAPWIDTH)
    {
      screenx = MAPWIDTH - SCREENWIDTH;
    }
  if (screeny < 0)
    {
      screeny = 0;
    }
  else if ((screeny + SCREENHEIGHT) >= MAPHEIGHT)
    {
      screeny = MAPHEIGHT - SCREENHEIGHT;
    }
  fovmap->computeFov(advmobs[0].x, advmobs[0].y, 10);
  TCODColor bgcolor;
  TCODColor color;
  int ch;
  for (int y = 0; y < SCREENHEIGHT; y++)
    {
      for (int x = 0; x < SCREENWIDTH; x++)
	{
	  bgcolor = advmap[y + screeny][x + screenx].bgcolor;
	  color = advmap[y + screeny][x + screenx].color;
	  ch = advmap[y + screeny][x + screenx].ch;
	  if ((advmap[y + screeny][x + screenx].building == 0) && (advmap[y + screeny][x + screenx].mob == NONE))
	    {
	      ch = 30;
	    }
	  if ((advmap[y + screeny][x + screenx].building == 1) && (advmap[y + screeny][x + screenx].mob == NONE))
	    {
	      ch = 31;
	    }
	  if (fovmap->isInFov(x + screenx, y + screeny))
	    {
	      if (!advmap[y + screeny][x + screenx].visited)
		{
		  advmap[y + screeny][x + screenx].visited = true;
		}
	      bgcolor = advmap[y + screeny][x + screenx].bgcolor + TCODColor::yellow;
	    }
	  else if ((ch != 30) && (ch != 178))
	    {
	      ch = ' ';
	    }
	  if (!advmap[y + screeny][x + screenx].visited)
	    {
	      color = TCODColor::black;
	      bgcolor = TCODColor::black;
	    }
	  TCODConsole::root->putCharEx(x, y, ch, color, bgcolor);
	}
    }

  draw_UI();
}

/**
 * Set player equipment on adventure start/end.
 * @param state 0 - start adventure, 1 - change level, 2 - end adventure.
 */
void set_equip(int state)
{
  int index;
  //Start adventure
  if (state == 0)
    {
      //Set equipment
      for (int i = 0; i < 5; i++)
	{
	  index = advmobs[0].equipment[i];
	  if (index == NONE)
	    {
	      continue;
	    }
	  advobjs.push_back(fortobjs[index]);
	  fortobjs[index].deleteitem();
	  advmobs[0].equipment[i] = advobjs.size() - 1;
	}
      //Set inventory
      if (!advmobs[0].inventory.isEmpty())
	{
	  for (int i = 0; i < advmobs[0].inventory.size(); i++)
	    {
	      index = advmobs[0].inventory.get(i);
	      advobjs.push_back(fortobjs[index]);
	      fortobjs[index].deleteitem();
	      advmobs[0].inventory.set(advobjs.size() - 1, i);
	    }
	}
    }
  //Change level of dungeons
  else if (state == 1)
    {
      TCODList<int> items;
      for (int i = 0; i < 5; i++)
	{
	  index = advmobs[0].equipment[i];
	  if (index == NONE)
	    {
	      continue;
	    }
	  items.push(index);
	}
      if (!advmobs[0].inventory.isEmpty())
	{
	  for (int i = 0; i < advmobs[0].inventory.size(); i++)
	    {
	      items.push(advmobs[0].inventory.get(i));
	    }
	}
      for (int i = 0; i < (int)advobjs.size(); i++)
	{
	  if (!items.contains(i))
	    {
	      advobjs[i].deleteitem();
	    }
	}
      items.clear();
    }
  //End of adventure
  else if (state == 2)
    {
      //Set equipment
      for (int i = 0; i < 5; i++)
	{
	  index = advmobs[0].equipment[i];
	  if (index == NONE)
	    {
	      continue;
	    }
	  fortobjs.push_back(advobjs[index]);
	  fortmobs[0].equipment[i] = advobjs.size() - 1;
	}
      //Set inventory
      if (!advmobs[0].inventory.isEmpty())
	{
	  for (int i = 0; i < advmobs[0].inventory.size(); i++)
	    {
	      index = advmobs[0].inventory.get(i);
	      fortobjs.push_back(advobjs[index]);
	      fortmobs[0].inventory.set(fortobjs.size() - 1, i);
	    }
	}
    }
}

/**
 * Finish adventure and back to fort.
 */
void end_adventure()
{
  advmobs[0].target = NONE;
  advmobs[0].targettype = NONE;
  fortmobs[0] = advmobs[0];
  fortmobs[0].setmob(MAPWIDTH / 2, MAPHEIGHT / 2, 0);
  screenx = (MAPWIDTH / 2) - (SCREENWIDTH / 2);
  screeny = (MAPHEIGHT / 2) - (SCREENHEIGHT / 2);
  for (int y = 0; y < MAPWIDTH; y++)
    {
      for (int x = 0; x < MAPHEIGHT; x++)
	{
	  fovmap->setProperties(x, y, fortmap[y][x].transparent, fortmap[y][x].walkable);
	}
    }
  set_equip(2);
  advmobs.clear();
  advobjs.clear();
  setVisibleAdvmenu(false);
  state = FORT;
  setTime();
  setVisibleGamemenu(true);
  tips->setVisible(true);
  Notice("Back from mission.");
}

/**
 * Go to next level of dungeons.
 */
void next_level()
{
  TCODConsole::root->clear();
  set_equip(1);
  Mobile mob = advmobs[0];
  advmobs.clear();
  advmobs.push_back(mob);
  advmobs[0].target = NONE;
  advmobs[0].targettype = NONE;
  render_dungeon();
  draw_adventure();
}

/**
 * Update state of all mobiles and items in adventure mode.
 */
void update_adventure()
{
  float time = TCODSystem::getLastFrameLength();
  for (int i = 0; i < (int)advmobs.size(); i++)
    {
      if ((advmobs[i].name.compare(mission.target) == 0) && ((advmobs[i].x == NONE) || (advmobs[i].y == NONE)))
	{
	  end_adventure();
	  return;
	}
      if ((advmobs[i].x == NONE) || (advmobs[i].y == NONE))
	{
	  continue;
	}
      //Move all mobs
      if (!advmobs[i].path->isEmpty())
	{
	  advmobs[i].movestep(time, i);
	}
      else if (advmobs[i].worktime > 0)
	{
	  //Reduce time of work
	  advmobs[i].setworktime(time);
	}
      //Update behavior of NPC's
      if ((advmobs[i].type != PLAYER) && (advmobs[i].type != ROBOT))
	{
	  set_movement(&advmobs[i]);
	}
    }

  update_combat();

  cast_spells();

  draw_adventure();

  //Stairs.
  if ((advmobs[0].targettype == BUILDING) && (advmobs[0].worktime <= 0))
    {
      if (advmobs[0].target == 0)
	{
	  if (dungeonlevel == 1)
	    {
	      end_adventure();
	    }
	  else
	    {
	      dungeonlevel --;
	      next_level();
	    }
	}
      else
	{
	  dungeonlevel ++;
	  next_level();
	}
    }

  //Take item.
  if ((advmobs[0].targettype == ITEM) && (advmobs[0].worktime <= 0))
    {
      if (advmobs[0].inventory.size() < 20)
	{
	  advmobs[0].inventory.push(advmobs[0].target);
	  int index = advmobs[0].target;
	  advobjs[index].useitem();
	}
      else
	{
	  Notice("You don't have room for that item.");
	}
      advmobs[0].targettype = NONE;
      advmobs[0].target = NONE;
    }
}

/**
 * Start adventure, create map, fill with player, mobs, etc.
 * @param index Index of mission.
 */
void start_adventure(int index)
{
  mission = protomissions[index];
  TCODConsole::root->clear();
  state = ADVENTURE;
  advmobs.push_back(fortmobs[0]);
  advmobs[0].target = NONE;
  advmobs[0].targettype = NONE;
  dungeonlevel = 1;
  set_equip(0);
  if (protomissions[index].location == DUNGEONS)
    {
      render_dungeon();
    }
  draw_adventure();
  lblname->setValue(advmobs[0].name.c_str());
  setVisibleAdvmenu(true);
  Notice("Mission started!");
}

/**
 * Move, set target for player
 * @param x X-axis position of point to move.
 * @param y Y-axis position of point to move.
 */
void move_player(int x, int y)
{
  x = screenx + x;
  y = screeny + y;
  advmobs[0].generatepath(x, y);
  advmobs[0].targettype = NONE;
  advmobs[0].target = NONE;
  if (advmap[y][x].mob != NONE)
    {
      advmobs[0].worktime = 1;
      advmobs[0].targettype = MOB;
      advmobs[0].target = advmap[y][x].mob;
    }
  else if (advmap[y][x].building != NONE)
    {
      advmobs[0].worktime = 1;
      advmobs[0].targettype = BUILDING;
      advmobs[0].target = advmap[y][x].building;
    }
  else if (!advmap[y][x].items.isEmpty())
    {
      advmobs[0].worktime = 0.5;
      advmobs[0].targettype = ITEM;
      advmobs[0].target = advmap[y][x].items.peek();
    }
}

/**
 * Show available spells
 * @param w Unused.
 * @param data Unused.
 */
void castClb(Widget *w, void* data)
{
  if (advmobs[0].spells.isEmpty())
    {
      Notice("You don't know any spells!");
      return;
    }
  state = CAST2;
  setVisibleSpells(true);
  setVisibleAdvmenu(false);
}

/**
 * Build adventure UI.
 */
void buildAdventureUI()
{
  advmenu->addWidget(lbldungeon);
  advmenu->addSeparator("");
  advmenu->addWidget(lblname);
  advmenu->addWidget(lbllevel);
  advmenu->addWidget(new Button("Stats", "Show information about character", mobClb, new int(0)));
  advmenu->addWidget(new Button("Cast spell", "Select target and spell to cast", castClb, NULL));
  advmenu->addWidget(new Button("Use item", "Use item from your inventory.", useClb, NULL));
  advmenu->addSeparator("");
  advmenu->addWidget(lblhp);
  advmenu->addWidget(lblexp);
  advmenu->addSeparator("");
  advmenu->addWidget(new Button("Help", "Usefull information", helpClb, NULL));
  advmenu->addWidget(new Button("Make screenshot", "Make screenshot and save it in main game directory.", screenshotClb, NULL));
  advmenu->addWidget(new Button("Save and quit", "Save game and quit to main menu.", asaveClb, NULL));
  advmenu->setVisible(false);
}

/**
 * Show/hide adventure menu.
 * @param visible If true show menu, otherwise hide.
 */
void setVisibleAdvmenu(bool visible)
{
  if (visible)
    {
      draw_UI();
    }
  advmenu->setVisible(visible);
  tips->setVisible(!visible);
  lblnotice->setVisible(visible);
}
