/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "ai.hpp"
#include "main.hpp"
#include <iostream>
#include "adventure.hpp"

/**
 * Generate random walk path.
 * @param mob Mob for which path is generated.
 * @param minx Min value for X-axis.
 * @param miny Min value for Y-axis.
 * @param maxx Max value for X-axis.
 * @param maxy Max value for Y-axis.
 */
void generate_move(Mobile *mob, int minx, int miny, int maxx, int maxy)
{
  TCODRandom *generator = TCODRandom::getInstance();
  int dx = generator->getInt(mob->x - minx, mob->x + maxx);
  int dy = generator->getInt(mob->y - miny, mob->y + maxy);
  mob->generatepath(dx, dy);
}

/**
 * Set movement for mobile.
 * @param mob Mobile for which movement will be set.
 */
void set_movement(Mobile *mob)
{
  //Escape from player or robots
  if ((mob->ais[2] == COWARD) && (mob->walktime <= 0))
    {
      TCODList<int> *tmplist = new TCODList<int>();
      tmplist->push(PLAYER);
      tmplist->push(ROBOT);
      int index = find_near_mob("", mob->x, mob->y, 5, tmplist, 50 + mob->stats[4]);
      delete tmplist;
      if (index != NONE)
	{
	  int minx, miny, maxx, maxy;
	  if (state == FORT)
	    {
	      if (fortmobs[index].x >= mob->x)
		{
		  minx = 10;
		  maxx = 0;
		}
	      else
		{
		  minx = 0;
		  maxx = 10;
		}
	      if (fortmobs[index].y >= mob->y)
		{
		  miny = 10;
		  maxy = 0;
		}
	      else
		{
		  miny = 0;
		  maxy = 10;
		}
	    }
	  else
	    {
	      if (advmobs[index].x >= mob->x)
		{
		  minx = 10;
		  maxx = 0;
		}
	      else
		{
		  minx = 0;
		  maxx = 10;
		}
	      if (fortmobs[index].y >= mob->y)
		{
		  miny = 10;
		  maxy = 0;
		}
	      else
		{
		  miny = 0;
		  maxy = 10;
		}
	    }
	  generate_move(mob, minx, miny, maxx, maxy);
	  return;
	}
    }
  //Go to player or robot
  if ((mob->ais[2] == HOSTILE) && (mob->walktime <= 0))
    {
      TCODList<int> *tmplist = new TCODList<int>();
      tmplist->push(PLAYER);
      tmplist->push(ROBOT);
      int index = find_near_mob("", mob->x, mob->y, 5, tmplist, 50 + mob->stats[4]);
      delete tmplist;
      if (index != NONE)
	{
	  if (state == FORT)
	    {
	      mob->generatepath(fortmobs[index].x, fortmobs[index].y);
	    }
	  else
	    {
	      mob->generatepath(advmobs[index].x, advmobs[index].y);
	    }
	  return;
	}
    }
  TCODRandom *generator = TCODRandom::getInstance();
  //Random movement
  if ((mob->ais[0] == RANDOM) && (mob->path->isEmpty()) && (mob->worktime <= 0))
    {
      TCODRandom *generator = TCODRandom::getInstance();
      if (generator->getInt(0, 10) > 8)
	{
	  generate_move(mob, 10, 10, 10, 10);
	}
      else
	{
	  mob->worktime = generator->getInt(0, 5);
	}
    }
  //Immobile
  if ((mob->ais[0] == NONE) && (mob->worktime <= 0))
    {
      mob->worktime = generator->getInt(0, 5);
    }
}
