/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include <string>
#include <vector>
#include <iostream>
#include "building.hpp"
#include "libtcod.hpp"
#include "main.hpp"
#include "utils.hpp"
#include "gui/gui.hpp"
#include "ui.hpp"
#include <stdio.h>

Container *buildmenu = new Container(0, 0, 100, 75);
VBox *constructmenu = new VBox(75, 2, 1);

/**
 * Standard class constructor. Fill some default values.
 */
Building::Building()
{
  constructed = false;
  underconstruction = 'N';
  unique = false;
  data = -1;
  taken = -1;
  description = "";
}

/**
 * Build build user interface.
 */
void buildBuildUI()
{
  buildmenu->setVisible(false);
  ToolBar *cmenu = new ToolBar(80, 30, "Movement of map");
  cmenu->addWidget(new Label(0, 0, "UP - up arrow"));
  cmenu->addWidget(new Label(0, 0, "LEFT - left arrow"));
  cmenu->addWidget(new Label(0, 0, "RIGHT - right arrow"));
  cmenu->addWidget(new Label(0, 0, "DOWN - down arrow"));
  cmenu->addSeparator("Build");
  cmenu->addWidget(new Label(0, 0, "Left button - build"));
  cmenu->addWidget(new Label(0, 0, "ESCAPE - back to menu"));
  constructmenu->addWidget(cmenu);
  constructmenu->setVisible(false);
}

/**
 * Start placing construction.
 * @param w Unused
 * @param data Name of building to construct.
 */
void buildingClb(Widget *w, void* data)
{
  string* p = (string*)data;
  int index = find_proto_build(*p);
  state = CONSTRUCT;
  stage = -1;
  setVisibleBuild(false);
  TCODMouse::showCursor(false);
  if (gtime[2] < 3)
    {
      Widget::setForegroundColor(TCODColor::black, TCODColor::red);
    }
  constructmenu->setVisible(true);
  construct(index);
}

/**
 * Show/hide information about build.
 * @param visible If true, show build options, otherwise hide menu.
 */
void setVisibleBuild(bool visible)
{
  if (visible)
    {
      constructmenu->setVisible(false);
      buildmenu->clear();
      buildmenu->addWidget(new Label(7, 2, "Name"));
      buildmenu->addWidget(new Label(47, 2, "Active"));
      buildmenu->addWidget(new Label(60, 2, "Material"));
      string text;
      bool construction;
      char* c;
      int line = 4, index;
      for (int i = 0; i < (int)protobuilds.size(); i++)
	{
	  if (protobuilds[i].unique)
	    {
	      index = find_build(protobuilds[i].name);
	      if (index != NONE)
		{
		  continue;
		}
	    }
	  text = "Construct " + protobuilds[i].name;
	  construction = false;
	  for (int j = 0; j < (int)fortbuilds.size(); j++)
	    {
	      if ((fortbuilds[j].name.compare(protobuilds[i].name) == 0) && (fortbuilds[j].underconstruction == 'Y'))
		{
		  construction = true;
		  break;
		}
	    }
	  if (construction)
	    {
	      protobuilds[i].underconstruction = 'Y';
	      buildmenu->addWidget(new Label(7, line, protobuilds[i].name.c_str()));	      
	    }
	  else
	    {
	      buildmenu->addWidget(new Button(7, line, 20, 1, text.c_str(), protobuilds[i].description.c_str(), buildingClb, &protobuilds[i].name));
	    }
	  c = new char[4];
	  sprintf(c, "%c", protobuilds[i].underconstruction);
	  buildmenu->addWidget(new Label(47, line, c));
	  c = new char[35];
	  sprintf(c, "%s: %d", protobuilds[i].mname.c_str(), protobuilds[i].mamount);
	  buildmenu->addWidget(new Label(60, line, c));
	  line ++;
	}
    }
  buildmenu->setVisible(visible);
  escinfo->setVisible(visible);
  TCODMouse::showCursor(true);
}

/**
 * Construct selected building
 * @param i Index of building
 * @param keycode Last key pressed code
 * @return Number of game stage
 */
int construct(int i, int keycode)
{
  TCOD_mouse_t mouse = TCODMouse::getStatus();
  if (stage == -1)
    {
      protobuilds[i].x = screenx + (SCREENWIDTH / 2);
      protobuilds[i].y = screeny + (SCREENHEIGHT / 2);
      stage = i;
    }
  draw_fort();
  //Draw square for construction
  TCODColor color;
  switch (keycode)
    {
    case TCODK_LEFT:
      screenx -=2;
      if (screenx < 0)
	{
	  screenx = 0;
	}
      break;
    case TCODK_RIGHT:
      screenx +=2;
      if (screenx + SCREENWIDTH > MAPWIDTH)
	{
	  screenx = MAPWIDTH - SCREENWIDTH;
	}
      break;
    case TCODK_UP:
      screeny -= 2;
      if (screeny < 0)
	{
	  screeny = 0;
	}
      break;
    case TCODK_DOWN:
      screeny += 2;
      if (screeny + SCREENHEIGHT > MAPHEIGHT)
	{
	  screeny = MAPHEIGHT - SCREENHEIGHT;
	}
      break;
    default:
      break;
    }
  offconsole->clear();
  bool valid = false;
  protobuilds[stage].x = mouse.cx + screenx;
  protobuilds[stage].y = mouse.cy + screeny;
  if (!protobuilds[stage].neededitems.isEmpty())
    {
      for (int j = 0; j < protobuilds[stage].height; j++)
	{
	  for (int k = 0; k < protobuilds[stage].width; k++)
	    {
	      if (find_obj("", protobuilds[stage].x + k, protobuilds[stage].y + j, &protobuilds[stage].neededitems) != NONE)
		{
		  valid = true;
		  break;
		}
	    }
	}
    }
  else
    {
      valid = true;
    }
  if (valid)
    {
      for (int j = 0; j < protobuilds[stage].height; j++)
	{
	  for (int k = 0; k < protobuilds[stage].width; k++)
	    {
	      if (fovmap->isWalkable(protobuilds[stage].x + k, protobuilds[stage].y + j))
		{
		  color = TCODColor::green;
		  if ((k == protobuilds[stage].enter[0]) && (j == protobuilds[stage].enter[1]))
		    {
		      color = TCODColor::black;
		    }
		}
	      else
		{
		  color = TCODColor::red;
		  valid = false;
		}
	      offconsole->putCharEx(k, j, 'X', color, TCODColor::black);
	    }
	}
    }
  else
    {
      for (int j = 0; j < protobuilds[stage].height; j++)
	{
	  for (int k = 0; k < protobuilds[stage].width; k++)
	    {
	      offconsole->putCharEx(k, j, 'X', TCODColor::red, TCODColor::black);
	    }
	}
    }
  TCODConsole::blit(offconsole, 0, 0, protobuilds[stage].width, protobuilds[stage].height, TCODConsole::root, protobuilds[stage].x - screenx, protobuilds[stage].y - screeny, 1.0, 0.0);
  if ((mouse.lbutton) && (valid))
    {
      protobuilds[stage].underconstruction = 'Y';
      Building build = protobuilds[stage];
      fortbuilds.push_back(build);
      for (int j = 0; j < build.height; j++)
	{
	  for (int k = 0; k < build.width; k++)
	    {
	      if ((k == build.enter[0]) && (j == build.enter[1]))
		{
		  fovmap->setProperties(build.x + k, build.y + j, false, true);
		  fortmap[build.y + j][build.x + k].walkable = true;
		  fortmap[build.y + j][build.x + k].color = TCODColor::white;
		}
	      else
		{
		  fovmap->setProperties(build.x + k, build.y + j, false, false);
		  fortmap[build.y + j][build.x + k].walkable = false;
		  fortmap[build.y + j][build.x + k].color = TCODColor::black;
		}
	      fortmap[build.y + j][build.x + k].ch = 'X';
	      fortmap[build.y + j][build.x + k].building = fortbuilds.size() - 1;
	      fortmap[build.y + j][build.x + k].transparent = false;
	    }
	}
      //If here is any moveable object, move it a bit
      int exists;
      for (int j = 0; j < build.height; j++)
	{
	  for (int k = 0; k < build.width; k++)
	    {
	      if (fortmap[build.y + j][build.x + k].items.size() > 0)
		{
		  for (int l = 0; l < fortmap[build.y + j][build.x + k].items.size(); l++)
		    {
		      exists = fortmap[build.y + j][build.x + k].items.get(l);
		      //Don't move veins
		      if (fortobjs[exists].types.contains(VEIN))
			{
			  continue;
			}
		      fortobjs[exists].setitem(build.x + k - 1, build.y + j - 1, exists);
		    }
		}
	      fortmap[build.y + j][build.x + k].items.clear();
	    }
	}
      protobuilds[stage].x = NONE;
      protobuilds[stage].y = NONE;
      exists = find_order("Construct " + build.name);
      if (exists == NONE)
	{
	  Order *order = new Order();
	  order->name = "Construct " + build.name;
	  order->stages[0][0] = ITEM;
	  order->stages[0][1] = build.material;
	  order->stages[0][2] = 1;
	  order->stages[0][6] = 0;
	  order->stages[0][7] = 1;
	  order->stages[1][0] = BUILDING;
	  order->stages[1][1] = build.type;
	  order->stages[1][2] = build.mamount;
	  order->stages[1][6] = 0;
	  order->stages[1][7] = 1;
	  order->stages[2][0] = BUILDING;
	  order->stages[2][1] = build.type;
	  order->stages[2][2] = build.buildtime;
	  order->stages[2][6] = 1;
	  order->stages[2][7] = 1;
	  order->stages[2][8] = 3;
	  order->infinite = false;
	  order->curamount = build.mamount;
	  order->type = OBUILD;
	  fortorders.push_back(*order);
	  delete order;
	}
      else
	{
	  fortorders[exists].enabled = true;
	}
      return 5;
    }
  return 6;
}

/**
 * Find proto building by name or type
 * @param name Name of building to find
 * @param type Type of building to find
 * @return Building index in protobuilds
 */
int find_proto_build(std::string name, int type)
{
  int build = -1;
  for (int i = 0; i < (int)protobuilds.size(); i++)
    {
      if ((name.size() == 0) && (type != NONE))
	{
	  if (protobuilds[i].type == type)
	    {
	      build = i;
	      break;
	    }
	}
      if ((name.size() > 0) && (type == NONE))
	{
	  if (protobuilds[i].name.compare(name) == 0)
	    {
	      build = i;
	      break;
	    }
	}
      if ((name.size() > 0) && (type != NONE))
	{
	  if ((protobuilds[i].name.compare(name) == 0) && (protobuilds[i].type == type))
	    {
	      build = i;
	      break;
	    }
	}
    }
  return build;
}

/**
 * Search for selected building.
 * @param name Name of building. Can be empty.
 * @param type Type of building. Default -1.
 * @param constructed Did search only for constructed buildings. Default true.
 * @param x X-axis of point. Default is -1.
 * @param y Y-axis of point. Default is -1.
 * @return Index of building or -1.
 */
int find_build(std::string name, int type, bool constructed, int x, int y)
{
  int build = NONE;
  //Search by position
  if ((x != NONE) && (y != NONE))
    {
      build = fortmap[y][x].building;
      if (build < 0)
	{
	  return NONE;
	}
      if ((name.size() > 0) && (fortbuilds[build].name.compare(name)))
	{
	  return NONE;
	}
      if (fortbuilds[build].constructed != constructed)
	{
	  return NONE;
	}
      if ((type != NONE) && (fortbuilds[build].type != type))
	{
	  return NONE;
	}
      return build;
    }
  for (int i = 0; i < (int)fortbuilds.size(); i++)
    {
      //Search by name
      if ((name.size() > 0) && (fortbuilds[i].name.compare(name)))
	{
	  continue;
	}
      //Search by type
      if ((type != NONE) && (fortbuilds[i].type != type))
	{
	  continue;
	}
      //Search by constructed
      if (fortbuilds[i].constructed != constructed)
	{
	  continue;
	}
      build = i;
      break;
    }
  return build;
}
