/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#ifndef BUILDING_HPP
#define BUILDING_HPP
#include <string>
#include "libtcod.hpp"

/**
 * Contains building types.
 */
typedef enum BuildingTypes
  {
    STOREHOUSE,
    QUARRY,
    COPPERMINE,
    TINMINE,
    COALMINE,
    SMELTER,
    WORKSHOP,
    FLAG,
    ARMORER,
    LABORATORY,
    IRONMINE,
    WEAPONSMITH,
    INN,
    CARPENTER,
    LIBRARY,
    LEATHERWORK,
    ALCHEMIST,
    STOCKADE,
    WALL
  } BUILDINGTYPE;

/**
 * Contains all information about building.
 */
class Building
{
public:
  /**
   * Building name
   */
  std::string name;
  /**
   * Building type
   */
  int type;
  /**
   * Building width
   */
  int width;
  /**
   * Building height
   */
  int height;
  /**
   * Coordinates of building enter
   */
  int enter[2];
  /**
   * Name of material used to construct building
   */
  std::string mname;
  /**
   * Type of material used to construct building
   */
  int material;
  /**
   * Amounf of materials used to construct building
   */
  int mamount;
  /**
   * Time (in seconds) needed to construct building
   */
  int buildtime;
  /**
   * X-axis position of left upper corner of building
   */
  int x;
  /**
   * Y-axis position of left upper corner of building
   */
  int y;
  /**
   * If true, building is constructed (working)
   */
  bool constructed;
  /**
   * Order name added by this building
   */
  std::string workname;
  /**
   * Additional building data (for example: storehouse capacity)
   */
  int data;
  /**
   * If 'Y' building is under construction, otherwise 'N'
   */
  char underconstruction;
  /**
   * Characters used to draw a building
   */
  TCODList<int> chars;
  /**
   * Colors of characters used to draw a building
   */
  TCODList<TCODColor> colors;
  /**
   * Items which are needed on map for construct this building.
   */
  TCODList<int> neededitems;
  /**
   * Did building can be build only once.
   */
  bool unique;
  /**
   * Did building is taken by someone.
   */
  int taken;
  /**
   * Description of building.
   */
  std::string description;

  Building();
};

void buildBuildUI();
void setVisibleBuild(bool visible);
int construct(int i = -1, int keycode = 0);
int find_proto_build(std::string name, int type = -1);
int find_build(std::string name, int type = -1, bool constructed = true, int x = -1, int y = -1);

#endif
