/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "combat.hpp"
#include "main.hpp"
#include <iostream>
#include <stdio.h>
#include "game.hpp"
#include "adventure.hpp"

/**
 * Make attack on mob.
 * @param attacker Attacking mob.
 * @param defender Defending mob.
 */
void make_attack(Mobile *attacker, Mobile *defender)
{
  if ((attacker->x == NONE) || (attacker->y == NONE) || (defender->x == NONE) || (defender->y == NONE) || (attacker->stats[7] == 0) || (defender->stats[7] == 0))
    {
      return;
    }
  int index;
  TCODRandom *generator = TCODRandom::getInstance();
  //Did we hit?
  int hit = generator->getInt(0, 24) + attacker->stats[3];
  int miss = generator->getInt(0, 24) + defender->stats[3];
  if (miss > hit)
    {
      return;
    }
  //Get random body position
  hit = generator->getInt(0, 3);
  //Count damage
  int damageres;
  if (defender->type == ROBOT)
    {
      damageres = defender->stats[8] / 10;
    }
  else
    {
      index = defender->equipment[hit];
      if (index == NONE)
	{
	  damageres = 0;
	}
      else
	{
	  if (state == FORT)
	    {
	      damageres = fortobjs[index].data;
	    }
	  else
	    {
	      damageres = advobjs[index].data;
	    }
	}
    }
  char *c = new char[7];
  sprintf(c, "1d%d", attacker->stats[2]);
  int damage = generator->diceRoll(c) - damageres;
  if (attacker->equipment[4] != NONE)
    {
      index = attacker->equipment[4];
      if (state == FORT)
	{
	  damage += generator->diceRoll(fortobjs[index].damage.c_str());
	}
      else
	{
	  damage += generator->diceRoll(advobjs[index].damage.c_str());
	}
    }
  if (damage < 0)
    {
      damage = 0;
    }
  defender->stats[7] -= damage;
  attacker->stats[0]--;
  if (defender->stats[7] > 0)
    {
      attacker->worktime = 1;
      attacker->gainexp(1);
      return;
    }
  //If player is dead, end game.
  if (defender->type == PLAYER)
    {
      end_game(true);
      return;
    }
  defender->stats[7] = 0;
  update_map_mob(defender->x, defender->y, NONE);
  defender->deletemob();
  int exp = 0;
  for (int i = 2; i < 9; i++)
    {
      exp += defender->stats[i];
    }
  attacker->gainexp(exp);
  if ((attacker->type == PLAYER) || (attacker->type == ROBOT))
    {
      attacker->work = "Ready";
    }
  if (state == ADVENTURE)
    {
      attacker->targettype = NONE;
      attacker->target = NONE;
    }
}

/**
 * Prepare attack, check range and fight orders.
 * @param attacker Attacking mob.
 * @param defender Defending mob.
 */
void prepare_attack(Mobile *attacker, Mobile *defender)
{
  if ((attacker->x == NONE) || (attacker->y == NONE) || (defender->x == NONE) || (defender->y == NONE))
    {
      return;
    }
  int atrange = 1;
  int defrange = 1;
  int index;
  if (defender->equipment[4] != NONE)
    {
      index = defender->equipment[4];
      if (state == FORT)
	{
	  defrange = fortobjs[index].range;
	}
      else
	{
	  defrange = advobjs[index].range;
	}
    }
  if (attacker->equipment[4] != NONE)
    {
      index = attacker->equipment[4];
      if (state == FORT)
	{
	  atrange = fortobjs[index].range;
	}
      else
	{
	  atrange = advobjs[index].range;
	}
    }
  bool attack = false;
  bool defend = false;
  if ((abs(attacker->x - defender->x) <= atrange) && (abs(attacker->y - defender->y) <= atrange))
    {
      attack = true;
      attacker->generatepath(attacker->x, attacker->y, 0);
    }
  if ((abs(attacker->x - defender->x) <= defrange) && (abs(attacker->y - defender->y) <= defrange))
    {
      defend = true;
      defender->generatepath(defender->x, defender->y, 0);
    }
  //Both can attack
  if (attack && defend)
    {
      if (attacker->stats[5] >= defender->stats[5])
	{
	  make_attack(attacker, defender);
	  make_attack(defender, attacker);
	}
      else
	{
	  make_attack(defender, attacker);
	  make_attack(attacker, defender);
	}
    }
  //Attacker can attack
  if (attack && !defend)
    {
      make_attack(attacker, defender);
      if ((defender->x > NONE) || (defender->y > NONE))
	{
	  defender->generatepath(attacker->x, attacker->y, defrange);
	}
    }
  //Defender can attack
  if (!attack && defend)
    {
      make_attack(defender, attacker);
      if ((attacker->x > NONE) || (defender->x > NONE))
	{
	  attacker->generatepath(defender->x, defender->y, atrange);
	}
    }
}

/**
 * Update combat status
 */
void update_combat()
{
  int index;
  if (state == FORT)
    {
      for (int i = 0; i < (int)fortmobs.size(); i++)
	{
	  if ((fortmobs[i].work.compare("Fighting")) || (fortmobs[i].worktime > 0))
	    {
	      continue;
	    }
	  index = fortmobs[i].target;
	  prepare_attack(&fortmobs[i], &fortmobs[index]);
	}
    }
  else
    {
      for (int i = 0; i < (int)advmobs.size(); i++)
	{
	  if ((advmobs[i].worktime > 0) || (advmobs[i].targettype != MOB))
	    {
	      continue;
	    }
	  index = advmobs[i].target;
	  prepare_attack(&advmobs[i], &advmobs[index]);
	}
    }
}
