/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "gui/gui.hpp"
#include "design.hpp"
#include "ui.hpp"
#include "main.hpp"
#include <iostream>
#include <stdio.h>

Container *designmenu = new Container(0, 0, 100, 75);
HBox *stats = new HBox(35, 30, 1);
VBox *parts = new VBox(25, 10, 1);
ToolBar *availparts = new ToolBar(55, 7, "Available parts");
Label *message = new Label(35, 2, "");
Label *lblparts[5];
Label *lblstats[5];
string valparts[5];
int valstats[5];

/**
 * Assign selected part to design.
 * @param w Unused.
 * @param data Name of location and name of robot part.
 */
void partClb(Widget *w, void *data)
{
  string* p = (string*)data;
  size_t pos = p->find(';');
  string loc = p->substr(0, pos);
  pos++;
  string name = p->substr(pos);
  int index = find_proto_obj(name);
  char *c = new char[5];
  if ((loc.compare("head") == 0) && (name.compare(valparts[0])))
    {
      valparts[0] = name;
      lblparts[0]->setValue(name.c_str());
      lblparts[0]->computeSize();
      valstats[0] += protoobjs[index].data;
      sprintf(c, "%d", valstats[0]);
      lblstats[0]->setValue(c);
      lblstats[0]->computeSize();
    }
  else if ((loc.compare("larm") == 0) && (name.compare(valparts[1])))
    {
      valparts[1] = name;
      lblparts[1]->setValue(name.c_str());
      lblparts[1]->computeSize();
      valstats[1] += protoobjs[index].data;
      sprintf(c, "%d", valstats[1]);
      lblstats[1]->setValue(c);
      lblstats[1]->computeSize();
    }
  else if ((loc.compare("rarm") == 0) && (name.compare(valparts[2])))
    {
      valparts[2] = name;
      lblparts[2]->setValue(name.c_str());
      lblparts[2]->computeSize();
      valstats[2] += protoobjs[index].data;
      sprintf(c, "%d", valstats[2]);
      lblstats[2]->setValue(c);
      lblstats[2]->computeSize();
    }
  else if ((loc.compare("torso") == 0) && (name.compare(valparts[3])))
    {
      valparts[3] = name;
      lblparts[3]->setValue(name.c_str());
      lblparts[3]->computeSize();
      valstats[3] += protoobjs[index].data;
      sprintf(c, "%d", valstats[3]);
      lblstats[3]->setValue(c);
      lblstats[3]->computeSize();
    }
  else if ((loc.compare("legs") == 0) && (name.compare(valparts[4])))
    {
      valparts[4] = name;
      lblparts[4]->setValue(name.c_str());
      lblparts[4]->computeSize();
      valstats[4] += protoobjs[index].data;
      sprintf(c, "%d", valstats[4]);
      lblstats[4]->setValue(c);
      lblstats[4]->computeSize();
    }
  parts->computeSize();
  stats->computeSize();
}

/**
 * Show available parts for selected location
 * @param w Unused.
 * @param data Name of location.
 */
void locationClb(Widget *w, void *data)
{
  string* p = (string*)data;
  TCODList<int> aparts;
  TCODList<int> types;
  if (p->compare("head") == 0)
    {
      types.push(HEAD);
    }
  else if ((p->compare("larm") == 0) || (p->compare("rarm") == 0))
    {
      types.push(ARM);
    }
  else if (p->compare("torso") == 0)
    {
      types.push(TORSO);
    }
  else if (p->compare("legs") == 0)
    {
      types.push(LEG);
    }
  aparts = find_all_obj(types);
  availparts->clear();
  if (aparts.size() > 0)
    {
      for (int i = 0; i < aparts.size(); i++)
	{
	  availparts->addWidget(new Button(fortobjs[aparts.get(i)].name.c_str(), "", partClb, new string(*p + ";" + fortobjs[aparts.get(i)].name)));
	}
    }
  else
    {
      availparts->addWidget(new Label(0, 0, "No steambot parts available."));
    }
  availparts->computeSize();
}

/**
 * Clear all design.
 * @param w Unused.
 * @param data Unused.
 */
void clearClb(Widget *w, void *data)
{
  for (int i = 0; i < 5; i++)
    {
      valparts[i] = "";
      valstats[i] = 0;
      lblparts[i]->setValue("");
      lblstats[i]->setValue("0");
    }
  stats->computeSize();
  parts->computeSize();
}

/**
 * Create data for construct selected robot.
 * @param w Unused.
 * @param data Unused.
 */
void robotClb(Widget *w, void *data)
{
  //Check if design is correct
  for (int i = 0; i < 5; i++)
    {
      if (valstats[i] == 0)
	{
	  message->setValue("Incomplete robot design!");
	  message->computeSize();
	  return;
	}
    }
  message->setValue("");
  message->computeSize();
  int index, stage = 0;
  //Add order to build robots.
  Order *order = new Order();
  order->name = "Construct robot";
  order->automatic = true;
  order->type = CRAFT;
  order->infinite = false;
  for (int i = 0; i < 5; i++)
    {
      index = find_proto_obj(valparts[i]);
      order->stages[stage][0] = ITEM;
      order->stages[stage][1] = protoobjs[index].types.get(1);
      order->stages[stage][2] = 1;
      order->stages[stage][6] = 0;
      order->stages[stage][7] = 1;
      order->stages[stage][10] = protoobjs[index].types.get(2);
      if (protoobjs[index].types.size() == 4)
	{
	  order->stages[stage][11] = protoobjs[index].types.get(3);
	}
      stage ++;
      order->stages[stage][0] = BUILDING;
      order->stages[stage][1] = WORKSHOP;
      order->stages[stage][2] = 1;
      order->stages[stage][6] = 0;
      order->stages[stage][7] = 1;
      stage++;
    }
  order->stages[stage][0] = BUILDING;
  order->stages[stage][1] = WORKSHOP;
  order->stages[stage][2] = 1;
  order->stages[stage][3] = MOB;
  order->stages[stage][4] = ROBOT;
  order->stages[stage][5] = 1;
  order->stages[stage][6] = 10;
  order->stages[stage][7] = 7;
  order->stages[stage][8] = 3;
  index = find_order(order->name);
  if (index == NONE)
    {
      fortorders.push_back(*order);
    }
  else
    {
      fortorders[index] = *order;
    }
  delete order;
  index = find_proto_mob("", ROBOT);
  Mobile mob = protomobs[index];
  mob.generatename();
  mob.work = "Under construction";
  mob.x = -2;
  mob.y = -2;
  int rstats[14] =  {200, 200, valstats[1], valstats[2], valstats[0], valstats[4], 1, valstats[3], valstats[3], NONE, NONE, NONE, NONE, NONE};
  for (int i = 0; i < 14; i++)
    {
      mob.stats[i] = rstats[i];
    }
  fortmobs.push_back(mob);
  setVisibleDesign(false);
  state = FORT;
  draw_fort();
  setVisibleGamemenu(true);
}

/**
 * Build design robots user interface.
 */
void buildDesignUI()
{
  designmenu->addWidget(new Button(7, 10, 10, 1, "Head", "", locationClb, new string("head")));
  designmenu->addWidget(new Button(7, 12, 10, 1, "Left arm", "", locationClb, new string("larm")));
  designmenu->addWidget(new Button(7, 14, 10, 1, "Right arm", "", locationClb, new string("rarm")));
  designmenu->addWidget(new Button(7, 16, 10, 1, "Torso", "", locationClb, new string("torso")));
  designmenu->addWidget(new Button(7, 18, 10, 1, "Legs", "", locationClb, new string("legs")));
  designmenu->addWidget(new Button(35, 60, 10, 1, "Construct", "", robotClb, NULL));
  designmenu->addWidget(new Button(55, 60, 10, 1, "Clear", "", clearClb, NULL));
  VBox *statname = new VBox(0, 0, 1);
  statname->addWidget(new Label(0, 0, "Senses:"));
  statname->addWidget(new Label(0, 0, "Strength:"));
  statname->addWidget(new Label(0, 0, "Dexterity:"));
  statname->addWidget(new Label(0, 0, "Hit Points:"));
  statname->addWidget(new Label(0, 0, "Speed:"));
  statname->computeSize();
  stats->addWidget(statname);
  designmenu->addWidget(availparts);
  designmenu->addWidget(message);
  VBox *statval = new VBox(0, 0, 1);
  for (int i = 0; i < 5; i++)
    {
      lblparts[i] = new Label(0, 0, "");
      valparts[i] = "";
      parts->addWidget(lblparts[i]);
      lblstats[i] = new Label(0, 0, "0");
      valstats[i] = 0;     
      statval->addWidget(lblstats[i]);
    }
  designmenu->addWidget(parts);
  stats->addWidget(statval);
  stats->computeSize();
  designmenu->addWidget(stats);
  designmenu->setVisible(false);
}

/**
 * Show/hide design robots menu
 * @param visible If true, show menu, otherwise hide.
 */
void setVisibleDesign(bool visible)
{
  escinfo->setVisible(visible);
  message->setVisible(visible);
  designmenu->setVisible(visible);
}
