/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "gui/gui.hpp"
#include "ui.hpp"
#include "main.hpp"
#include "designation.hpp"
#include <iostream>

Container *marksmenu = new Container(0, 0, 100, 75);
VBox *markmenu = new VBox(60, 2, 1);
int designtype = NONE;
int started[2];

/**
 * Show/hide designation menu
 * @param visible If true, show menu, otherwise hide.
 */
void setVisibleDesignation(bool visible)
{
  markmenu->setVisible(false);
  marksmenu->setVisible(visible);
  escinfo->setVisible(visible);
}

/**
 * Start designating things.
 * @param w Unused.
 * @param data Type of designation (trees, rocks, etc)
 */
void markClb(Widget *w, void* data)
{
  int *p = static_cast<int*>(data);
  state = MARK;
  stage = -2;
  designtype = *p;
  started[0] = NONE;
  started[1] = NONE;
  if (gtime[2] < 3)
    {
      Widget::setForegroundColor(TCODColor::black, TCODColor::red);
    }
  setVisibleDesignation(false);
  markmenu->setVisible(true);
  draw_fort();
}

/**
 * Build designation user interface.
 */
void buildDesignationUI()
{
  marksmenu->addWidget(new Button(37, 20, 10, 1, "Mark trees", "Mark trees to chop.", markClb, new int(TREE)));
  marksmenu->addWidget(new Button(37, 22, 10, 1, "Mark rocks", "Mark rocks for collect stones.", markClb, new int(ROCK)));
  marksmenu->addWidget(new Button(37, 24, 10, 1, "Mark plants", "Mark plants for collection.", markClb, new int(2)));
  marksmenu->addWidget(new Button(37, 26, 10, 1, "Build stockade", "Each stockade cost 1 Wood.", markClb, new int(3)));
  marksmenu->addWidget(new Button(37, 28, 10, 1, "Build wall", "Each wall cost 1 Stone.", markClb, new int(4)));
  marksmenu->addWidget(new Button(30, 30, 20, 1, "Remove designations", "Remove all designations.", markClb, new int(NONE)));
  marksmenu->setVisible(false);
  ToolBar *dmenu = new ToolBar(60, 30, "Info");
  dmenu->addWidget(new Label(0, 0, "First left click start selection."));
  dmenu->addWidget(new Label(0, 0, "Second left click end selection."));
  dmenu->addWidget(new Label(0, 0, "Hit ESC to back to menu."));
  markmenu->addWidget(dmenu);
  markmenu->setVisible(false);
}

/**
 * Designate selected resources.
 * @param mouse Mouse information.
 */
void designation(TCOD_mouse_t *mouse)
{
  if (stage == -2)
    {
      mouse->lbutton_pressed = false;
      stage = designtype;
    }
  int drawx, drawy, width, height;
  if (mouse->cx > started[0])
    {
      drawx= started[0];
    }
  else
    {
      drawx = mouse->cx;
    }
  if (mouse->cy > started[1])
    {
      drawy = started[1];
    }
  else
    {
      drawy = mouse->cy;
    }
  width = abs(started[0] - mouse->cx);
  height = abs(started[1] - mouse->cy);
  if (mouse->lbutton_pressed)
    {
      if (started[0] == NONE)
	{
	  started[0] = mouse->cx;
	  started[1] = mouse->cy;
	}
      else
	{
	  int index, type;
	  for (int y = drawy + screeny + 1; y < drawy + screeny + height; y++)
	    {
	      for (int x = drawx + screenx + 1; x < drawx + screenx + width; x++)
		{
		  //Designate resources
		  if ((designtype > NONE) && (designtype < 3))
		    {
		      if (designtype == 2)
			{
			  type = PLANT;
			}
		      else
			{
			  type = designtype;
			}
		      if (!fortmap[y][x].items.isEmpty())
			{
			  for (int i = 0; i < fortmap[y][x].items.size(); i++)
			    {
			      index = fortmap[y][x].items.get(i);
			      if (fortobjs[index].types.contains(type))
				{
				  fortobjs[index].designated = true;
				  designated[designtype].push(index);
				}
			    }
			}
		    }
		    //Clear designation
		    if (designtype == NONE)
		      {
			if (!fortmap[y][x].items.isEmpty())
			  {
			    for (int i = 0; i < fortmap[y][x].items.size(); i++)
			      {
				index = fortmap[y][x].items.get(i);
				fortobjs[index].designated = false;
				if (fortobjs[index].types.contains(TREE))
				  {
				    designated[TREE].remove(index);
				  }
				else if (fortobjs[index].types.contains(ROCK))
				  {
				    designated[ROCK].remove(index);
				  }
				else if (fortobjs[index].types.contains(PLANT))
				  {
				    designated[2].remove(index);
				  }
			      }
			  }
		      }
		}
	    }
	  if (designated[TREE].size() == 0)
	    {
	      designated[TREE].clear();
	    }
	  else if (designated[ROCK].size() == 0)
	    {
	      designated[ROCK].clear();
	    }
	  else if (designated[2].size() == 0)
	    {
	      designated[2].clear();
	    }
	  state = MARKS;
	  Widget::setForegroundColor(TCODColor::white, TCODColor::red);
	  setVisibleDesignation(true);
	  return;
	}
    }
  if (started[0] != NONE)
    {
      draw_fort();
      offconsole->clear();
      TCODConsole::blit(offconsole, 0, 0, width, height, TCODConsole::root, drawx, drawy, 1.0, 0.5);
    }
}
