/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "main.hpp"
#include "message.hpp"
#include "ui.hpp"
#include "ai.hpp"
#include "game.hpp"
#include "combat.hpp"
#include <iostream>

/**
 * Contains all mobiles in fort (player, robots, etc).
 */
vector<Mobile> fortmobs;
/**
 * Contains all items in fort (resources, materials, etc).
 */
vector<Item> fortobjs;
/**
 * Contains all available orders.
 */
vector<Order> fortorders;
/**
 * Contains all available buildings.
 */
vector<Building> fortbuilds;
/**
 * Contains all available recipes.
 */
vector<Recipe> fortrecipes;
/**
 * Indexes of designated resources (firt element = resource type)
 */
TCODList<int> designated[3];
/**
 * Contains all informations about fort map.
 */
Cell fortmap[MAPHEIGHT][MAPWIDTH];
/**
 * Limits for fort - 0 - trees, 1 - animals, 1 - plants
 */
int limits[3] = {100, 20, 100};

/**
 * Draw fort area.
 */
void draw_fort()
{
  TCODConsole::root->clear();
  if (screenx < 0) 
    {
      screenx = 0;
    }
  else if ((screenx + SCREENWIDTH) >= MAPWIDTH)
    {
      screenx = MAPWIDTH - SCREENWIDTH;
    }
  if (screeny < 0)
    {
      screeny = 0;
    }
  else if ((screeny + SCREENHEIGHT) >= MAPHEIGHT)
    {
      screeny = MAPHEIGHT - SCREENHEIGHT;
    }
  //Draw a screen
  int obj;
  bool visible;
  TCODColor bgcol;
  for (int y = 0; y < SCREENHEIGHT; y++)
    {
      for (int x = 0; x < SCREENWIDTH; x++)
	{
	  //Seasons
	  if (gtime[2] <= 3)
	    {
	      bgcol = fortmap[y + screeny][x + screenx].bgcolor + TCODColor::lightestGrey;
	    }
	  else if ((gtime[2] > 3) && (gtime[2] <= 9))
	    {
	      bgcol = fortmap[y + screeny][x + screenx].bgcolor;
	    }
	  else if (gtime[2] > 9)
	    {
	      if (fortmap[y + screeny][x + screenx].landtype != WATER)
		{
		  bgcol = fortmap[y + screeny][x + screenx].bgcolor + TCODColor::darkerSepia;
		}
	      else
		{
		  bgcol = fortmap[y + screeny][x + screenx].bgcolor;
		}
	    }
	  //Day/night
	  if ((gtime[0] < 6) || (gtime[0] > 21))
	    {
	      bgcol = bgcol * TCODColor::lighterBlue;
	    }
	  TCODConsole::root->putCharEx(x, y, fortmap[y + screeny][x + screenx].ch, fortmap[y + screeny][x + screenx].color, bgcol);
	  if (!fortmap[y + screeny][x + screenx].items.isEmpty())
	    {
	      obj = fortmap[y + screeny][x + screenx].items.peek();
	      if (fortobjs[obj].designated)
		{
		  TCODConsole::root->putCharEx(x, y, fortmap[y + screeny][x + screenx].ch, fortmap[y + screeny][x + screenx].color, TCODColor::yellow);
		}
	    }
	  if (fortmap[y + screeny][x + screenx].mob != NONE)
	    {
	      //Check if mobile is visible
	      obj = fortmap[y + screeny][x + screenx].mob;
	      if ((fortmobs[obj].type == ROBOT) || (fortmobs[obj].type == PLAYER))
		{
		  continue;
		}
	      visible = false;
	      for (int i = 0; i < (int)fortmobs.size(); i++)
		{
		  if ((fortmobs[i].type != ROBOT) && (fortmobs[i].type != PLAYER))
		    {
		      continue;
		    }
		  if ((fortmobs[i].x < 0) || (fortmobs[i].y < 0))
		    {
		      continue;
		    }
		  fovmap->computeFov(fortmobs[i].x, fortmobs[i].y, 50 + fortmobs[i].stats[4]);
		  if (!fovmap->isInFov(x + screenx, y + screeny))
		    {
		      continue;
		    }
		  visible = true;
		  break;
		}
	      if (!visible)
		{
		  TCODConsole::root->putCharEx(x, y, 0, bgcol, bgcol);
		}
	    }
	}
    }
}

/**
 * Update state of all mobiles and items in fort mode.
 */
void update_fort()
{
  //Assign work, movement etc thing to player if needed
  if (fortmobs[0].work.compare("IDLE") == 0)
    {
      for (int i = 1; i < (int)fortmobs.size(); i++)
	{
	  if (fortmobs[i].work.compare("Assigned") == 0)
	    {
	      fortmobs[0].target = i;
	      fortmobs[0].targettype = MOB;
	      fortmobs[0].generatepath(fortmobs[i].x, fortmobs[i].y);
	      fortmobs[0].work = "Manage";
	      fortmobs[0].worktime = 3;
	      break;
	    }
	}
    }
  int index;
  float time = TCODSystem::getLastFrameLength();
  for (int i = 0; i < (int)fortmobs.size(); i++)
    {
      if ((fortmobs[i].x == NONE) || (fortmobs[i].y == NONE))
	{
	  continue;
	}
      //Fall asleep
      if ((fortmobs[i].type != ROBOT) && (fortmobs[i].stats[0] <= 0) && (fortmobs[i].work.compare("Sleep")))
	{
	  fortmobs[i].work = "Sleep";
	  fortmobs[i].worktime = 10;
	  if (i == 0)
	    {
	      addMessage(new Message("You are fall asleep", fortmobs[0].x, fortmobs[0].y));
	    }
	  continue;
	}
      //Sleep or awake
      if (fortmobs[i].work.compare("Sleep") == 0)
	{
	  if (fortmobs[i].worktime >= 0)
	    {
	      fortmobs[i].setworktime(time);
	    }
	  else
	    {
	      fortmobs[i].stats[0] = fortmobs[i].stats[1];
	      fortmobs[i].work = "IDLE";
	    }
	  continue;
	}
      if (fortmobs[i].stats[0] <= 0)
	{
	  continue;
	}
      //Move all mobs
      if (!fortmobs[i].path->isEmpty())
	{
	  fortmobs[i].movestep(time, i);
	}
      else if (fortmobs[i].worktime > 0)
	{
	  //Reduce time of work
	  fortmobs[i].setworktime(time);
	}
      //Update behavior of NPC's
      if ((fortmobs[i].type != PLAYER) && (fortmobs[i].type != ROBOT))
	{
	  set_movement(&fortmobs[i]);
	}
    }
  if ((fortmobs[0].worktime <= 0) && (fortmobs[0].work.compare("Manage") == 0))
    {
      index = fortmobs[0].target;
      fortmobs[index].work = "Ready";
      fortmobs[0].stats[0] -= 3;
      fortmobs[0].targettype = -1;
      fortmobs[0].target = -1;
      fortmobs[0].work = "IDLE";
      fortmobs[0].gainexp(3);
    }
  //Update orders
  update_orders();

  //Resolve combat
  update_combat();

  //Cast spells
  cast_spells();

  //Update time
  secs += time;
  if (secs >= 10)
    {
      secs = 0;
      gtime[0] ++;
      //Advance day
      if (gtime[0] > 23)
	{
	  gtime[1] ++;
	  gtime[0] = 0;
	  TCODList<int> types;
	  types.push(TREE);
	  TCODList<int> tmplist = find_all_obj(types);
	  int result, diff, amount, x, y;
	  //Add more trees
	  if (tmplist.size() < limits[0])
	    {
	      diff = limits[0] - tmplist.size();
	      amount = TCODRandom::getInstance()->getInt(0, diff);
	      tmplist.clear();
	      tmplist = find_all_proto_obj(types);
	      for (int i = 0; i < amount; i++)
		{
		  result = TCODRandom::getInstance()->getInt(0, tmplist.size() - 1);
		  x = TCODRandom::getInstance()->getInt(0, MAPWIDTH - 1);
		  y = TCODRandom::getInstance()->getInt(0, MAPHEIGHT - 1);
		  protoobjs[tmplist.get(result)].setitem(x, y, fortobjs.size() - 1);
		}
	    }
	  //Add more animals
	  tmplist.clear();
	  tmplist = find_all_mobs(ANIMAL);
	  if (tmplist.size() < limits[1])
	    {
	      diff = limits[1] - tmplist.size();
	      amount = TCODRandom::getInstance()->getInt(0, diff);
	      tmplist.clear();
	      tmplist = find_all_proto_mobs(ANIMAL, ADVENTUREONLY);
	      for (int i = 0; i < amount; i++)
		{
		  result = TCODRandom::getInstance()->getInt(0, tmplist.size() - 1);
		  x = TCODRandom::getInstance()->getInt(0, MAPWIDTH - 1);
		  y = TCODRandom::getInstance()->getInt(0, MAPHEIGHT - 1);
		  protomobs[tmplist.get(result)].setmob(x, y);
		}
	    }
	  //Generate new missions
	  generatemissions();
	  //Add more plants
	  tmplist.clear();
	  types.clear();
	  types.push(PLANT);
	  tmplist = find_all_obj(types);
	  if (tmplist.size() < limits[2])
	    {
	      diff = limits[2] - tmplist.size();
	      amount = TCODRandom::getInstance()->getInt(0, diff);
	      tmplist.clear();
	      tmplist = find_all_proto_obj(types);
	      for (int i = 0; i < amount; i++)
		{
		  result = TCODRandom::getInstance()->getInt(0, tmplist.size() - 1);
		  x = TCODRandom::getInstance()->getInt(0, MAPWIDTH - 1);
		  y = TCODRandom::getInstance()->getInt(0, MAPHEIGHT - 1);
		  protoobjs[tmplist.get(result)].setitem(x, y, fortobjs.size() - 1);
		}
	    }
	}
      //Advance month
      if (gtime[1] > 30)
	{
	  gtime[2]++;
	  gtime[1] = 1;
	  if (gtime[2] == 4)
	    {
	      for (int y = 0; y < MAPHEIGHT; y++)
		{
		  for (int x = 0; x < MAPWIDTH; x++)
		    {
		      if (fortmap[y][x].landtype == WATER)
			{
			  fortmap[y][x].walkable = false;
			  fovmap->setProperties(x, y, true, false);
			  if (fortmap[y][x].mob != NONE)
			    {
			      index = fortmap[y][x].mob;
			      if (fortmobs[index].type == PLAYER)
				{
				  end_game(true);
				  return;
				}
			      if (!fortmobs[index].flying)
				{
				  fortmobs[index].deletemob();
				  update_map_mob(x, y, NONE);
				  fortmap[y][x].ch = 0;
				  fortmap[y][x].color = fortmap[y][x].bgcolor;
				}
			    }
			  if (!fortmap[y][x].items.isEmpty())
			    {
			      for (int i = 0; i < fortmap[y][x].items.size(); i++)
				{
				  index = fortmap[y][x].items.get(i);
				  fortobjs[index].deleteitem();
				}
			      fortmap[y][x].ch = 0;
			      fortmap[y][x].color = fortmap[y][x].bgcolor;
			      fortmap[y][x].items.clear();
			    }
			  if (!fortmap[y][x].hitems.isEmpty())
			    {
			      for (int i = 0; i < fortmap[y][x].hitems.size(); i++)
				{
				  index = fortmap[y][x].hitems.get(i);
				  fortobjs[index].deleteitem();
				}
			      fortmap[y][x].hitems.clear();
			    }
			}
		    }
		}
	    }
	  Widget::setForegroundColor(TCODColor::white, TCODColor::red);
	}
      //Advance year
      if (gtime[2] > 12)
	{
	  gtime[3]++;
	  gtime[2] = 1;
	  for (int y = 0; y < MAPHEIGHT; y++)
	    {
	      for (int x = 0; x < MAPWIDTH; x++)
		{
		  if (fortmap[y][x].landtype == WATER)
		    {
		      fortmap[y][x].walkable = true;
		      fovmap->setProperties(x, y, true, true);
		    }
		}
	    }
	  Widget::setForegroundColor(TCODColor::black, TCODColor::red);
	}
      setTime();
      //Heal every living creature
      for (int i = 0; i < (int)fortmobs.size(); i++)
	{
	  if (fortmobs[i].type == ROBOT)
	    {
	      continue;
	    }
	  if (fortmobs[i].stats[7] < fortmobs[i].stats[8])
	    {
	      fortmobs[i].stats[7]++;
	    }
	  if (fortmobs[i].stats[14] < fortmobs[i].stats[15])
	    {
	      fortmobs[i].stats[14]++;
	    }
	}
    }

  draw_fort();
}
