/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "libtcod.hpp"
#include "utils.hpp"
#include "main.hpp"
#include "item.hpp"
#include "mobile.hpp"
#include "order.hpp"
#include "building.hpp"
#include "parser.hpp"
#include "recipe.hpp"
#include <iostream>
#include "ui.hpp"
#include "adventure.hpp"
#include "game.hpp"

string saveversion = "0.5";

/**
 * Load saved game
 */
bool load_game()
{
  if (!save_exists())
    {
      return false;
    }
  TCODZip *savegame = new TCODZip();
  savegame->loadFromFile("data/savegame.dat");
  //Load save game version
  string version = savegame->getString();
  if (version.compare(saveversion))
    {
      delete savegame;
      return false;
    }
  //Load game state
  state = (GameState)savegame->getInt();
  if (state == ADVENTURE)
    {
      dungeonlevel = savegame->getInt();
    }
  //Load game time
  for (int i = 0; i < 4; i++)
    {
      gtime[i] = savegame->getInt();
    }
  //Load fort and fov map
  int amount;
  for (int y = 0; y < MAPHEIGHT; y++)
    {
      for (int x = 0; x < MAPWIDTH; x++)
	{
	  fortmap[y][x].ch = savegame->getInt();
	  fortmap[y][x].color = savegame->getColor();
	  fortmap[y][x].bgcolor = savegame->getColor();
	  fortmap[y][x].walkable = savegame->getInt();
	  fortmap[y][x].transparent = savegame->getInt();
	  fortmap[y][x].mob = savegame->getInt();
	  amount = savegame->getInt();
	  for (int i = 0; i < amount; i ++)
	    {
	      fortmap[y][x].items.push(savegame->getInt());
	    }
	  amount = savegame->getInt();
	  for (int i = 0; i < amount; i ++)
	    {
	      fortmap[y][x].hitems.push(savegame->getInt());
	    }
	  fortmap[y][x].building = savegame->getInt();
	  fortmap[y][x].landtype = savegame->getInt();
	  fortmap[y][x].height = savegame->getFloat();
	  fortmap[y][x].visited = savegame->getInt();
	  if (state == FORT)
	    {
	      fovmap->setProperties(x, y, fortmap[y][x].transparent, fortmap[y][x].walkable);
	    }
	}
    }
  //Load adventure map
  if (state == ADVENTURE)
    {
      for (int y = 0; y < MAPHEIGHT; y++)
	{
	  for (int x = 0; x < MAPWIDTH; x++)
	    {
	      advmap[y][x].ch = savegame->getInt();
	      advmap[y][x].color = savegame->getColor();
	      advmap[y][x].bgcolor = savegame->getColor();
	      advmap[y][x].walkable = savegame->getInt();
	      advmap[y][x].transparent = savegame->getInt();
	      advmap[y][x].mob = savegame->getInt();
	      amount = savegame->getInt();
	      for (int i = 0; i < amount; i ++)
		{
		  advmap[y][x].items.push(savegame->getInt());
		}
	      amount = savegame->getInt();
	      for (int i = 0; i < amount; i ++)
		{
		  advmap[y][x].hitems.push(savegame->getInt());
		}
	      advmap[y][x].building = savegame->getInt();
	      advmap[y][x].landtype = savegame->getInt();
	      advmap[y][x].height = savegame->getFloat();
	      advmap[y][x].visited = savegame->getInt();
	      fovmap->setProperties(x, y, advmap[y][x].transparent, advmap[y][x].walkable);
	    }
	}
    }
  //Load objects (in fort)
  amount = savegame->getInt();
  int amount2;
  Item *item = new Item();
  for (int i = 0; i < amount; i++)
    {
      item->name = savegame->getString();
      item->types.clear();
      amount2 = savegame->getInt();
      for (int j = 0; j < amount2; j++)
	{
	  item->types.push(savegame->getInt());
	}
      item->x = savegame->getInt();
      item->y = savegame->getInt();
      item->ch = savegame->getInt();
      item->color = savegame->getColor();
      item->walkable = savegame->getInt();
      item->transparent = savegame->getInt();
      item->taken = savegame->getInt();
      item->stored = savegame->getInt();
      item->hidden = savegame->getInt();
      item->description = savegame->getString();
      item->designated = savegame->getInt();
      item->data = savegame->getInt();
      item->damage = savegame->getString();
      item->range = savegame->getInt();
      item->use = savegame->getInt();
      fortobjs.push_back(*item);
    }
  //Load objects (in adventure)
  if (state == ADVENTURE)
    {
      amount = savegame->getInt();
      for (int i = 0; i < amount; i++)
	{
	  item->name = savegame->getString();
	  item->types.clear();
	  amount2 = savegame->getInt();
	  for (int j = 0; j < amount2; j++)
	    {
	      item->types.push(savegame->getInt());
	    }
	  item->x = savegame->getInt();
	  item->y = savegame->getInt();
	  item->ch = savegame->getInt();
	  item->color = savegame->getColor();
	  item->walkable = savegame->getInt();
	  item->transparent = savegame->getInt();
	  item->taken = savegame->getInt();
	  item->stored = savegame->getInt();
	  item->hidden = savegame->getInt();
	  item->description = savegame->getString();
	  item->designated = savegame->getInt();
	  item->data = savegame->getInt();
	  item->damage = savegame->getString();
	  item->range = savegame->getInt();
	  item->use = savegame->getInt();
	  advobjs.push_back(*item);
	}
    }
  delete item;
  //Load buildings
  amount = savegame->getInt();
  Building *build = new Building();
  for (int i = 0; i < amount; i++)
    {
      build->name = savegame->getString();
      build->type = savegame->getInt();
      build->width = savegame->getInt();
      build->height = savegame->getInt();
      build->enter[0] = savegame->getInt();
      build->enter[1] = savegame->getInt();
      build->mname = savegame->getString();
      build->material = savegame->getInt();
      build->mamount = savegame->getInt();
      build->buildtime = savegame->getInt();
      build->x = savegame->getInt();
      build->y = savegame->getInt();
      build->constructed = savegame->getInt();
      build->workname = savegame->getString();
      build->data = savegame->getInt();
      build->underconstruction = savegame->getInt();
      build->chars.clear();
      build->colors.clear();
      build->neededitems.clear();
      amount2 = savegame->getInt();
      for (int j = 0; j < amount2; j++)
	{
	  build->chars.push(savegame->getInt());
	}
      amount2 = savegame->getInt();
      for (int j = 0; j < amount2; j++)
	{
	  build->colors.push(savegame->getColor());
	}
      amount2 = savegame->getInt();
      for (int j = 0; j < amount2; j++)
	{
	  build->neededitems.push(savegame->getInt());
	}
      build->taken = savegame->getInt();
      build->description = savegame->getString();
      fortbuilds.push_back(*build);
    }
  delete build;
  //Load mobs (in fort)
  amount = savegame->getInt();
  Mobile *mob = new Mobile();
  for (int i = 0; i < amount; i++)
    {
      mob->name = savegame->getString();
      mob->type = savegame->getInt();
      mob->x = savegame->getInt();
      mob->y = savegame->getInt();
      mob->ch = savegame->getChar();
      mob->color = savegame->getColor();
      for (int j = 0; j < 16; j++)
	{
	  mob->stats[j] = savegame->getInt();
	}
      mob->work = savegame->getString();
      mob->targettype = savegame->getInt();
      mob->target = savegame->getInt();
      mob->worktime = savegame->getFloat();
      mob->carries = savegame->getInt();
      mob->taken = savegame->getInt();
      mob->workstage = savegame->getInt();
      mob->description = savegame->getString();
      if ((mob->x != NONE) || (mob->y != NONE))
	{
	  mob->path = new TCODPath(fovmap);
      	}
      for (int j = 0; j < 5; j++)
	{
	  mob->equipment[j] = savegame->getInt();
	}
      for (int j = 0; j < 3; j++)
	{
	  mob->ais[j] = savegame->getInt();
	}
      mob->walktime = savegame->getFloat();
      amount2 = savegame->getInt();
      if (amount2 > 0)
	{
	  for (int j = 0; j < amount2; j++)
	    {
	      mob->inventory.push(savegame->getInt());
	    }
	}
      amount2 = savegame->getInt();
      if (amount2 > 0)
	{
	  for (int j = 0; j < amount2; j++)
	    {
	      mob->loot.push(savegame->getInt());
	    }
	}
      mob->flying = savegame->getInt();
      amount2 = savegame->getInt();
      if (amount2 > 0)
	{
	  for (int j = 0; j < amount2; j++)
	    {
	      mob->spells.push(savegame->getInt());
	    }
	}
      fortmobs.push_back(*mob);
    }
  //Regenerate path for mobiles
  int target;
  for (int i = 0; i < (int)fortmobs.size(); i++)
    {
      if (fortmobs[i].targettype != NONE)
	{
	  target = fortmobs[i].target;
	}
      else
	{
	  continue;
	}
      switch (fortmobs[i].targettype)
	{
	case ITEM:
	  fortmobs[i].generatepath(fortobjs[target].x, fortobjs[target].y);
	  break;
	case MOB:
	  fortmobs[i].generatepath(fortmobs[target].x, fortmobs[target].y);
	  break;
	case BUILDING:
	  fortmobs[i].generatepath(fortbuilds[target].x + fortbuilds[target].enter[0], fortbuilds[target].y + fortbuilds[target].enter[1]);
	  break;
	default:
	  break;
	}
    }
  //Load mobs (in adventure)
  if (state == ADVENTURE)
    {
      amount = savegame->getInt();
      for (int i = 0; i < amount; i++)
	{
	  mob->name = savegame->getString();
	  mob->type = savegame->getInt();
	  mob->x = savegame->getInt();
	  mob->y = savegame->getInt();
	  mob->ch = savegame->getChar();
	  mob->color = savegame->getColor();
	  for (int j = 0; j < 16; j++)
	    {
	      mob->stats[j] = savegame->getInt();
	    }
	  mob->work = savegame->getString();
	  mob->targettype = savegame->getInt();
	  mob->target = savegame->getInt();
	  mob->worktime = savegame->getFloat();
	  mob->carries = savegame->getInt();
	  mob->taken = savegame->getInt();
	  mob->workstage = savegame->getInt();
	  mob->description = savegame->getString();
	  mob->path = new TCODPath(fovmap);
	  for (int j = 0; j < 5; j++)
	    {
	      mob->equipment[j] = savegame->getInt();
	    }
	  for (int j = 0; j < 3; j++)
	    {
	      mob->ais[j] = savegame->getInt();
	    }
	  mob->walktime = savegame->getFloat();
	  amount2 = savegame->getInt();
	  if (amount2 > 0)
	    {
	      for (int j = 0; j < amount2; j++)
		{
		  mob->inventory.push(savegame->getInt());
		}
	    }
	  amount2 = savegame->getInt();
	  if (amount2 > 0)
	    {
	      for (int j = 0; j < amount2; j++)
		{
		  mob->loot.push(savegame->getInt());
		}
	    }
	  mob->flying = savegame->getInt();
	  amount2 = savegame->getInt();
	  if (amount2 > 0)
	    {
	      for (int j = 0; j < amount2; j++)
		{
		  mob->spells.push(savegame->getInt());
		}
	    }
	  advmobs.push_back(*mob);
	}
      //Regenerate mobs path
      for (int i = 0; i < (int)advmobs.size(); i++)
	{
	  if (advmobs[i].targettype != NONE)
	    {
	      target = advmobs[i].target;
	    }
	  else
	    {
	      continue;
	    }
	  switch (advmobs[i].targettype)
	    {
	    case ITEM:
	      advmobs[i].generatepath(advobjs[target].x, advobjs[target].y);
	      break;
	    case MOB:
	      advmobs[i].generatepath(advmobs[target].x, advmobs[target].y);
	      break;
	    default:
	      break;
	    }
	}
    }
  delete mob;
  //Load orders
  amount = savegame->getInt();
  Order *order = new Order();
  for (int i = 0; i < amount; i++)
    {
      order->name = savegame->getString();
      order->infinite = savegame->getInt();
      order->active = savegame->getChar();
      order->workers.clear();
      amount2 = savegame->getInt();
      for (int j = 0; j < amount2; j++)
	{
	  order->workers.push_back(savegame->getInt());
	}
      for (int j = 0; j < STEPS; j++)
	{
	  for (int k = 0; k < STEPLENGTH; k++)
	    {
	      order->stages[j][k] = savegame->getInt();
	    }
	}
      order->curamount = savegame->getInt();
      order->automatic = savegame->getInt();
      order->enabled = savegame->getInt();
      order->type = savegame->getInt();
      order->playeronly = savegame->getInt();
      order->times = savegame->getInt();
      fortorders.push_back(*order);
    }
  delete order;
  //Load recipes
  amount = savegame->getInt();
  Recipe *recipe = new Recipe();
  for (int i = 0; i < amount; i++)
    {
      recipe->name = savegame->getString();
      for (int j = 0; j < RESOURCES; j++)
	{
	  for (int k = 0; k < 3; k++)
	    {
	      recipe->resources[j][k] = savegame->getInt();
	    }
	}
      recipe->fuel = savegame->getInt();
      recipe->amount = savegame->getInt();
      recipe->worktime = savegame->getInt();
      recipe->building = savegame->getInt();
      recipe->known = savegame->getInt();
      recipe->unlocktype[0] = savegame->getInt();
      recipe->unlocktype[1] = savegame->getInt();
      recipe->unlockname = savegame->getString();
      recipe->materials = savegame->getString();
      fortrecipes.push_back(*recipe);
    }
  delete recipe;
  //Load designations
  for (int i = 0; i < 3; i++)
    {
      amount = savegame->getInt();
      if (amount > 0)
	{
	  for (int j = 0; j < amount; j++)
	    {
	      designated[i].push(savegame->getInt());
	    }
	}
    }
  
  loadproto();
  TCODConsole::root->clear();
  loadorders();

  if (state == FORT)
    {
      if (gtime[2] < 3)
	{
	  Widget::setForegroundColor(TCODColor::black, TCODColor::red);
	}
      draw_fort();
      setTime();
    }
  else
    {
      mission = protomissions[savegame->getInt()];
      draw_adventure();
    }

  delete savegame;

  return true;
}

/**
 * Start new game, generate fort area and mobs.
 */
void new_game()
{
  TCODConsole::root->clear();
  TCODConsole::root->setDefaultBackground(TCODColor::black);

  loadproto();

  TCODRandom *generator = TCODRandom::getInstance();
  int result, x, y, result2;
  //Get all resource type objects
  vector<Item> trees;
  for (vector<Item>::iterator it = protoobjs.begin(); it != protoobjs.end(); it++)
    {
      if ((*it).types.contains(TREE))
	{
	  trees.push_back(*it);
	}
    }
  vector<Item> rocks;
  for (vector<Item>::iterator it = protoobjs.begin(); it != protoobjs.end(); it++)
    {
      if ((*it).types.contains(ROCK))
	{
	  rocks.push_back(*it);
	}
    }
  vector<Item> veins;
  for (vector<Item>::iterator it = protoobjs.begin(); it != protoobjs.end(); it++)
    {
      if ((*it).types.contains(VEIN))
	{
	  veins.push_back(*it);
	}
    }
  vector<Item> plants;
  for (vector<Item>::iterator it = protoobjs.begin(); it != protoobjs.end(); it++)
    {
      if ((*it).types.contains(PLANT))
	{
	  plants.push_back(*it);
	}
    }
  //Generate map
  float nval;
  uint8 val;
  int minwalkable = (MAPWIDTH * MAPHEIGHT) / 10;
  int walkable;
  do
    {
      walkable = 0;
      TCODNoise *noise = new TCODNoise(2);
      TCODHeightMap *hm = new TCODHeightMap(MAPWIDTH, MAPHEIGHT);
      hm->addFbm(noise,1,1,0,0,6,0.5,1);
      hm->scaleFbm(noise,1,1,100,0,6,0.5,1);
      hm->normalize(-1,1);
      for (y = 0; y < MAPHEIGHT; y++)
	{
	  for (x = 0; x < MAPWIDTH; x++)
	    {
	      //Get cell background
	      nval = hm->getValue(x, y);
	      if (nval > 0.7)
		{
		  nval = 0.7;
		}
	      fortmap[y][x].ch = 0;
	      fortmap[y][x].transparent = true;
	      fortmap[y][x].walkable = true;
	      fortmap[y][x].mob = NONE;
	      fortmap[y][x].building = NONE;
	      fortmap[y][x].items.clear();
	      fortmap[y][x].hitems.clear();
	      fortmap[y][x].visited = true;
	      fortmap[y][x].height = nval;
	      fovmap->setProperties(x, y, true, true);
	      if (nval > -0.69)
		{
		  if (nval < 0)
		    {
		      nval = 0;
		    }
		  val = (uint8)((nval + 1.0f) / 2.0f * 255);
		  TCODColor color((uint8)val,val,val);
		  fortmap[y][x].color = TCODColor::chartreuse * color;
		  fortmap[y][x].bgcolor = TCODColor::chartreuse * color;
		  fortmap[y][x].landtype = EARTH;
		}
	      else if ((nval <= -0.69) && (nval >= -0.7))
		{
		  fortmap[y][x].color = TCODColor::darkYellow;
		  fortmap[y][x].bgcolor = TCODColor::darkYellow;
		  fortmap[y][x].landtype = LSAND;
		}
	      else if (nval < -0.7)
		{
		  val = (uint8)((nval + 1.0f) / 2.0f * 255);
		  TCODColor color((uint8)val,val,val);
		  fortmap[y][x].color = TCODColor::blue - color;
		  fortmap[y][x].bgcolor = TCODColor::blue - color;
		  fortmap[y][x].walkable = false;
		  fortmap[y][x].landtype = WATER;
		  fovmap->setProperties(x, y, true, false);
		  continue;
		}
	      result = generator->getInt(0, 99);
	      switch (result)
		{
		case 91:
		case 92:
		case 93:
		  result2 = generator->getInt(0, plants.size() - 1);
		  plants[result2].setitem(x, y, NONE, true);
		  break;
		case 94:
		  result2 = generator->getInt(0, veins.size() - 1);
		  veins[result2].setitem(x, y, NONE, true);
		  break;
		case 95:
		case 96:
		case 97:
		  result2 = generator->getInt(0, trees.size() - 1);
		  trees[result2].setitem(x, y, NONE, true);
		  break;
		case 98:
		case 99:
		  result2 = generator->getInt(0, rocks.size() - 1);
		  rocks[result2].setitem(x, y, NONE, true);
		  break;
		default:
		  break;
		}
	      if (fortmap[y][x].walkable)
		{
		  walkable++;
		}
	    }
	}
      delete hm;
      delete noise;
    } while (walkable < minwalkable);

  //Add player
  protomobs[0].setmob(MAPWIDTH / 2, MAPHEIGHT / 2, NONE, true);
  fortmobs[0].generatestats();
  fortmobs[0].generatename();

  //Add robots
  result = find_proto_mob("", ROBOT);
  for (int i = 0; i < 4; i++)
    {
      protomobs[result].generatename();
      protomobs[result].setmob(MAPWIDTH / 2, MAPHEIGHT / 2, NONE, true);
    }

  //Add some animals
  TCODList<int> npcs = find_all_proto_mobs(ANIMAL, ADVENTUREONLY);
  int amount = generator->getInt(0, 20);
  for (int i = 0; i < amount; i++)
    {
      if (npcs.size() > 1)
	{
	  result = generator->getInt(0, npcs.size() - 1);
	}
      else
	{
	  result = 0;
	}
      x = generator->getInt(0, MAPWIDTH - 1);
      y = generator->getInt(0, MAPHEIGHT - 1);
      protomobs[npcs.get(result)].setmob(x, y, NONE, true);
      fortmobs[fortmobs.size() - 1].generatestats();
    }

  loadorders();

  generatemissions();

  draw_fort();
}

/**
 * Clear data from old game.
 * @param deletesave If true, delete savegame.
 */
void end_game(bool deletesave)
{
  if (save_exists() && deletesave)
    {
      TCODSystem::deleteFile("data/savegame.dat");
    }
  gtime[0] = 8;
  gtime[1] = 1;
  gtime[2] = 4;
  gtime[3] = 400;
  for (int y = 0; y < MAPHEIGHT; y++)
    {
      for (int x = 0; x < MAPWIDTH; x++)
	{
	  fortmap[y][x].items.clear();
	  fortmap[y][x].hitems.clear();
	}
    }
  fortobjs.clear();
  fortmobs.clear();
  fortorders.clear();
  fortbuilds.clear();
  fortrecipes.clear();
  advmobs.clear();
  advobjs.clear();
  for (int i = 0; i < 2; i++)
    {
      designated[i].clear();
    }
  state = MAINMENU;
  mainmenu->setVisible(true);
  credits->setVisible(true);
  setVisibleGamemenu(false);
  TCODConsole::root->clear();
}

/**
 * Save and clear all data from old game
 */
void save_game()
{
  if (save_exists())
    {
      TCODSystem::deleteFile("data/savegame.dat");
    }
  TCODZip *savegame = new TCODZip();
  //Enter game version
  savegame->putString(saveversion.c_str());
  //Save game state
  savegame->putInt(state);
  if (state == ADVENTURE)
    {
      savegame->putInt(dungeonlevel);
    }
  //Save game time
  for (int i = 0; i < 4; i++)
    {
      savegame->putInt(gtime[i]);
    }
  //Save fort map
  for (int y = 0; y < MAPHEIGHT; y++)
    {
      for (int x = 0; x < MAPWIDTH; x++)
	{
	  savegame->putInt(fortmap[y][x].ch);
	  savegame->putColor(&fortmap[y][x].color);
	  savegame->putColor(&fortmap[y][x].bgcolor);
	  savegame->putInt(fortmap[y][x].walkable);
	  savegame->putInt(fortmap[y][x].transparent);
	  savegame->putInt(fortmap[y][x].mob);
	  savegame->putInt(fortmap[y][x].items.size());
	  for (int i = 0; i < fortmap[y][x].items.size(); i++)
	    {
	      savegame->putInt(fortmap[y][x].items.get(i));
	    }
	  savegame->putInt(fortmap[y][x].hitems.size());
	  for (int i = 0; i < fortmap[y][x].hitems.size(); i++)
	    {
	      savegame->putInt(fortmap[y][x].hitems.get(i));
	    }
	  savegame->putInt(fortmap[y][x].building);
	  savegame->putInt(fortmap[y][x].landtype);
	  savegame->putFloat(fortmap[y][x].height);
	  savegame->putInt(fortmap[y][x].visited);
	}
    }
  //Save adventure map
  if (state == ADVENTURE)
    {
      for (int y = 0; y < MAPHEIGHT; y++)
	{
	  for (int x = 0; x < MAPWIDTH; x++)
	    {
	      savegame->putInt(advmap[y][x].ch);
	      savegame->putColor(&advmap[y][x].color);
	      savegame->putColor(&advmap[y][x].bgcolor);
	      savegame->putInt(advmap[y][x].walkable);
	      savegame->putInt(advmap[y][x].transparent);
	      savegame->putInt(advmap[y][x].mob);
	      savegame->putInt(advmap[y][x].items.size());
	      for (int i = 0; i < advmap[y][x].items.size(); i++)
		{
		  savegame->putInt(advmap[y][x].items.get(i));
		}
	      savegame->putInt(advmap[y][x].hitems.size());
	      for (int i = 0; i < advmap[y][x].hitems.size(); i++)
		{
		  savegame->putInt(advmap[y][x].hitems.get(i));
		}
	      savegame->putInt(advmap[y][x].building);
	      savegame->putInt(advmap[y][x].landtype);
	      savegame->putFloat(advmap[y][x].height);
	      savegame->putInt(advmap[y][x].visited);
	    }
	}
    }
  //Save objects (in fort)
  savegame->putInt(fortobjs.size());
  for (int i = 0; i < (int)fortobjs.size(); i++)
    {
      savegame->putString(fortobjs[i].name.c_str());
      savegame->putInt(fortobjs[i].types.size());
      for (int j = 0; j < fortobjs[i].types.size(); j++)
	{
	  savegame->putInt(fortobjs[i].types.get(j));
	}
      savegame->putInt(fortobjs[i].x);
      savegame->putInt(fortobjs[i].y);
      savegame->putInt(fortobjs[i].ch);
      savegame->putColor(&fortobjs[i].color);
      savegame->putInt(fortobjs[i].walkable);
      savegame->putInt(fortobjs[i].transparent);
      savegame->putInt(fortobjs[i].taken);
      savegame->putInt(fortobjs[i].stored);
      savegame->putInt(fortobjs[i].hidden);
      savegame->putString(fortobjs[i].description.c_str());
      savegame->putInt(fortobjs[i].designated);
      savegame->putInt(fortobjs[i].data);
      savegame->putString(fortobjs[i].damage.c_str());
      savegame->putInt(fortobjs[i].range);
      savegame->putInt(fortobjs[i].use);
    }
  //Save objects (in adventure)
  if (state == ADVENTURE)
    {
      savegame->putInt(advobjs.size());
      for (int i = 0; i < (int)advobjs.size(); i++)
	{
	  savegame->putString(advobjs[i].name.c_str());
	  savegame->putInt(advobjs[i].types.size());
	  for (int j = 0; j < advobjs[i].types.size(); j++)
	    {
	      savegame->putInt(advobjs[i].types.get(j));
	    }
	  savegame->putInt(advobjs[i].x);
	  savegame->putInt(advobjs[i].y);
	  savegame->putInt(advobjs[i].ch);
	  savegame->putColor(&advobjs[i].color);
	  savegame->putInt(advobjs[i].walkable);
	  savegame->putInt(advobjs[i].transparent);
	  savegame->putInt(advobjs[i].taken);
	  savegame->putInt(advobjs[i].stored);
	  savegame->putInt(advobjs[i].hidden);
	  savegame->putString(advobjs[i].description.c_str());
	  savegame->putInt(advobjs[i].designated);
	  savegame->putInt(advobjs[i].data);
	  savegame->putString(advobjs[i].damage.c_str());
	  savegame->putInt(advobjs[i].range);
	  savegame->putInt(advobjs[i].use);
	}
    }
  //Save buildings
  savegame->putInt(fortbuilds.size());
  for (int i = 0; i < (int)fortbuilds.size(); i++)
    {
      savegame->putString(fortbuilds[i].name.c_str());
      savegame->putInt(fortbuilds[i].type);
      savegame->putInt(fortbuilds[i].width);
      savegame->putInt(fortbuilds[i].height);
      savegame->putInt(fortbuilds[i].enter[0]);
      savegame->putInt(fortbuilds[i].enter[1]);
      savegame->putString(fortbuilds[i].mname.c_str());
      savegame->putInt(fortbuilds[i].material);
      savegame->putInt(fortbuilds[i].mamount);
      savegame->putInt(fortbuilds[i].buildtime);
      savegame->putInt(fortbuilds[i].x);
      savegame->putInt(fortbuilds[i].y);
      savegame->putInt(fortbuilds[i].constructed);
      savegame->putString(fortbuilds[i].workname.c_str());
      savegame->putInt(fortbuilds[i].data);
      savegame->putInt(fortbuilds[i].underconstruction);
      savegame->putInt(fortbuilds[i].chars.size());
      for (int j = 0; j < fortbuilds[i].chars.size(); j++)
	{
	  savegame->putInt(fortbuilds[i].chars.get(j));
	}
      savegame->putInt(fortbuilds[i].colors.size());
      for (int j = 0; j < fortbuilds[i].colors.size(); j++)
	{
	  savegame->putColor(new TCODColor(fortbuilds[i].colors.get(j)));
	}
      savegame->putInt(fortbuilds[i].neededitems.size());
      for (int j = 0; j < fortbuilds[i].neededitems.size(); j++)
	{
	  savegame->putInt(fortbuilds[i].neededitems.get(j));
	}
      savegame->putInt(fortbuilds[i].taken);
      savegame->putString(fortbuilds[i].description.c_str());
    }
  //Save mobs (in fort)
  savegame->putInt(fortmobs.size());
  for (int i = 0; i < (int)fortmobs.size(); i++)
    {
      savegame->putString(fortmobs[i].name.c_str());
      savegame->putInt(fortmobs[i].type);
      savegame->putInt(fortmobs[i].x);
      savegame->putInt(fortmobs[i].y);
      savegame->putChar(fortmobs[i].ch);
      savegame->putColor(&fortmobs[i].color);
      for (int j = 0; j < 16; j++)
	{
	  savegame->putInt(fortmobs[i].stats[j]);
	}
      savegame->putString(fortmobs[i].work.c_str());
      savegame->putInt(fortmobs[i].targettype);
      savegame->putInt(fortmobs[i].target);
      savegame->putFloat(fortmobs[i].worktime);
      savegame->putInt(fortmobs[i].carries);
      savegame->putInt(fortmobs[i].taken);
      savegame->putInt(fortmobs[i].workstage);
      savegame->putString(fortmobs[i].description.c_str());
      for (int j = 0; j < 5; j++)
	{
	  savegame->putInt(fortmobs[i].equipment[j]);
	}
      for (int j = 0; j < 3; j++)
	{
	  savegame->putInt(fortmobs[i].ais[j]);
	}
      savegame->putFloat(fortmobs[i].worktime);
      savegame->putInt(fortmobs[i].inventory.size());
      if (!fortmobs[i].inventory.isEmpty())
	{
	  for (int j = 0; j < fortmobs[i].inventory.size(); j++)
	    {
	      savegame->putInt(fortmobs[i].inventory.get(j));
	    }
	}
      savegame->putInt(fortmobs[i].loot.size());
      if (!fortmobs[i].loot.isEmpty())
	{
	  for (int j = 0; j < fortmobs[i].loot.size(); j++)
	    {
	      savegame->putInt(fortmobs[i].loot.get(j));
	    }
	}
      savegame->putInt(fortmobs[i].flying);
      savegame->putInt(fortmobs[i].spells.size());
      if (!fortmobs[i].spells.isEmpty())
	{
	  for (int j = 0; j < fortmobs[i].spells.size(); j++)
	    {
	      savegame->putInt(fortmobs[i].spells.get(j));
	    }
	}
    }
  //Save mobs (in adventure)
  if (state == ADVENTURE)
    {
      savegame->putInt(advmobs.size());
      for (int i = 0; i < (int)advmobs.size(); i++)
	{
	  savegame->putString(advmobs[i].name.c_str());
	  savegame->putInt(advmobs[i].type);
	  savegame->putInt(advmobs[i].x);
	  savegame->putInt(advmobs[i].y);
	  savegame->putChar(advmobs[i].ch);
	  savegame->putColor(&advmobs[i].color);
	  for (int j = 0; j < 16; j++)
	    {
	      savegame->putInt(advmobs[i].stats[j]);
	    }
	  savegame->putString(advmobs[i].work.c_str());
	  savegame->putInt(advmobs[i].targettype);
	  savegame->putInt(advmobs[i].target);
	  savegame->putFloat(advmobs[i].worktime);
	  savegame->putInt(advmobs[i].carries);
	  savegame->putInt(advmobs[i].taken);
	  savegame->putInt(advmobs[i].workstage);
	  savegame->putString(advmobs[i].description.c_str());
	  for (int j = 0; j < 5; j++)
	    {
	      savegame->putInt(advmobs[i].equipment[j]);
	    }
	  for (int j = 0; j < 3; j++)
	    {
	      savegame->putInt(advmobs[i].ais[j]);
	    }
	  savegame->putFloat(advmobs[i].worktime);
	  savegame->putInt(advmobs[i].inventory.size());
	  if (!advmobs[i].inventory.isEmpty())
	    {
	      for (int j = 0; j < advmobs[i].inventory.size(); j++)
		{
		  savegame->putInt(advmobs[i].inventory.get(j));
		}
	    }
	  savegame->putInt(advmobs[i].loot.size());
	  if (!advmobs[i].loot.isEmpty())
	    {
	      for (int j = 0; j < advmobs[i].loot.size(); j++)
		{
		  savegame->putInt(advmobs[i].loot.get(j));
		}
	    }
	  savegame->putInt(advmobs[i].flying);
	  savegame->putInt(advmobs[i].spells.size());
	  if (!advmobs[i].spells.isEmpty())
	    {
	      for (int j = 0; j < advmobs[i].spells.size(); j++)
		{
		  savegame->putInt(advmobs[i].spells.get(j));
		}
	    }
	}
    }
  //Save orders
  savegame->putInt(fortorders.size());
  for (int i = 0; i < (int)fortorders.size(); i++)
    {
      savegame->putString(fortorders[i].name.c_str());
      savegame->putInt(fortorders[i].infinite);
      savegame->putChar(fortorders[i].active);
      savegame->putInt(fortorders[i].workers.size());
      for (int j = 0; j < (int)fortorders[i].workers.size(); j++)
	{
	  savegame->putInt(fortorders[i].workers[j]);
	}
      for (int j = 0; j < STEPS; j++)
	{
	  for (int k = 0; k < STEPLENGTH; k++)
	    {
	      savegame->putInt(fortorders[i].stages[j][k]);
	    }
	}
      savegame->putInt(fortorders[i].curamount);
      savegame->putInt(fortorders[i].automatic);
      savegame->putInt(fortorders[i].enabled);
      savegame->putInt(fortorders[i].type);
      savegame->putInt(fortorders[i].playeronly);
      savegame->putInt(fortorders[i].times);
    }
  //Save recipes
  savegame->putInt(fortrecipes.size());
  for (int i = 0; i < (int)fortrecipes.size(); i++)
    {
      savegame->putString(fortrecipes[i].name.c_str());
      for (int j = 0; j < RESOURCES; j++)
	{
	  for (int k = 0; k < 3; k++)
	    {
	      savegame->putInt(fortrecipes[i].resources[j][k]);
	    }
	}
      savegame->putInt(fortrecipes[i].fuel);
      savegame->putInt(fortrecipes[i].amount);
      savegame->putInt(fortrecipes[i].worktime);
      savegame->putInt(fortrecipes[i].building);
      savegame->putInt(fortrecipes[i].known);
      savegame->putInt(fortrecipes[i].unlocktype[0]);
      savegame->putInt(fortrecipes[i].unlocktype[1]);
      savegame->putString(fortrecipes[i].unlockname.c_str());
      savegame->putString(fortrecipes[i].materials.c_str());
    }
  //Save designations
  for (int i = 0; i < 3; i++)
    {
      savegame->putInt(designated[i].size());
      if (designated[i].size() > 0)
	{
	  for (int j = 0; j < (int)designated[i].size(); j++)
	    {
	      savegame->putInt(designated[i].get(j));
	    }
	  designated[i].clear();
	}
    }
  //Save mission
  if (state == ADVENTURE)
    {
      savegame->putInt(find_mission(mission.name));
    }
  //Write to file
  savegame->saveToFile("data/savegame.dat");
  delete savegame;
  end_game();
}
