/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "help.hpp"
#include "ui.hpp"
#include <iostream>

vector<Help> helps;
VBox *helpmenu = new VBox(40, SCREENHEIGHT - 19, 1);

/**
 * Show selected help entry after click on button.
 * @param w Unused.
 * @param data Name of help entry to show.
 */
void help2Clb(Widget *w, void* data)
{
  string *name = static_cast<string*>(data);
  int index = find_help(*name);
  showHelp(index);
}

/**
 * Show selected help entry.
 * @param index Index of help entry or -1 to clear screen
 */
void showHelp(int index)
{
  TCODConsole::root->clear();
  if (index == NONE)
    {
      escinfo->setVisible(false);
      helpmenu->setVisible(false);
      return;
    }
  if (!helpmenu->isVisible())
    {
      helpmenu->setVisible(true);
    }
  escinfo->setVisible(true);
  TCODConsole::root->print(40, 2, helps[index].name.c_str());
  TCODConsole::root->printRect(2, 4, SCREENWIDTH - 4, SCREENHEIGHT - 20, helps[index].body.c_str());
  helpmenu->clear();
  helpmenu->addWidget(new Label(0, 0, ""));
  if (helps[index].options.size() > 0)
    {
      for (int i = 0; i < (int)helps[index].options.size(); i++)
	{
	  helpmenu->addWidget(new Button(helps[index].options[i].c_str(), NULL, help2Clb, new string(helps[index].options[i])));
	}
    }
  if (helps[index].parent.size() > 0)
    {
      helpmenu->addWidget(new Button("Back", NULL, help2Clb, new string(helps[index].parent)));
    }
  helpmenu->addWidget(new Label(0, 0, ""));
  helpmenu->computeSize();
}

/**
 * Find help entry with selected name.
 * @param name Name of help entry
 * @return Index of help entry.
 */
int find_help(string name)
{
  int help = NONE;
  for (int i = 0; i < (int)helps.size(); i++)
    {
      if (helps[i].name.compare(name) == 0)
	{
	  help = i;
	  break;
	}
    }
  return help;
}
