/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#ifndef ITEM_HPP
#define ITEM_HPP
#include <string>
#include "libtcod.hpp"

/**
 * Contains items types.
 */
typedef enum ItemTypes
  {
    TREE,
    ROCK,
    STONE,
    WOOD,
    RESOURCE,
    OBJECT,
    FUEL,
    VEIN,
    TIN,
    COPPER,
    COAL,
    ORE,
    BRONZE,
    METAL,
    LEG,
    ARM,
    TORSO,
    HEAD,
    HELMET,
    BRACERS,
    ARMOR,
    GREAVES,
    UPGRADE1,
    UPGRADE2,
    UPGRADE3,
    UPGRADE4,
    UPGRADE5,
    UPGRADE6,
    UPGRADE7,
    UPGRADE8,
    UPGRADE9,
    IRON,
    STEEL,
    COKE,
    SAND,
    HUPGRADE,
    AUPGRADE,
    TUPGRADE,
    LUPGRADE,
    WEAPON,
    LEATHER,
    BONE,
    PLANT,
    SEED,
    FRUIT,
    LEAF,
    HPOTION,
    MPOTION,
    EPOTION,
    USEABLE
  } ITEMTYPE;

/**
 * Contains all informations about item.
 */
class Item
{
public:
  /**
   * Name of item
   */
  std::string name;
  /**
   * Types of item
   */
  TCODList<int> types;
  /**
   * X-axis position of item
   */
  int x;
  /**
   * Y-axis position of item
   */
  int y;
  /**
   * Character to show on screen
   */
  int ch;
  /**
   * Color of character to show on screen
   */
  TCODColor color;
  /**
   * Did we can move by this item
   */
  bool walkable;
  /**
   * Did item block line of sigh
   */
  bool transparent;
  /**
   * Did item is used in any task
   */
  bool taken;
  /**
   * Did item is stored
   */
  bool stored;
  /**
   * Did item is hidden (invisible on screen)
   */
  bool hidden;
  /**
   * Description of item.
   */
  std::string description;
  /**
   * Did item is designated.
   */
  bool designated;
  /**
   * Various item data.
   */
  int data;
  /**
   * Damage done by item.
   */
  std::string damage;
  /**
   * Range for weapon
   */
  int range;
  /**
   * Use of item.
   */
  int use;

  Item();
  int additem(bool start);
  void setitem(int x1, int y1, int iindex = -1, bool start = false);
  void useitem();
  void deleteitem();
};

void buildItemsUI();
void setVisibleItems(bool visible);
int find_proto_obj(std::string name, TCODList<int> *types = new TCODList<int>());
int find_obj(std::string name, int x = -1, int y = -1, TCODList<int> *types = new TCODList<int>(), bool hidden = false);
int find_near_obj(std::string name, int x, int y, int radius, TCODList<int> *types, bool stored = true, bool hidden = false);
TCODList<int> find_all_proto_obj(TCODList<int> types);
TCODList<int> find_all_obj(TCODList<int> types);

#endif
