/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "main.hpp"
#include "ui.hpp"
#include "adventure.hpp"
#include <stdio.h>
#include <iostream>
#include "game.hpp"

Container *spellsmenu = new Container(0, 0, SCREENWIDTH, SCREENHEIGHT);

/**
 * Create spells menu.
 */
void buildSpellsUI()
{
  spellsmenu->setVisible(false);
}

/**
 * Cast all waiting spells.
 */
void cast_spells()
{
  int sindex, target, result;
  if (state == FORT)
    {
      for (int i = 0; i < (int)fortmobs.size(); i++)
	{
	  if (fortmobs[i].spells.isEmpty())
	    {
	      continue;
	    }
	  sindex = find_spell(fortmobs[i].work);
	  if (sindex == NONE)
	    {
	      continue;
	    }
	  target = fortmobs[i].target;
	  //Cast spell
	  if (fortmobs[i].worktime <= 0)
	    {
	      result = TCODRandom::getInstance()->diceRoll(protospells[sindex].dice.c_str());
	      if (protospells[sindex].type == DAMAGE)
		{
		  result = 0 - result;
		}
	      fortmobs[target].stats[7] += result;
	      if (fortmobs[target].stats[7] > fortmobs[target].stats[8])
		{
		  fortmobs[target].stats[7] = fortmobs[target].stats[8];
		}
	      if (fortmobs[target].stats[7] <= 0)
		{
		  fortmobs[target].stats[7] = 0;
		  fortmobs[target].deletemob();
		  if (target == 0)
		    {
		      end_game(true);
		      return;
		    }
		}
	      else if (protospells[sindex].type == DAMAGE)
		{
		  fortmobs[target].targettype = MOB;
		  fortmobs[target].target = i;
		  fortmobs[target].work = "Fighting";
		  fortmobs[target].generatepath(fortmobs[i].x, fortmobs[i].y);
		}
	      fortmobs[i].stats[14] -= protospells[sindex].cost;
	      fortmobs[i].gainexp(protospells[sindex].cost);
	      fortmobs[i].target = NONE;
	      fortmobs[i].targettype = NONE;
	      fortmobs[i].work = "IDLE";
	      if (i == 0)
		{
		  Notice("Spell was casted.");
		}
	    }
	}
    }
  else
    {
      for (int i = 0; i < (int)advmobs.size(); i++)
	{
	  if (advmobs[i].spells.isEmpty())
	    {
	      continue;
	    }
	  sindex = find_spell(advmobs[i].work);
	  if (sindex == NONE)
	    {
	      continue;
	    }
	  target = advmobs[i].target;
	  //Cast spell
	  if (advmobs[i].worktime <= 0)
	    {
	      result = TCODRandom::getInstance()->diceRoll(protospells[sindex].dice.c_str());
	      if (protospells[sindex].type == DAMAGE)
		{
		  result = 0 - result;
		}
	      advmobs[target].stats[7] += result;
	      if (advmobs[target].stats[7] > advmobs[target].stats[8])
		{
		  advmobs[target].stats[7] = advmobs[target].stats[8];
		}
	      if (advmobs[target].stats[7] <= 0)
		{
		  advmobs[target].stats[7] = 0;
		  advmobs[target].deletemob();
		  if (target == 0)
		    {
		      end_game(true);
		      return;
		    }
		}
	      else if (protospells[sindex].type == DAMAGE)
		{
		  advmobs[target].targettype = MOB;
		  advmobs[target].target = i;
		  advmobs[target].work = "Fighting";
		  advmobs[target].generatepath(advmobs[i].x, advmobs[i].y);
		}
	      advmobs[i].stats[14] -= protospells[sindex].cost;
	      advmobs[i].gainexp(protospells[sindex].cost);
	      advmobs[i].target = NONE;
	      advmobs[i].targettype = NONE;
	      advmobs[i].work = "IDLE";
	      if (i == 0)
		{
		  Notice("Spell was casted.");
		}
	    }
	}
    }
}

/**
 * Set target for spell
 * @param sindex Index of spell.
 */
void magic_target(int sindex)
{
  TCOD_mouse_t mouse = TCODMouse::getStatus();
  TCODColor color = TCODColor::red;
  int target;
  bool valid = false;
  if (state == MTARGET)
    {
      draw_fort();
      if (protospells[sindex].target == ENEMY) 
	{
	  if (fortmap[mouse.cy + screeny][mouse.cx + screenx].mob != NONE)
	    {
	      target = fortmap[mouse.cy + screeny][mouse.cx + screenx].mob;
	      if ((fortmobs[target].type != ROBOT) && (target != 0))
		{
		  color = TCODColor::green;
		  valid = true;
		}
	    }
	}
    }
  else
    {
      draw_adventure();
      if (protospells[sindex].target == ENEMY) 
	{
	  if (advmap[mouse.cy + screeny][mouse.cx + screenx].mob != NONE)
	    {
	      target = advmap[mouse.cy + screeny][mouse.cx + screenx].mob;
	      if ((advmobs[target].type != ROBOT) && (target != 0))
		{
		  color = TCODColor::green;
		  valid = true;
		}
	    }
	}
    }
  offconsole->clear();
  offconsole->putCharEx(0, 0, '+', color, TCODColor::black);
  TCODConsole::blit(offconsole, 0, 0, 1, 1, TCODConsole::root, mouse.cx, mouse.cy, 1.0, 0.0);
  if (!escinfo->isVisible())
    {
      escinfo->setVisible(true);
    }
  if ((mouse.lbutton) && (valid))
    {
      if (state == MTARGET)
	{
	  fortmobs[0].targettype = MOB;
	  fortmobs[0].target = fortmap[mouse.cy + screeny][mouse.cx + screenx].mob;
	  fortmobs[0].work = protospells[sindex].name;
	  fortmobs[0].worktime = 1;
	  fortmobs[0].generatepath(mouse.cx, mouse.cy, protospells[sindex].range);
	  setVisibleGamemenu(true);
	  state = FORT;
	}
      else
	{
	  advmobs[0].targettype = MOB;
	  advmobs[0].target = advmap[mouse.cy + screeny][mouse.cx + screenx].mob;
	  advmobs[0].work = protospells[sindex].name;
	  advmobs[0].worktime = 1;
	  advmobs[0].generatepath(mouse.cx, mouse.cy, protospells[sindex].range);
	  setVisibleAdvmenu(true);
	  state = ADVENTURE;
	}
      escinfo->setVisible(false);
      TCODMouse::showCursor(true);
    }
}

/**
 * Cast or select target for selected spell
 * @param w Unused.
 * @param data Index of spell to cast
 */
void magicClb(Widget *w, void* data)
{
  int *index = static_cast<int*>(data);
  if (state == CAST)
    {
      if (fortmobs[0].stats[14] < protospells[*index].cost)
	{
	  return;
	}
    }
  else
    {
      if (advmobs[0].stats[14] < protospells[*index].cost)
	{
	  return;
	}
    }
  //Cast spell on self
  if (protospells[*index].target == SELF)
    {
      if (state == CAST)
	{
	  fortmobs[0].targettype = MOB;
	  fortmobs[0].target = 0;
	  fortmobs[0].work = protospells[*index].name;
	  fortmobs[0].worktime = 1;
	  setVisibleGamemenu(true);
	  state = FORT;
	}
      else
	{
	  advmobs[0].targettype = MOB;
	  advmobs[0].target = 0;
	  advmobs[0].work = protospells[*index].name;
	  advmobs[0].worktime = 1;
	  setVisibleAdvmenu(true);
	  state = ADVENTURE;
	}
      setVisibleSpells(false);
    }
  //Select target for cast
  else
    {
      setVisibleSpells(false);
      TCODMouse::showCursor(false);
      if (state == CAST)
	{
	  state = MTARGET;
	}
      else
	{
	  state = MTARGET2;
	}
      stage = *index;
      magic_target(*index);
    }
}

/**
 * Show/hide spells menu
 * @param visible Did menu will be visible or not.
 */
void setVisibleSpells(bool visible)
{
  if (visible)
    {
      spellsmenu->clear();
      spellsmenu->addWidget(new Label(7, 2, "Name"));
      spellsmenu->addWidget(new Label(62, 2, "Mana"));
      int index;
      char *c;
      if (state == CAST)
	{
	  for (int i = 0; i < fortmobs[0].spells.size(); i++)
	    {
	      index = fortmobs[0].spells.get(i);
	      spellsmenu->addWidget(new Button(7, i + 4, 10, 1, protospells[index].name.c_str(), "Cast selected spell", magicClb, new int(index)));
	      c = new char[5];
	      sprintf(c, "%d", protospells[index].cost);
	      spellsmenu->addWidget(new Label(62, i + 4, c));
	    }
	}
      else
	{
	  for (int i = 0; i < advmobs[0].spells.size(); i++)
	    {
	      index = advmobs[0].spells.get(i);
	      spellsmenu->addWidget(new Button(7, i + 4, 10, 1, protospells[index].name.c_str(), "Cast selected spell", magicClb, new int(index)));
	      c = new char[5];
	      sprintf(c, "%d", protospells[index].cost);
	      spellsmenu->addWidget(new Label(62, i + 4, c));
	    }
	}
    }
  escinfo->setVisible(visible);
  spellsmenu->setVisible(visible);
  TCODMouse::showCursor(true);
}

/**
 * Find selected spell.
 * @param name Name of spell to find.
 * @return Index of spell or -1 if spell not found.
 */
int find_spell(std::string name)
{
  int spell = NONE;
  for (int i = 0; i < (int)protospells.size(); i++)
    {
      if (protospells[i].name.compare(name) == 0)
	{
	  spell = i;
	  break;
	}
    }
  return spell;
}
