/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#ifndef MAGIC_HPP
#define MAGIC_HPP
#include <string>

/**
 * Contains spells types
 */
typedef enum SpellTypes
  {
    DAMAGE,
    HEALING
  } SPELLTYPE;

/**
 * Contains types of targets for spells
 */
typedef enum SpellTargets
  {
    SELF,
    ENEMY
  } SPELLTARGET;

/**
 * Contains all informations about spell.
 */
struct Spell
{
  /**
   * Name of spell
   */
  std::string name;
  /**
   * Type of spell
   */
  int type;
  /**
   * Target type of spell
   */
  int target;
  /**
   * Dice (for damage, healing, duration, etc)
   */
  std::string dice;
  /**
   * Additional effect of spell
   */
  int effect;
  /**
   * Cost in mana for cast spell.
   */
  int cost;
  /**
   * Amount of memory required by spell
   */
  int memory;
  /**
   * Range of spell
   */
  int range;
};

void buildSpellsUI();
void setVisibleSpells(bool visible);
int find_spell(std::string name);
void magic_target(int sdindex);
void cast_spells();

#endif
