/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include <vector>
#include <iostream>
#include "libtcod.hpp"
#include "main.hpp"
#include "building.hpp"
#include "item.hpp"
#include "mobile.hpp"
#include "order.hpp"
#include "utils.hpp"
#include "mainmenu.hpp"
#include "game.hpp"
#include "ui.hpp"
#include "gui/gui.hpp"
#include "designation.hpp"
#include "design.hpp"
#include "message.hpp"
#include "ai.hpp"
#include "combat.hpp"
#include "fort.hpp"
#include "mission.hpp"
#include "adventure.hpp"
#include "help.hpp"
#include "mobileui.hpp"

using namespace std;

/**
 * Contains all proto items
 */
vector<Item> protoobjs;
/**
 * Contains all proto mobiles.
 */
vector<Mobile> protomobs;
/**
 * Contains all proto buildings.
 */
vector<Building> protobuilds;
/**
 * Contains all informations about missions.
 */
vector<Mission> protomissions;
/**
 * Contains all informations about spells.
 */
vector<Spell> protospells;
/**
 * Map with information about cells (walkable, transparent).
 */
TCODMap *fovmap = new TCODMap(MAPWIDTH, MAPHEIGHT);
/**
 * Used in some game options.
 */
int stage;
/**
 * Offscreen console, used to draw temporary things
 */
TCODConsole *offconsole = new TCODConsole(SCREENWIDTH, SCREENHEIGHT);
/**
 * X-axis position of top left corner of screen
 */
int screenx = (MAPWIDTH / 2) - (SCREENWIDTH / 2);
/**
 * Y-axis position of top left corner of screen
 */
int screeny = (MAPHEIGHT / 2) - (SCREENHEIGHT / 2);
/**
 * State of game
 */
GameState state = MAINMENU;
/**
 * Amount of second elapsed (used to count game time)
 */
float secs = 0;
/**
 * Game time 0 - hour, 1 - day, 2 - month, 3 - year
 */
int gtime[4] = {8, 1, 4, 400};

/**
 * Main game function. Contains main game loop
 */
int main ()
{
  TCODConsole::setCustomFont("data/terminal.png");
  TCODConsole::initRoot(SCREENWIDTH, SCREENHEIGHT, "Steam Fort 0.6 alpha", false, TCOD_RENDERER_SDL);
  TCODSystem::setFps(50);
  TCODConsole *guicon = new TCODConsole(SCREENWIDTH, SCREENHEIGHT);
  Widget::setConsole(guicon);
  Widget::setForegroundColor(TCODColor::white, TCODColor::red);
  Widget::setBackgroundColor(TCODColor::black, TCODColor::black);
  createUI();
  bool endgame = false;
  stage = 0;
  TCOD_key_t key;
  TCOD_mouse_t mouse;
  bool paused = false;
  while (!endgame) 
    {
      guicon->clear();
      Widget::renderWidgets();
      switch (state)
	{
	case MAINMENU:
	case FORT:
	case CONSTRUCT:
	case MARK:
	case ADVENTURE:
	case HELP:
	case HELP2:
	case MTARGET:
	case MTARGET2:
	  TCODConsole::blit(guicon, 0, 0, SCREENWIDTH, SCREENHEIGHT, TCODConsole::root, 0, 0, 1, 0);
	  break;
	case ABOUT:
	  break;
	default:
	  TCODConsole::blit(guicon, 0, 0, SCREENWIDTH, SCREENHEIGHT, TCODConsole::root, 0, 0, 1, 0.3);
	  break;
	}
      TCODConsole::flush();
      key = TCODConsole::checkForKeypress(TCOD_KEY_PRESSED);
      mouse.lbutton_pressed = false;
      mouse = TCODMouse::getStatus();
      Widget::updateWidgets(key);
      if ((key.lctrl) && (key.vk == TCODK_PRINTSCREEN))
	{
	  TCODSystem::saveScreenshot(NULL);
	  continue;
	}
      if (state == MAINMENU)
	{
	  TCODConsole::root->clear();
	  continue;
	}
      if (state == ENDGAME)
	{
	  endgame = true;
	  continue;
	}
      if ((state == ABOUT) && (key.vk != TCODK_NONE))
	{
	  state = MAINMENU;
	  mainmenu->setVisible(true);
	  TCODConsole::root->clear();
	  continue;
	}
      if (state == FORT)
	{
	  switch (key.vk)
	    {
	    case TCODK_LEFT:
	      screenx -= 2;
	      break;
	    case TCODK_RIGHT:
	      screenx += 2;
	      break;
	    case TCODK_UP:
	      screeny -= 2;
	      break;
	    case TCODK_DOWN:
	      screeny += 2;
	      break;
	    case TCODK_SPACE:
	      paused = !paused;
	      if (paused)
		{
		  Notice("Game paused");
		}
	      else
		{
		  Notice("Game unpaused");
		}
	      break;
	    default:
	      break;
	    }
	  if (mouse.rbutton)
	    {
	      screenx += (3 * mouse.dcx);
	      screeny += (3 * mouse.dcy);
	    }
	  if (mouse.lbutton)
	    {
	      setInfo(true, mouse.cx, mouse.cy);
	    }
	  if (mouse.lbutton_pressed)
	    {
	      setInfo(false);
	    }
	  if (!paused)
	    {
	      update_fort();
	    }
	  else
	    {
	      draw_fort();
	    }
	  continue;
	}
      if (state == ADVENTURE)
	{
	  if (mouse.lbutton_pressed)
	    {
	      move_player(mouse.cx, mouse.cy);
	    }
	  update_adventure();
	  continue;
	}
      if (key.vk == TCODK_ESCAPE)
	{
	  switch (state)
	    {
	    case CREW:
	      setVisibleCrew(false);
	    case ORDERS:
	      setVisibleOrders(false);
	    case BUILD:
	      setVisibleBuild(false);
	    case OBJECTS:
	      setVisibleItems(false);
	    case MARKS:
	      setVisibleDesignation(false);
	    case DESIGN:
	      setVisibleDesign(false);
	    case MESSAGES:
	      setVisibleMessage(false);
	    case MISSIONS:
	      setVisibleMission(false);
	    case HELP:
	      showHelp(NONE);
	    case CAST:
	      setVisibleSpells(false);
	    case USE:
	      setVisibleUse(false);
	      state = FORT;
	      if (gtime[2] < 3)
		{
		  Widget::setForegroundColor(TCODColor::black, TCODColor::red);
		}
	      draw_fort();
	      setVisibleGamemenu(true);
	      break;
	    case CONSTRUCT:
	      state = BUILD;
	      Widget::setForegroundColor(TCODColor::white, TCODColor::red);
	      setVisibleBuild(true);
	      break;
	    case MOBINFO:
	      state = CREW;
	      setVisibleCrew(true);
	      break;
	    case ORDER:
	      state = ORDERS;
	      setVisibleOrders(true);
	      break;
	    case MARK:
	      state = MARKS;
	      Widget::setForegroundColor(TCODColor::white, TCODColor::red);
	      setVisibleDesignation(true);
	      break;
	    case MOBINV:
	      state = MOBINFO;
	      showMobInfo(stage);
	      break;
	    case MOBINFO2:
	      showMobInfo(NONE);
	      setVisibleCrew(false);
	    case HELP2:
	      showHelp(NONE);
	    case CAST2:
	      setVisibleSpells(false);
	    case USE2:
	      state = ADVENTURE;
	      setVisibleAdvmenu(true);
	      draw_adventure();
	      break;
	    case MOBINV2:
	      state = MOBINFO2;
	      showMobInfo(stage);
	      break;
	    case MTARGET:
	      state = CAST;
	      Widget::setForegroundColor(TCODColor::white, TCODColor::red);
	      setVisibleSpells(true);
	      break;
	    case MTARGET2:
	      state = CAST2;
	      setVisibleSpells(true);
	      break;
	    default:
	      break;
	    }
	  continue;
	}
      if ((state == CONSTRUCT) && (key.vk != NONE))
	{
	  state = (GameState)construct(stage, key.vk);
	  if (state == BUILD)
	    {
	      Widget::setForegroundColor(TCODColor::white, TCODColor::red);
	      setVisibleBuild(true);
	    }
	  continue;
	}
      if (state == MARK)
	{
	  if (stage == NONE)
	    {
	      stage = 0;
	    }
	  else
	    {
	      designation(&mouse);
	      continue;
	    }
	}
      if ((state == MTARGET) || (state == MTARGET2))
	{
	  magic_target(stage);
	  continue;
	}
    }
  return 0;
}
