/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "map.hpp"
#include "main.hpp"
#include "adventure.hpp"

/**
 * Function update map when mob value was changed.
 * @param x X-axis position on map.
 * @param y Y-axis position on map.
 * @param mob Index of mob.
 */
void update_map_mob(int x, int y, int mob)
{
  //Fort mode
  if (state == FORT)
    {
      //Remove mob.
      if (mob == NONE)
	{
	  fortmap[y][x].mob = NONE;
	  int i = fortmap[y][x].items.size();
	  if (i == 0)
	    {
	      i = fortmap[y][x].building;
	      if (i == NONE)
		{
		  fovmap->setProperties(x, y, true, true);
		  fortmap[y][x].transparent = true;
		  fortmap[y][x].walkable = true;
		  fortmap[y][x].color = fortmap[y][x].bgcolor;
		  fortmap[y][x].ch = 0;
		}
	      else
		{
		  if (fortbuilds[i].constructed)
		    {
		      fortmap[y][x].color = fortbuilds[i].colors.get((fortbuilds[i].width * fortbuilds[i].enter[1]) + fortbuilds[i].enter[0]);
		      fortmap[y][x].ch = fortbuilds[i].chars.get((fortbuilds[i].width * fortbuilds[i].enter[1]) + fortbuilds[i].enter[0]);
		    }
		  else
		    {
		      fortmap[y][x].color = TCODColor::white;
		      fortmap[y][x].ch = 'X';
		    }
		  fortmap[y][x].transparent = false;
		  fortmap[y][x].walkable = true;
		  fovmap->setProperties(x, y, false, true);
		}
	    }
	  else
	    {
	      i = fortmap[y][x].items.peek();
	      fovmap->setProperties(x, y, fortobjs[i].transparent, fortobjs[i].walkable);
	      fortmap[y][x].transparent = fortobjs[i].transparent;
	      fortmap[y][x].walkable = fortobjs[i].walkable;
	      fortmap[y][x].color = fortobjs[i].color;
	      fortmap[y][x].ch = fortobjs[i].ch;
	    }
	}
      //Add mob
      else
	{
	  fortmap[y][x].transparent = true;
	  fortmap[y][x].walkable = false;
	  fortmap[y][x].ch = fortmobs[mob].ch;
	  fortmap[y][x].color = fortmobs[mob].color;
	  fortmap[y][x].mob = mob;
	  fovmap->setProperties(x, y, true, false);
	}
    }
  //Adventure mode
  else
    {
      //Remove mob
      if (mob == NONE)
	{
	  advmap[y][x].mob = NONE;
	  int i = advmap[y][x].items.size();
	  if (i == 0)
	    {
	      fovmap->setProperties(x, y, true, true);
	      advmap[y][x].transparent = true;
	      advmap[y][x].walkable = true;
	      advmap[y][x].color = advmap[y][x].bgcolor;
	      advmap[y][x].ch = 0;
	    }
	  else
	    {
	      i = advmap[y][x].items.peek();
	      fovmap->setProperties(x, y, advobjs[i].transparent, advobjs[i].walkable);
	      advmap[y][x].transparent = advobjs[i].transparent;
	      advmap[y][x].walkable = advobjs[i].walkable;
	      advmap[y][x].color = advobjs[i].color;
	      advmap[y][x].ch = advobjs[i].ch;
	    }
	}
      //Add mob
      else
	{
	  advmap[y][x].transparent = true;
	  advmap[y][x].walkable = false;
	  advmap[y][x].ch = advmobs[mob].ch;
	  advmap[y][x].color = advmobs[mob].color;
	  advmap[y][x].mob = mob;
	  fovmap->setProperties(x, y, true, false);
	}
    }
}

/**
 * Function update map when item value was changed.
 * @param x X-axis position on map.
 * @param y Y-axis position on map.
 * @param item Index of item.
 * @param remove If true, remove item, otherwise add item. Default false.
 */
void update_map_item(int x, int y, int item, bool remove)
{
  //Fort mode
  if (state != ADVENTURE)
    {
      //Add item to map
      if (!remove)
	{
	  if (!fortobjs[item].hidden)
	    {
	      if ((fortmap[y][x].mob == NONE) && (fortmap[y][x].building == NONE))
		{
		  fortmap[y][x].walkable = fortobjs[item].walkable;
		  fortmap[y][x].transparent = fortobjs[item].transparent;
		  fortmap[y][x].ch = fortobjs[item].ch;
		  fortmap[y][x].color = fortobjs[item].color;
		  fovmap->setProperties(x, y, fortobjs[item].transparent, fortobjs[item].walkable);

		}
	      fortmap[y][x].items.push(item);
	    }
	  else
	    {
	      fortmap[y][x].hitems.push(item);
	    }
	}
      //Remove item
      else
	{
	  if (!fortobjs[item].hidden)
	    {
	      fortmap[y][x].items.remove(item);
	      if ((fortmap[y][x].mob == NONE) && (fortmap[y][x].building == NONE))
		{
		  if (!fortmap[y][x].items.isEmpty())
		    {
		      int index = fortmap[y][x].items.peek();
		      fortmap[y][x].walkable = fortobjs[index].walkable;
		      fortmap[y][x].transparent = fortobjs[index].transparent;
		      fortmap[y][x].ch = fortobjs[index].ch;
		      fortmap[y][x].color = fortobjs[index].color;
		      fovmap->setProperties(x, y, fortobjs[index].transparent, fortobjs[index].walkable);
		    }
		  else
		    {
		      fortmap[y][x].walkable = true;
		      fortmap[y][x].transparent = true;
		      fortmap[y][x].ch = 0;
		      fortmap[y][x].color = fortmap[y][x].bgcolor;
		      fovmap->setProperties(x, y, true, true);
		    }
		}
	    }
	  else
	    {
	      fortmap[y][x].hitems.remove(item);
	    }
	}
    }
  //Adventure mode
  else
    {
      //Add item to map
      if (!remove)
	{
	  if (!advobjs[item].hidden)
	    {
	      if ((advmap[y][x].mob == NONE) && (advmap[y][x].building == NONE))
		{
		  advmap[y][x].walkable = advobjs[item].walkable;
		  advmap[y][x].transparent = advobjs[item].transparent;
		  advmap[y][x].ch = advobjs[item].ch;
		  advmap[y][x].color = advobjs[item].color;
		  fovmap->setProperties(x, y, advobjs[item].transparent, advobjs[item].walkable);

		}
	      advmap[y][x].items.push(item);
	    }
	  else
	    {
	      advmap[y][x].hitems.push(item);
	    }
	}
      //Remove item
      else
	{
	  if (!advobjs[item].hidden)
	    {
	      advmap[y][x].items.remove(item);
	      if ((advmap[y][x].mob == NONE) && (advmap[y][x].building == NONE))
		{
		  if (!advmap[y][x].items.isEmpty())
		    {
		      int index = advmap[y][x].items.peek();
		      advmap[y][x].walkable = advobjs[index].walkable;
		      advmap[y][x].transparent = advobjs[index].transparent;
		      advmap[y][x].ch = advobjs[index].ch;
		      advmap[y][x].color = advobjs[index].color;
		      fovmap->setProperties(x, y, advobjs[index].transparent, advobjs[index].walkable);
		    }
		  else
		    {
		      advmap[y][x].walkable = true;
		      advmap[y][x].transparent = true;
		      advmap[y][x].ch = 0;
		      advmap[y][x].color = advmap[y][x].bgcolor;
		      fovmap->setProperties(x, y, true, true);
		    }
		}
	    }
	  else
	    {
	      advmap[y][x].hitems.remove(item);
	    }
	}
    }
}
