/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "message.hpp"
#include "main.hpp"
#include "ui.hpp"
#include <iostream>
#include <stdio.h>

vector<Message> msgs;
Container *messgs = new Container(0, 0, 100, 75);

/**
 * Standard class constructor.
 * @param msg Text of message.
 * @param x1 X-axis position where message occurs.
 * @param y1 Y-axis position where message occurs.
 */
Message::Message(std::string msg, int x1, int y1)
{
  text = strdup(msg.c_str());
  x = x1;
  y = y1;
}

/**
 * Add new message to list and delete olders if needed.
 * @param msg Message to add.
 */
void addMessage(Message *msg)
{
  if (msgs.size() > 29)
    {
      msgs.erase(msgs.begin());
    }
  Notice(msg->text);
  msgs.push_back(*msg);
}

/**
 * Delete all messages.
 * @param w Unused.
 * @param data Unused.
 */
void clearmsgClb(Widget *w, void *data)
{
  msgs.clear();
  setVisibleMessage(false);
  state = FORT;
  draw_fort();
  setVisibleGamemenu(true);
}

/**
 * Build messages UI.
 */
void buildMessageUI()
{
  messgs->setVisible(false);
}

/**
 * Go to place where selected message occurs.
 * @param w Unused.
 * @param data Selected message.
 */
void gotomsgClb(Widget *w, void *data)
{
  Message *msg = (Message *)data;
  screenx = msg->x - (SCREENWIDTH / 2);
  if (screenx < 0)
    {
      screenx = 0;
    }
  screeny = msg->y - (SCREENHEIGHT / 2);
  if (screeny < 0)
    {
      screeny = 0;
    }
  setVisibleMessage(false);
  state = FORT;
  draw_fort();
  setVisibleGamemenu(true);
}

/**
 * Show/hide messages menu
 * @param visible If true, show menu, otherwise false.
 */
void setVisibleMessage(bool visible)
{
  if (visible)
    {
      int line = 2;
      messgs->clear();
      for (int i = 0; i < (int)msgs.size(); i++)
	{
	  messgs->addWidget(new Button(0, line, 30, 1, msgs[i].text, "Go to place where message occur.", gotomsgClb, &msgs[i]));
	  line ++;
	}
      messgs->addWidget(new Button(40, 60, 10, 1, "Clear", "Clear all messages and quit.", clearmsgClb, NULL));
    }
  messgs->setVisible(visible);
  escinfo->setVisible(visible);
}
