/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "mission.hpp"
#include "ui.hpp"
#include "main.hpp"
#include <iostream>
#include "adventure.hpp"

Container *missionmenu = new Container(0, 0, SCREENWIDTH, SCREENHEIGHT);
int missions[3];

/**
 * Standard class constructor.
 */
Mission::Mission()
{
  location = DUNGEONS;
  length = NONE;
  difficult = NONE;
  name = "";
  description = "";
  target = "";
  targettype = NONE;
}

/**
 * Build missions user interface.
 */
void buildMissionUI()
{
  missionmenu->setVisible(false);
}

/**
 * Start adventure
 * @param w Unused.
 * @param data Number of mission.
 */
void missionClb(Widget *w, void *data)
{
  int *p = static_cast<int*>(data);
  int index = missions[*p];
  missions[*p] = NONE;
  setVisibleMission(false);
  start_adventure(index);
}

/**
 * Show mission difficulty
 * @param index Index of mission.
 * @param line Line of text.
 */
void missiondiff(int index, int line)
{
  if (protomissions[index].difficult <= 0)
    {
      missionmenu->addWidget(new Label(35, line, "Difficulty: Easy"));
    }
  else if ((protomissions[index].difficult > 0) && (protomissions[index].difficult < 5))
    {
      missionmenu->addWidget(new Label(35, line, "Difficulty: Moderate"));
    }
  else if (protomissions[index].difficult > 5)
    {
      missionmenu->addWidget(new Label(35, line, "Difficulty: Hard"));
    }
}

/**
 * Show mission length
 * @param index Index of mission.
 * @param line Line of text.
 */
void missionlength(int index, int line)
{
  if (protomissions[index].length < 4)
    {
      missionmenu->addWidget(new Label(60, line, "Lenght: Short"));
    }
  else if ((protomissions[index].length > 3) && (protomissions[index].length < 11))
    {
      missionmenu->addWidget(new Label(60, line, "Lenght: Medium"));
    }
  else if (protomissions[index].length > 10)
    {
      missionmenu->addWidget(new Label(60, line, "Lenght: Long"));
    }
}

/**
 * Show mission location
 * @param index Index of mission.
 * @param line Line of text.
 */
void missionloc(int index, int line)
{
  switch (protomissions[index].location)
    {
    case DUNGEONS:
      missionmenu->addWidget(new Label(80, line, "Location: Dungeons"));
      break;
    default:
      break;
    }
}

/**
 * Generate available missions.
 */
void generatemissions()
{
  for (int i = 0; i < 3; i++)
    {
      missions[i] = NONE;
    }
  int amount = protomissions.size();
  if (amount < 4)
    {
      for (int i = 0; i < amount; i++)
	{
	  missions[i] = i;
	}
    }
  else
    {
      TCODRandom *generator = TCODRandom::getInstance();
      int index = 0, result;
      bool valid = true;
      while (index < 3)
	{
	  result = generator->getInt(0, amount - 1);
	  for (int i = 0; i < 3; i++)
	    {
	      if (missions[i] == result)
		{
		  valid = false;
		  break;
		}
	    }
	  if (valid)
	    {
	      missions[index] = result;
	      index ++;
	    }
	}
    }
}

/**
 * Show/hide information about missions.
 * @param visible If true, show missions, otherwise hide menu.
 */
void setVisibleMission(bool visible)
{
  if (visible)
    {
      missionmenu->clear();
      int line = 2, index;
      for (int i = 0; i < 3; i++)
	{
	  if (missions[i] == NONE)
	    {
	      continue;
	    }
	  index = missions[i];
	  missionmenu->addWidget(new Button(5, line, 15, 1, protomissions[index].name.c_str(), "Go on mission!", missionClb, new int(i)));
	  missiondiff(index, line);
	  missionlength(index, line);
	  missionloc(index, line);
	  missionmenu->addWidget(new Label(2, line + 2, protomissions[index].description.c_str()));
	  line += 10;
	}
      if (line == 2)
	{
	  missionmenu->addWidget(new Label(35, 30, "No missions available today."));
	}
      missionmenu->computeSize();
    }
  missionmenu->setVisible(visible);
  escinfo->setVisible(visible);
}

/**
 * Find mission by name.
 * @param name Name of mission to find.
 * @return Index of mission or -1 if not found.
 */
int find_mission(string name)
{
  int mission = NONE;
  for (int i = 0; i < (int)protomissions.size(); i++)
    {
      if (protomissions[i].name.compare(name) == 0)
	{
	  mission = i;
	  break;
	}
    }
  return mission;
}
