/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "libtcod.hpp"
#include "mobile.hpp"
#include "utils.hpp"
#include "main.hpp"
#include <iostream>
#include <stdio.h>
#include "message.hpp"
#include <map>
#include "adventure.hpp"
#include "game.hpp"

/**
 * Standard class constructor.
 * Set mobile path finder.
 */
Mobile::Mobile()
{
  taken = NONE;
  description = "";
  path = 0;
  flying = false;
}

/**
 * Class constructor.
 * Create new mobile and fill some variables.
 * @param c Character used to display mobile on screen
 * @param col Color for character used to display mobile on screen
 * @param t Type of mobile (player, robot, etc)
 */
Mobile::Mobile(char c, TCODColor col, int t)
{
  ch = c;
  color = col;
  type = t;
  work = (char *)"IDLE";
  targettype = -1;
  target = -1;
  worktime = 0;
  carries = -1;
  taken = NONE;
  workstage = 0;
  for (int i = 0; i < 16; i++)
    {
      stats[i] = NONE;
    }
  for (int i = 0; i < 5; i++)
    {
      equipment[i] = NONE;
    }
  walktime = 0;
  description = "";
  for (int i = 0; i < 3; i++)
    {
      ais[i] = NONE;
    }
  path = 0;
  flying = false;
}

/**
 * Add mob to mobs vector.
 * @param start True mean we start game.
 */
int Mobile::addmob(bool start)
{
  int mob = NONE;
  if (state == FORT)
    {
      if ((fortmobs.size() > 0) && (!start))
	{
	  for (int i = 0; i < (int)fortmobs.size(); i++)
	    {
	      if ((fortmobs[i].x == NONE) || (fortmobs[i].y == NONE))
		{
		  fortmobs[i] = *this;
		  mob = i;
		  break;
		}
	    }
	}
      if (mob == NONE)
	{
	  mob = fortmobs.size();
	  fortmobs.push_back(*this);
	}
      //Set mobile equipment
      int index;
      for (int i = 0; i < 5; i ++)
	{
	  index = fortmobs[mob].equipment[i];
	  if (index == NONE)
	    {
	      continue;
	    }
	  protoobjs[index].setitem(0, 0, NONE);
	  index = find_obj(protoobjs[index].name);
	  fortobjs[index].useitem();
	  fortmobs[mob].equipment[i] = index;
	}
      //Set mobile inventory
      if (!fortmobs[mob].inventory.isEmpty())
	{
	  for (int i = 0; i < fortmobs[mob].inventory.size(); i ++)
	    {
	      index = fortmobs[mob].inventory.get(i);
	      if (index == NONE)
		{
		  continue;
		}
	      protoobjs[index].setitem(0, 0, NONE);
	      index = find_obj(protoobjs[index].name);
	      fortobjs[index].useitem();
	      fortmobs[mob].inventory.set(index, i);
	    }
	}
      fortmobs[mob].path = new TCODPath(fovmap);
    }
  else
    {
      if ((advmobs.size() > 0) && (!start))
	{
	  for (int i = 0; i < (int)advmobs.size(); i++)
	    {
	      if ((advmobs[i].x == NONE) || (advmobs[i].y == NONE))
		{
		  advmobs[i] = *this;
		  mob = i;
		  break;
		}
	    }
	}
      if (mob == NONE)
	{
	  mob = advmobs.size();
	  advmobs.push_back(*this);
	}
      //Set mobile equipment
      int index;
      for (int i = 0; i < 5; i ++)
	{
	  index = advmobs[mob].equipment[i];
	  if (index == NONE)
	    {
	      continue;
	    }
	  protoobjs[index].setitem(0, 0, NONE);
	  index = find_obj(protoobjs[index].name);
	  advobjs[index].useitem();
	  advmobs[mob].equipment[i] = index;
	}
      //Set mobile inventory
      if (!advmobs[mob].inventory.isEmpty())
	{
	  for (int i = 0; i < advmobs[mob].inventory.size(); i ++)
	    {
	      index = advmobs[mob].inventory.get(i);
	      if (index == NONE)
		{
		  continue;
		}
	      protoobjs[index].setitem(0, 0, NONE);
	      index = find_obj(protoobjs[index].name);
	      advobjs[index].useitem();
	      advmobs[mob].inventory.set(index, i);
	    }
	}
      advmobs[mob].path = new TCODPath(fovmap);
    }
  return mob;
}

/**
 * Draw mobile on sceen.
 * @param x1 X-axis position on screen
 * @param y1 Y-axis position on screen
 * @param mindex Index of mobile.
 * @param start True means we start game. Default is false.
 */
void Mobile::setmob(int x1, int y1, int mindex, bool start)
{
  TCODRandom *generator = TCODRandom::getInstance();
  while (true)
    {
      if (valid_pos(x1, y1))
	{
	  x = x1;
	  y = y1;
	  if (mindex == NONE)
	    {
	      mindex = addmob(start);
	    }
	  update_map_mob(x, y, mindex);
	  break;
	}
      else
	{
	  x1 = generator->getInt(x1 - 5, x1 + 5);
	  if ((x1 < 0) || (x1 >= MAPWIDTH))
	    {
	      x1 = MAPWIDTH / 2;
	    }
	  y1 = generator->getInt(y1 - 5, y1 + 5);
	  if ((y1 < 0) || (y1 >= MAPHEIGHT))
	    {
	      y1 = MAPHEIGHT / 2;
	    }
	}
    };
}

/**
 * Generate stats of mobile.
 * @param newlevel New level of mobile. Default -1.
 */
void Mobile::generatestats(int newlevel)
{
  if (newlevel != NONE)
    {
      stats[10] = newlevel;
    }
  TCODRandom *generator = TCODRandom::getInstance();
  char *c;
  for (int i = 2; i <= 6; i++)
    {
      c = new char[4];
      sprintf(c, "1d%d + %d", stats[i], stats[10]);
      stats[i] = generator->diceRoll(c);
    }
  c = new char[4];
  sprintf(c, "1d%d + %d", stats[9], stats[10]);
  stats[9] = generator->diceRoll(c);
  stats[7] = stats[9] + stats[7];
  stats[8] = stats[9] + stats[8];
  stats[14] = stats[14] + stats[6];
  stats[15] = stats[15] + stats[6];
}

/**
 * Generate name for mob.
 */
void Mobile::generatename()
{
  switch (type)
    {
    case PLAYER:
      TCODNamegen::parse("data/namegen/mingos_standard.cfg");
      name = TCODNamegen::generate((char *)"male");
      break;
    default:
      TCODNamegen::parse("data/namegen/jice_mesopotamian.cfg");
      name = TCODNamegen::generate((char *)"Mesopotamian male");
      break;
    }
  TCODNamegen::destroy();
}

/**
 * Generate path to selected target.
 * @param x1 X-axis position of target
 * @param y1 Y-axis position of target
 * @param range Size of range in which destination will be checked.
 * @return True if path was computed, otherwise false
 */
bool Mobile::generatepath(int x1, int y1, int range)
{
  if ((x1 < 0) || (y1 < 0))
    {
      return false;
    }
  if (!valid_pos(x1, y1))
    {
      bool walkable = true;
      if (state == FORT)
	{
	  walkable = !flying;
	}
      TCODList<int> steps;
      for (int y2 = 0 - range; y2 <= range; y2++)
	{
	  for (int x2 = 0 - range; x2 <= range; x2++)
	    {
	      if (valid_pos(x1 + x2, y1 + y2, walkable))
		{
		  path->compute(x, y, x1 + x2, y1 + y2);
		  steps.push(path->size());
		}
	    }
	}
      int min = 1000, index;
      for (int i = 0; i < steps.size(); i++)
	{
	  if (min > steps.get(i))
	    {
	      min = steps.get(i);
	      index = i;
	    }
	}
      int index2 = 0;
      for (int y2 = 0 - range; y2 <= range; y2++)
	{
	  for (int x2 = 0 - range; x2 <= range; x2++)
	    {
	      if (valid_pos(x1 + x2, y1 + y2, walkable))
		{
		  if (index2 == index)
		    {
		      path->compute(x, y, x1 + x2, y1 + y2);
		      return true;
		    }
		  index2++;
		}
	    }
	}
    }
  else
    {
      path->compute(x, y, x1, y1);
      return true;
    }
  return false;
}

/**
 * Move mobile by one step in path.
 * @param time Length in seconds of the last rendered frame
 * @param mindex Index of current mob.
 */
void Mobile::movestep(float time, int mindex)
{
  if (!path->isEmpty())
    {
      if ((stats[0] == 0) && (state == FORT))
	{
	  return;
	}
      walktime -= time;
      if (walktime <= 0)
	{
	  if (state == FORT)
	    {
	      stats[0]--;
	    }
	  update_map_mob(x, y, NONE);
	  path->walk(&x, &y, true);
	  fovmap->setProperties(x, y, true, false);
	  if (targettype == MOB)
	    {
	      int x1, y1, index, range = 1;
	      path->getDestination(&x1, &y1);
	      if (state == FORT)
		{
		  index = find_order(work);
		  if ((fortorders[index].type == COMBAT) && (equipment[4] != NONE))
		    {
		      int windex = equipment[4];
		      range = fortobjs[windex].range;
		    }
		  if (index == NONE)
		    {
		      index = find_spell(work);
		      if (index != NONE)
			{
			  range = protospells[index].range;
			}
		    }
		  if ((abs(x1 - fortmobs[target].x) > range) || (abs(y1 - fortmobs[target].y) > range))
		    {
		      if (!generatepath(fortmobs[target].x, fortmobs[target].y, range))
			{
			  generatepath(x, y);
			}
		    }
		}
	      else
		{
		  if (equipment[4] != NONE)
		    {
		      int windex = equipment[4];
		      range = advobjs[windex].range;
		    }
		  index = find_spell(work);
		  if (index != NONE)
		    {
		      range = protospells[index].range;
		    }
		  if ((abs(x1 - advmobs[target].x) > range) || (abs(y1 - advmobs[target].y) > range))
		    {
		      if (!generatepath(advmobs[target].x, advmobs[target].y, range))
			{
			  generatepath(x, y);
			}
		    }
		}
	    }
	  update_map_mob(x, y, mindex);
	  if (state == FORT)
	    {
	      walktime = 1 - ((float)stats[5] / 100);
	    }
	  else
	    {
	      walktime = 0.5 - ((float)stats[5] / 200);
	    }
	  if ((stats[0] <= 0) && (type == ROBOT))
	    {
	      addMessage(new Message(name + " need refueling.", x, y));
	    }
	}
    }
  else if (targettype == MOB)
    {
      int x1, y1, index, range = 1;
      path->getDestination(&x1, &y1);
      if (state == FORT)
	{
	  index = find_order(work);
	  if ((fortorders[index].type == COMBAT) && (equipment[4] != NONE))
	    {
	      int windex = equipment[4];
	      range = fortobjs[windex].range;
	    }
	  if (index == NONE)
	    {
	      index = find_spell(work);
	      if (index != NONE)
		{
		  range = protospells[index].range;
		}
	    }
	  if ((abs(x1 - fortmobs[target].x) > range) || (abs(y1 - fortmobs[target].y) > range))
	    {
	      if (!generatepath(fortmobs[target].x, fortmobs[target].y))
		{
		  generatepath(x, y);
		}
	    }
	}
      else
	{
	  if (equipment[4] != NONE)
	    {
	      int windex = equipment[4];
	      range = advobjs[windex].range;
	    }
	  index = find_spell(work);
	  if (index != NONE)
	    {
	      range = protospells[index].range;
	    }
	  if ((abs(x1 - advmobs[target].x) > range) || (abs(y1 - advmobs[target].y) > range))
	    {
	      if (!generatepath(advmobs[target].x, advmobs[target].y))
		{
		  generatepath(x, y);
		}
	    }
	}
    }
}

/**
 * Gaining experience by mob.
 * @param amount Amount of experience points gained.
 */
void Mobile::gainexp(int amount)
{
  if (type != PLAYER)
    {
      return;
    }
  stats[11] += amount;
  int needexp = stats[10] * stats[12];
  if (stats[11] >= needexp)
    {
      stats[10]++;
      stats[11] -= needexp;
      stats[13] += 3;
    }
}

/**
 * Set working time of mob
 * @param time New value of time or length in seconds of the last rendered frame
 * @param stat Index of stat to use. Default -1.
 */
void Mobile::setworktime(float time, int stat)
{
  switch (stat)
    {
    case -2:
      worktime -= time;
      break;
    case -1:
      worktime = time;
      break;
    default:
      worktime = time - ((stats[stat] / 100) * time);
      break;
    }
}

/**
 * Remove mobile from game.
 */
void Mobile::deletemob()
{
  //Drop equipment
  int index;
  for (int i = 0; i < 5; i++)
    {
      index = equipment[i];
      if (index == NONE)
	{
	  continue;
	}
      if (state == FORT)
	{
	  fortobjs[index].setitem(x, y, index);
	}
      else
	{
	  advobjs[index].setitem(x, y, index);
	}
    }
  //Drop inventory
  for (int i = 0; i < inventory.size(); i++)
    {
      index = inventory.get(i);
      if (state == FORT)
	{
	  fortobjs[index].setitem(x, y, index);
	}
      else
	{
	  advobjs[index].setitem(x, y, index);
	}
    }
  //Drop loot
  int chance = TCODRandom::getInstance()->getInt(0, 100);
  if ((chance < 50) && (!loot.isEmpty()))
    {
      TCODList<int> *tmptypes = new TCODList<int>();
      for (int i = 0; i < loot.size(); i++)
	{
	  tmptypes->push(loot.get(i));
	  index = find_proto_obj("", tmptypes);
	  protoobjs[index].setitem(x, y);
	  tmptypes->clear();
	}
      delete tmptypes;
    }
  inventory.clear();
  delete path;
  x = NONE;
  y = NONE;
}

/**
 * Find proto mobile by name or type
 * @param name Name of object to find
 * @param type Type of object to find
 * @return Object index in protoobjs
 */
int find_proto_mob(std::string name, int type)
{
  int mob = -1;
  for (int i = 0; i < (int)protomobs.size(); i++)
    {
      if ((name.size() == 0) && (type != NONE))
	{
	  if (protomobs[i].type == type)
	    {
	      mob = i;
	      break;
	    }
	}
      if ((name.size() > 0) && (type == NONE))
	{
	  if (protomobs[i].name.compare(name) == 0)
	    {
	      mob = i;
	      break;
	    }
	}
      if ((name.size() > 0) && (type != NONE))
	{
	  if ((protomobs[i].name.compare(name) == 0) && (protomobs[i].type == type))
	    {
	      mob = i;
	      break;
	    }
	}
    }
  return mob;
}

/**
 * Find mobile by name or position.
 * @param name Name of mobile to find. Default is empty.
 * @param x X-axis position of mob. Default is -1.
 * @param y Y-axis position of mob. Default is -1.
 * @return Index of mobile in fortmobiles vector.
 * If not found, return -1.
 */
int find_mob(string name, int x, int y)
{
  int mob = -1;
  //Search by position
  if ((x != NONE) && (y != NONE))
    {
      mob = fortmap[y][x].mob;
      if (mob < 0)
	{
	  return NONE;
	}
      if ((name.size() > 0) && (name.compare(fortmobs[mob].name)))
	{
	  return NONE;
	}
      return mob;
    }
  for (int i = 0; i < (int)fortmobs.size(); i++)
    {
      //Search by name
      if ((name.size() > 0) && (name.compare(fortmobs[i].name)))
	{
	  continue;
	}
      mob = i;
      break;
    }
  return mob;
}

/**
 * Find all prototypes with selected type.
 * @param type Mobile type to find.
 * @param notlocation From which location mobiles should be avoid.
 * @param maxlevel Max level of mobs to find.
 * @return List of indexes of mobiles.
 */
TCODList<int> find_all_proto_mobs(int type, int notlocation, int maxlevel)
{
  TCODList<int> results;
  for (int i = 0; i < (int)protomobs.size(); i++)
    {
      //Search by type
      if (protomobs[i].type != type)
	{
	  continue;
	}
      //Search by location
      if (protomobs[i].location == notlocation)
	{
	  continue;
	}
      //Search by level
      if ((maxlevel != NONE) && (protomobs[i].stats[10] > maxlevel))
	{
	  continue;
	}
      results.push(i);
    }
  return results;
}

/**
 * Find nearest mobile from selected position
 * @param name Name of mob to find.
 * @param x X-axis of start search position.
 * @param y Y-axis of start search position.
 * @param radius Size of radius to search.
 * @param types Types of mob to find. Default empty.
 * @param fovradius Size of radius of field of view.
 * @param visible Search only for visible mobs. Default is true.
 * @return Index of mobile in fortmobs vector.
 * If mobile not found, return -1.
 */
int find_near_mob(std::string name, int x, int y, int radius, TCODList<int> *types, int fovradius, bool visible)
{
  int mob = NONE, index, sum, oldsum = 1000;
  if (visible)
    {
      fovmap->computeFov(x, y, fovradius);
    }
  for (int y1 = y - (radius + 1); y1 < y + (radius + 1); y1++)
    {
      for (int x1 = x - (radius + 1); x1 < x + (radius + 1); x1++)
	{
	  if ((y1 < 0) || (x1 < 0) || (y1 >= MAPHEIGHT) || (x1 >= MAPWIDTH))
	    {
	      continue;
	    }
	  sum = abs(x - x1) + abs(y - y1);
	  if (sum > oldsum)
	    {
	      continue;
	    }
	  if (state == FORT)
	    {
	      if (fortmap[y1][x1].mob == NONE)
		{
		  continue;
		}
	      index = fortmap[y1][x1].mob;
	      if ((name.size() > 0) && (name.compare(fortmobs[index].name)))
		{
		  continue;
		}
	      if ((!types->isEmpty()) && (!types->contains(fortmobs[index].type)))
		{
		  continue;
		}
	    }
	  else
	    {
	      if (advmap[y1][x1].mob == NONE)
		{
		  continue;
		}
	      index = advmap[y1][x1].mob;
	      if ((name.size() > 0) && (name.compare(advmobs[index].name)))
		{
		  continue;
		}
	      if ((!types->isEmpty()) && (!types->contains(advmobs[index].type)))
		{
		  continue;
		}
	    }
	  if ((visible) && (!fovmap->isInFov(x1, y1)))
	    {
	      continue;
	    }
	  mob = index;
	  oldsum  = sum;
	}
    }
  return mob;
}

/**
 * Find all mobiles with selected type
 * @param type Type of mobiles to find.
 * @return List of indexes of mobiles.
 */
TCODList<int> find_all_mobs(int type)
{
  TCODList<int> results;
  for (int i = 0; i < (int)fortmobs.size(); i++)
    {
      //Search by type
      if (fortmobs[i].type != type)
	{
	  continue;
	}
      results.push(i);
    }
  return results;
}
