/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#ifndef MOBILE_HPP
#define MOBILE_HPP
#include <string>
#include "libtcod.hpp"
#include "gui/gui.hpp"

/**
 * Contains mobile types.
 */
typedef enum MobileTypes
  {
    PLAYER,
    ROBOT,
    ANIMAL,
    MONSTER
  } MOBILETYPE;

typedef enum MobileLocations
  {
    FORTONLY,
    ADVENTUREONLY
  } MOBILELOCATION;

/**
 * Contains all informations and functions about mobiles.
 */
class Mobile
{
public:
  /**
   * Name of mobile
   */
  std::string name;
  /**
   * Type of mobile
   */
  int type;
  /**
   * X-axis position of mobile
   */
  int x;
  /**
   * Y-axis position of mobile
   */
  int y;
  /**
   * Char displayed on screen for mobile
   */
  char ch;
  /**
   * Char color for displaying on screen for mobile.
   */
  TCODColor color;
  /**
   * Mobile statistics
   */
  int stats[16];
  /**
   * Name of current mobile work
   */
  std::string work;
  /**
   * Current type of target of mobile
   */
  int targettype;
  /**
   * Target index
   */
  int target;
  /**
   * Path finder for mobile
   */
  TCODPath *path;
  /**
   * Working time for mobile
   */
  float worktime;
  /**
   * Index of item carried by mob
   */
  int carries;
  /**
   * If true, mob is used in order.
   */
  int taken;
  /**
   * Stage of order of mob.
   */
  int workstage;
  /**
   * Time before next step
   */
  float walktime;
  /**
   * Description of mobile.
   */
  std::string description;
  /**
   * Equipment of mob (for robots - upgrades)
   */
  int equipment[5];
  /**
   * AI's of mob 0 - movement, 1 - other NPC's, 2 - PC and robots.
   */
  int ais[3];
  /**
   * Inventory of mob
   */
  TCODList<int> inventory;
  /**
   * Where mobile can be meet (0 - fort, 1 - adventure, 2 - fort and adventure)
   */
  int location;
  /**
   * Loot available from mobile.
   */
  TCODList<int> loot;
  /**
   * Did mob can fly.
   */
  bool flying;
  /**
   * Spells known by mobile
   */
  TCODList<int> spells;
  
  Mobile();
  Mobile(char c, TCODColor col, int t);
  int addmob(bool start);
  void setmob(int x1, int y1, int mindex = -1, bool start = false);
  void generatestats(int newlevel = -1);
  void generatename();
  bool generatepath(int x1, int y1, int range = 1);
  void movestep(float time, int mindex);
  void gainexp(int amount);
  void setworktime(float time, int stat = -2);
  void deletemob();
};

int find_proto_mob(std::string name, int type = -1);
int find_mob(std::string name = "", int x = -1, int y = -1);
TCODList<int> find_all_proto_mobs(int type, int notlocation = -1, int maxlevel = -1);
int find_near_mob(std::string name, int x, int y, int radius, TCODList<int> *types, int fovradius, bool visible = true);
TCODList<int> find_all_mobs(int type);

#endif
