/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "main.hpp"
#include "adventure.hpp"
#include "mobileui.hpp"
#include <map>
#include <stdio.h>
#include "ui.hpp"

Container *mobsmenu = new Container(0, 0, SCREENWIDTH, SCREENHEIGHT);
Container *mobmenu = new Container(30, 10, 80, 70);
ToolBar *tmpbar = new ToolBar(65, 16, "Available items");
Container *mobinv = new Container(0, 0, SCREENWIDTH, SCREENHEIGHT);
ToolBar *inv = new ToolBar(2, 2, "Inventory");
ToolBar *available = new ToolBar(60, 2, "Available items");

/**
 * Build crew user interface.
 */
void buildMobileUI()
{
  mobsmenu->setVisible(false);
  mobmenu->setVisible(false);
  tmpbar->setVisible(false);
  mobinv->addWidget(available);
  mobinv->addWidget(inv);
  mobinv->setVisible(false);
}

/**
 * Update user stats
 * @param w Unused
 * @param data Value from button from UI.
 */
void advanceClb(Widget *w, void* data)
{
  int *index = static_cast<int*>(data);
  if (state == MOBINFO)
    {
      if (fortmobs[stage].stats[*index] == 50)
	{
	  return;
	}
      fortmobs[stage].stats[*index]++;
      fortmobs[stage].stats[13]--;
      if (*index == 9)
	{
	  fortmobs[stage].stats[8]++;
	  fortmobs[stage].stats[7]++;
	}
      else if (*index == 6)
	{
	  fortmobs[stage].stats[14]++;
	  fortmobs[stage].stats[15]++;
	}
    }
  else
    {
      if (advmobs[stage].stats[*index] == 50)
	{
	  return;
	}
      advmobs[stage].stats[*index]++;
      advmobs[stage].stats[13]--;
      if (*index == 9)
	{
	  advmobs[stage].stats[8]++;
	  advmobs[stage].stats[7]++;
	}
      else if (*index == 6)
	{
	  advmobs[stage].stats[14]++;
	  advmobs[stage].stats[15]++;
	}
    }
  showMobInfo(stage);
}

/**
 * Wear selected item
 * @param w Unused.
 * @param data Location and name of item to wear.
 */
void equip2Clb(Widget *w, void *data)
{
  string* p = (string*)data;
  size_t pos = p->find(';');
  string loc = p->substr(0, pos);
  pos++;
  string name = p->substr(pos);
  string locs[] = {"head", "arms", "torso", "legs", "weapon"};
  int index;
  for (int i = 0; i < 5; i++)
    {
      if (loc.compare(locs[i]) == 0)
	{
	  index = i;
	  break;
	}
    }
  int index2 = NONE;
  if (state == MOBINFO)
    {
      int index3, x, y;
      index2 = find_obj(name);
      //Remove old equipment
      if (fortmobs[stage].equipment[index] != NONE)
	{
	  index3 = fortmobs[stage].equipment[index];
	  fortobjs[index3].setitem(fortmobs[stage].x, fortmobs[stage].y, index3);
	}
      fortmobs[stage].equipment[index] = index2;
      //Put new equipment
      if (index2 != NONE)
	{
	  x = fortobjs[index2].x;
	  y = fortobjs[index2].y;
	  update_map_item(x, y, index2, true);
	  fortobjs[index2].useitem();
	}
    }
  else
    {
      for (int i = 0; i < (int)advobjs.size(); i++)
	{
	  if (advobjs[i].name.compare(name) == 0)
	    {
	      index2 = i;
	      break;
	    }
	}
      if (advmobs[stage].equipment[index] != NONE)
	{
	  advmobs[stage].inventory.push(advmobs[stage].equipment[index]);
	}
      advmobs[stage].equipment[index] = index2;
    }
  tmpbar->setVisible(false);
  showMobInfo(stage);
}

/**
 * Show available equipment for selected location
 * @param w Unused.
 * @param data Name of location to show.
 */
void equipClb(Widget *w, void *data)
{
  string* p = (string*)data;
  tmpbar->clear();
  if (!tmpbar->isVisible())
    {
      tmpbar->setVisible(true);
    }
  tmpbar->addWidget(new Button("None", NULL, equip2Clb, new string(*p + ";NONE")));
  TCODList<int> tmptypes;
  if (fortmobs[stage].type == PLAYER)
    {
      if (p->compare("head") == 0)
	{
	  tmptypes.push(HELMET);
	}
      else if (p->compare("arms") == 0)
	{
	  tmptypes.push(BRACERS);
	}
      else if (p->compare("torso") == 0)
	{
	  tmptypes.push(ARMOR);
	}
      else if (p->compare("legs") == 0)
	{
	  tmptypes.push(GREAVES);
	}
    }
  else
    {
      if (p->compare("head") == 0)
	{
	  tmptypes.push(HUPGRADE);
	}
      else if (p->compare("arms") == 0)
	{
	  tmptypes.push(AUPGRADE);
	}
      else if (p->compare("torso") == 0)
	{
	  tmptypes.push(TUPGRADE);
	}
      else if (p->compare("legs") == 0)
	{
	  tmptypes.push(LUPGRADE);
	}
    }
  if (p->compare("weapon") == 0)
    {
      tmptypes.push(WEAPON);
    }
  if (state == MOBINFO)
    {
      TCODList<int> result = find_all_obj(tmptypes);
      if (result.size() > 0)
	{
	  for (int i = 0; i < result.size(); i++)
	    {
	      tmpbar->addWidget(new Button(fortobjs[result.get(i)].name.c_str(), NULL, equip2Clb, new string(*p + ";" + fortobjs[result.get(i)].name)));
	    }
	}
    }
  else
    {
      if (!advmobs[stage].inventory.isEmpty())
	{
	  int index;
	  for (int i = 0; i < advmobs[stage].inventory.size(); i++)
	    {
	      index = advmobs[stage].inventory.get(i);
	      tmpbar->addWidget(new Button(advobjs[index].name.c_str(), NULL, equip2Clb, new string(*p + ";" + advobjs[index].name)));
	    }
	}
    }
  tmpbar->computeSize();
  showMobInfo(stage);
}

/**
 * Rename selected mob
 * @param wid Unused.
 * @param val New name of mob.
 * @param data Index of mob.
 */
void renameClb(Widget *wid, char *val, void *data)
{
  int *index = static_cast<int*>(data);
  string *value = new string(val);
  if (value->size() == 0)
    {
      return;
    }
  if (state == MOBINFO)
    {
      fortmobs[*index].name = *value;
    }
  else
    {
      advmobs[*index].name = *value;
    }
}

/**
 * Add item to inventory
 * @param w Unused.
 * @param data Name of item to add to inventory.
 */
void addinvClb(Widget *w, void *data)
{
  if (fortmobs[stage].inventory.size() == 20)
    {
      return;
    }
  string *name = static_cast<string*>(data);
  int index = find_obj(*name);
  int x = fortobjs[index].x;
  int y = fortobjs[index].y;
  update_map_item(x, y, index, true);
  fortobjs[index].stored = false;
  fortobjs[index].useitem();
  fortmobs[stage].inventory.push(index);
  drawItems(stage);
}

/**
 * Remove item from inventory
 * @param w Unused
 * @param data Index of item to remove
 */
void removeinvClb(Widget *w, void *data)
{
  int *index = static_cast<int*>(data);
  if (state == MOBINV)
    {
      fortobjs[*index].setitem(fortmobs[stage].x, fortmobs[stage].y, *index);
      fortmobs[stage].inventory.remove(*index);
    }
  else
    {
      advobjs[*index].setitem(advmobs[stage].x, advmobs[stage].y, *index);
      advmobs[stage].inventory.remove(*index);
    }
  drawItems(stage);
}

/**
 * Draw all items which can be put to inventory.
 * @param index Index of mob.
 */
void drawItems(int index)
{
  mobmenu->setVisible(false);
  mobinv->setVisible(true);
  inv->clear();
  inv->addWidget(new Label(0, 0, ""));
  int line = 4;
  if (state == MOBINV)
    {
      available->setVisible(true);
      available->clear();
      available->addWidget(new Label(0, 0, ""));
      map<string, int> items;
      for (int i = 0; i < (int)fortobjs.size(); i++)
	{
	  if ((fortobjs[i].x > -1) && (fortobjs[i].y > -1) && (fortobjs[i].types.contains(OBJECT)) && (fortobjs[i].stored))
	    {
	      if (items.find(fortobjs[i].name) == items.end())
		{
		  items[fortobjs[i].name] =  0;
		}
	      items[fortobjs[i].name] ++;
	    }
	}
      //Show all available items
      char* c;
      if (!items.empty())
	{
	  for (map<string, int>::iterator i = items.begin(); i != items.end(); i++)
	    {
	      c = new char[30];
	      sprintf(c, "%s: %d", (*i).first.c_str(), (*i).second);
	      available->addWidget(new Button(c, "Add item to inventory", addinvClb, new string((*i).first)));
	      line ++;
	    }
	}
      available->addWidget(new Label(0, line, ""));
      available->computeSize();
    }
  else
    {
      available->setVisible(false);
    }
  //Show player inventory
  int index2;
  line = 4;
  if (state == MOBINV)
    {
      if (!fortmobs[index].inventory.isEmpty())
	{
	  for (int i = 0; i < fortmobs[index].inventory.size(); i++)
	    {
	      index2 = fortmobs[index].inventory.get(i);
	      inv->addWidget(new Button(fortobjs[index2].name.c_str(), "Remove item from inventory", removeinvClb, new int(index2)));
	      line++;
	    }
	}
    }
  else
    {
      if (!advmobs[index].inventory.isEmpty())
	{
	  for (int i = 0; i < advmobs[index].inventory.size(); i++)
	    {
	      index2 = advmobs[index].inventory.get(i);
	      inv->addWidget(new Button(advobjs[index2].name.c_str(), "Remove item from inventory", removeinvClb, new int(index2)));
	      line++;
	    }
	}
    }
  inv->addWidget(new Label(0, line, ""));
  inv->computeSize();
  mobinv->addWidget(inv);
}

/**
 * Manage player inventory
 * @param w Unused.
 * @param data Index of player.
 */
void inventoryClb(Widget *w, void *data)
{
  int *index = static_cast<int*>(data);
  stage = *index;
  if (state == MOBINFO)
    {
      state = MOBINV;
    }
  else
    {
      state = MOBINV2;
    }
  drawItems(*index);
}

/**
 * Show informations about selected mobile.
 * @param index Index of selected mobile.
 * @return Index of selected mobile.
 */
int showMobInfo(int index)
{
  if (index == NONE)
    {
      mobmenu->setVisible(false);
      mobinv->setVisible(false);
      return NONE;
    }
  mobinv->setVisible(false);
  mobsmenu->setVisible(false);
  int line = 18;
  mobmenu->clear();
  mobmenu->setVisible(true);
  Mobile *mob = new Mobile();
  if (state == MOBINFO)
    {
      mob = &fortmobs[index];
    }
  else
    {
      mob = &advmobs[index];
    }
  //Show mob stats
  TextBox *txtbox = new TextBox(10, 16, 15, 20, "Name: ", mob->name.c_str(), NULL);
  txtbox->setCallback(renameClb, new int(index));
  mobmenu->addWidget(txtbox);
  char* c;
  if (mob->type == PLAYER)
    {
      c = new char[10];
      mobmenu->addWidget(new Label(10, line, "Level:"));
      sprintf(c, "%d", mob->stats[10]);
      mobmenu->addWidget(new Label(22, line, c));
      c = new char[10];
      sprintf(c, "%d", mob->stats[11]);
      mobmenu->addWidget(new Label(10, line + 2, "Exp:"));
      mobmenu->addWidget(new Label(22, line + 2, c));
      mobmenu->addWidget(new Label(10, line + 4, "Points:"));
      c = new char[10];
      sprintf(c, "%d", mob->stats[13]);
      mobmenu->addWidget(new Label(22, line + 4, c));
      line = 24;
    }
  c = new char[10];
  int value = mob->stats[0];
  if (value < 0)
    {
      value = 0;
    }
  sprintf(c, "%d/%d", value, mob->stats[1]);
  mobmenu->addWidget(new Label(10, line, "Energy:"));
  mobmenu->addWidget(new Label(22, line, c));
  c = new char[10];
  sprintf(c, "%d/%d", mob->stats[7], mob->stats[8]);
  mobmenu->addWidget(new Label(10, line + 2, "Hit Points:"));
  mobmenu->addWidget(new Label(22, line + 2, c));
  if (mob->type == PLAYER)
    {
      mobmenu->addWidget(new Label(10, line + 4, "Mana Points:"));
      c = new char[10];
      sprintf(c, "%d/%d", mob->stats[14], mob->stats[15]);
      mobmenu->addWidget(new Label(22, line + 4, c));
      line = 26;
    }
  mobmenu->addWidget(new Label(10, line + 4, "Order:"));
  mobmenu->addWidget(new Label(22, line + 4, mob->work.c_str()));
  if ((mob->type == PLAYER) && (mob->stats[13] > 0))
    {
      mobmenu->addWidget(new Button(7, 32, 2, 1, "+", NULL, advanceClb, new int(2)));
      mobmenu->addWidget(new Button(7, 34, 2, 1, "+", NULL, advanceClb, new int(3)));
      mobmenu->addWidget(new Button(7, 36, 2, 1, "+", NULL, advanceClb, new int(4)));
      mobmenu->addWidget(new Button(7, 38, 2, 1, "+", NULL, advanceClb, new int(5)));
      mobmenu->addWidget(new Button(7, 40, 2, 1, "+", NULL, advanceClb, new int(6)));
      mobmenu->addWidget(new Button(7, 42, 2, 1, "+", NULL, advanceClb, new int(9)));
    }
  c = new char[4];
  sprintf(c, "%d", mob->stats[2]);
  mobmenu->addWidget(new Label(22, 32, c));
  c = new char[4];
  sprintf(c, "%d", mob->stats[3]);
  mobmenu->addWidget(new Label(22, 34, c));
  c = new char[4];
  sprintf(c, "%d", mob->stats[4]);
  mobmenu->addWidget(new Label(22, 36, c));
  c = new char[4];
  sprintf(c, "%d", mob->stats[5]);
  mobmenu->addWidget(new Label(22, 38, c));
  c = new char[4];
  sprintf(c, "%d", mob->stats[6]);
  mobmenu->addWidget(new Label(22, 40, c));
  mobmenu->addWidget(new Label(10, 32, "Strenght:"));
  mobmenu->addWidget(new Label(10, 34, "Dexterity:"));
  mobmenu->addWidget(new Label(10, 36, "Senses:"));
  mobmenu->addWidget(new Label(10, 38, "Speed:"));
  mobmenu->addWidget(new Label(10, 40, "Memory:"));
  if (mob->type == PLAYER)
    {
      mobmenu->addWidget(new Label(10, 42, "Condition:"));
      c = new char[4];
      sprintf(c, "%d", mob->stats[9]);
      mobmenu->addWidget(new Label(22, 42, c));
    }
  //Show mob equipment/upgrades
  mobmenu->addWidget(new Label(35, 16, "Head:"));
  mobmenu->addWidget(new Label(35, 18, "Arms:"));
  mobmenu->addWidget(new Label(35, 20, "Torso:"));
  mobmenu->addWidget(new Label(35, 22, "Legs:"));
  mobmenu->addWidget(new Label(35, 24, "Weapon:"));
  if (mob->type == PLAYER)
    {
      mobmenu->addWidget(new Button(35, 28, 10, 1, "Inventory", "Show player inventory", inventoryClb, new int(index)));
    }
  line = 16;
  string names[] = {"head", "arms", "torso", "legs", "weapon"};
  int index2;
  for (int i = 0; i < 5; i++)
    {
      index2 = mob->equipment[i];
      if (index2 == NONE)
	{
	  mobmenu->addWidget(new Button(48, line, 10, 1, "None", "Assign item", equipClb, new string(names[i])));
	}
      else
	{
	  if (state == MOBINFO)
	    {
	      mobmenu->addWidget(new Button(48, line, 10, 1, fortobjs[index2].name.c_str(), "Change item", equipClb, new string(names[i])));
	    }
	  else
	    {
	      mobmenu->addWidget(new Button(48, line, 10, 1, advobjs[index2].name.c_str(), "Change item", equipClb, new string(names[i])));
	    }
	}
      line += 2;
    }
  return index;
}

/**
 * Show information about selected mobile.
 * @param w Unused.
 * @param data Selected mob name.
 */
void mobClb(Widget *w, void* data)
{
  int *index = static_cast<int*>(data);
  if (state == CREW)
    {
      state = MOBINFO;
    }
  else
    {
      setVisibleAdvmenu(false);
      state = MOBINFO2;
    }
  stage = showMobInfo(*index);
}

/**
 * Show/hide information about fort crew.
 * @param visible If true, show fort crew, otherwise hide menu.
 */
void setVisibleCrew(bool visible)
{
  if (visible)
    { 
      mobmenu->setVisible(false);
      mobsmenu->clear();
      mobsmenu->addWidget(new Label(7, 2, "Name"));
      mobsmenu->addWidget(new Label(30, 2, "Type"));
      mobsmenu->addWidget(new Label(45, 2, "Energy"));
      mobsmenu->addWidget(new Label(57, 2, "Hit Points"));
      mobsmenu->addWidget(new Label(70, 2, "Order"));
      int line = 4, j = 0, value;
      char* c;
      char* c2;
      for (int i = 0; i < (int)fortmobs.size(); i++)
	{
	  c = new char[10];
	  c2 = new char[10];
	  if ((fortmobs[i].type != PLAYER) && (fortmobs[i].type != ROBOT))
	    {
	      continue;
	    }
	  mobsmenu->addWidget(new Button(7, line + j, 10, 1, fortmobs[i].name.c_str(), NULL, mobClb, new int(i)));
	  if (fortmobs[i].type == PLAYER)
	    {
	      mobsmenu->addWidget(new Label(30, line + j, "YOU"));
	    }
	  else
	    {
	      mobsmenu->addWidget(new Label(30, line + j, "Steambot"));
	    }
	  value = fortmobs[i].stats[0];
	  if (value < 0)
	    {
	      value = 0;
	    }
	  sprintf(c, "%d/%d", value, fortmobs[i].stats[1]);
	  mobsmenu->addWidget(new Label(45, line + j, c));
	  sprintf(c2, "%d/%d", fortmobs[i].stats[7], fortmobs[i].stats[8]);
	  mobsmenu->addWidget(new Label(57, line + j, c2));
	  mobsmenu->addWidget(new Label(70, line + j, fortmobs[i].work.c_str()));
	  j++;
	}
    }
  mobsmenu->setVisible(visible);
  escinfo->setVisible(visible);
  tmpbar->setVisible(false);
}

/**
 * Use selected item
 * @param mob Mobile which use item
 * @param item Item which been use
 * @param iindex Index of item
 */
void useitem(Mobile *mob, Item *item, int iindex)
{
  if (item->types.contains(HPOTION))
    {
      mob->stats[7] += item->data;
      if (mob->stats[7] > mob->stats[8])
	{
	  mob->stats[7] = mob->stats[8];
	}
    }
  else if (item->types.contains(MPOTION))
    {
      mob->stats[14] += item->data;
      if (mob->stats[14] > mob->stats[15])
	{
	  mob->stats[14] = mob->stats[15];
	}
    }
  else if (item->types.contains(EPOTION))
    {
      mob->stats[0] += item->data;
      if (mob->stats[0] > mob->stats[1])
	{
	  mob->stats[0] = mob->stats[1];
	}
    }
  item->use --;
  if (item->use == 0)
    {
      mob->inventory.remove(iindex);
      item->deleteitem();
    }
}

/**
 * Use selected item.
 * @param w Unused.
 * @param data Index of item to use.
 */
void useitemClb(Widget *w, void *data)
{
  int *p = static_cast<int*>(data);
  setVisibleUse(false);
  if (state == USE)
    {
      useitem(&fortmobs[0], &fortobjs[*p], *p);
      setVisibleGamemenu(true);
      state = FORT;
    }
  else
    {
      setVisibleAdvmenu(true);
      state = ADVENTURE;
    }
}

/**
 * Show/hide items which can be used
 * @param visible If true, show items which can be used, otherwise hide menu
 */
void setVisibleUse(bool visible)
{
  if (visible)
    {
      mobsmenu->clear();
      mobsmenu->addWidget(new Label(7, 2, "Name"));
      mobsmenu->addWidget(new Label(45, 2, "Use"));
      int line = 4, j = 0;
      char *c;
      if (state == USE)
	{
	  for (int i = 0; i < fortmobs[0].inventory.size(); i++)
	    {
	      if (!fortobjs[fortmobs[0].inventory.get(i)].types.contains(USEABLE))
		{
		  continue;
		}
	      mobsmenu->addWidget(new Button(7, line + j, 30, 1, fortobjs[fortmobs[0].inventory.get(i)].name.c_str(), fortobjs[fortmobs[0].inventory.get(i)].description.c_str(), useitemClb, new int(fortmobs[0].inventory.get(i))));
	      c = new char[4];
	      sprintf(c, "%d", fortobjs[fortmobs[0].inventory.get(i)].use);
	      mobsmenu->addWidget(new Label(45, line + j, c));
	      j++;
	    }
	}
    }
  mobsmenu->setVisible(visible);
  escinfo->setVisible(visible);
}
