/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include <string>
#include <vector>
#include <algorithm>
#include <iostream>
#include "libtcod.hpp"
#include "order.hpp"
#include "mobile.hpp"
#include "item.hpp"
#include "main.hpp"
#include "utils.hpp"
#include "gui/gui.hpp"
#include "ui.hpp"
#include <stdio.h>
#include "message.hpp"

Container *ordersmenu = new Container(0, 0, SCREENWIDTH, SCREENHEIGHT);
Container *ordermenu = new Container(0, 0, SCREENWIDTH, SCREENHEIGHT);
ToolBar *assigned = new ToolBar(2, 2, "Assigned");
ToolBar *unassigned = new ToolBar(60, 2, "Unassigned");

/**
 * Standard class constructor. Create order and fill
 * some default variables.
 */
Order::Order()
{
  active = 'N';
  for (int i = 0; i < STEPS; i++)
    {
      for (int j = 0; j < STEPLENGTH; j++)
	{
	  stages[i][j] = -1;
	}
    }
  automatic = false;
  curamount = NONE;
  enabled = true;
  curstage = 0;
  playeronly = false;
  times = 1;
}

/**
 * Build order user interface.
 */
void buildOrderUI()
{
  ordersmenu->setVisible(false);
  ordermenu->addWidget(assigned);
  ordermenu->addWidget(unassigned);
  ordermenu->setVisible(false);
}

/**
 * Assign selected worker to order.
 * @param w Unused.
 * @param data Name of order to edit.
 */
void assignClb(Widget *w, void* data)
{
  string* p = (string*)data;
  size_t pos = p->find(';');
  string work = p->substr(0, pos);
  pos++;
  string option = p->substr(pos);
  int index = find_order(work);
  int index2 = find_mob(option);
  fortorders[index].workers.push_back(index2);
  if (fortmobs[index2].type != PLAYER)
    {
      fortmobs[index2].work = "Assigned";
    }
  drawWorkers(index);
}

/**
 * Remove selected worker from order.
 * @param w Unused.
 * @param data Name of order to edit.
 */
void removeClb(Widget *w, void* data)
{
  string* p = (string*)data;
  size_t pos = p->find(';');
  string work = p->substr(0, pos);
  pos++;
  string option = p->substr(pos);
  int index = find_order(work);
  int index2 = find_mob(option);
  fortmobs[index2].work = "IDLE";
  fortmobs[index2].worktime = 0;
  vector<int> tmpvector;
  for (int i = 0; i < (int)fortorders[index].workers.size(); i++)
    {
      if (fortorders[index].workers[i] == index2)
	{
	  continue;
	}
      tmpvector.push_back(fortorders[index].workers[i]);
    }
  fortorders[index].workers = tmpvector;
  drawWorkers(index);
}

/**
 * Draw available/assigned workers for selected orders.
 * @param index Index of order.
 */
void drawWorkers(int index)
{
  assigned->clear();
  unassigned->clear();
  int index2;
  assigned->addWidget(new Label(2, 2, ""));
  unassigned->addWidget(new Label(60, 2, ""));
  //Show all assigned workers
  int line = 3;
  if (fortorders[index].workers.size() > 0)
    {
      for (int i = 0; i < (int)fortorders[index].workers.size(); i++)
	{
	  index2 = fortorders[index].workers[i];
	  assigned->addWidget(new Button(fortmobs[index2].name.c_str(), NULL, removeClb, new string(fortorders[index].name + ";" + fortmobs[index2].name)));
	  line++;
	}
    }
  assigned->addWidget(new Label(2, line, ""));
  assigned->computeSize();
  line = 3;
  //Show all available workers
  if ((fortorders[index].automatic) || (fortorders[index].playeronly))
    {
      bool plassigned = false;
      for (int i = 0; i < (int)fortorders[index].workers.size(); i++)
	{
	  if (fortorders[index].workers[i] == 0)
	    {
	      plassigned = true;
	      break;
	    }
	}
      if (!plassigned)
	{
	  unassigned->addWidget(new Button(fortmobs[0].name.c_str(), NULL, assignClb, new string(fortorders[index].name + ";" + fortmobs[0].name)));
	}
    }
  if (!fortorders[index].playeronly)
    {
      for (int i = 1; i < (int)fortmobs.size(); i++)
	{
	  if (fortmobs[i].type != ROBOT)
	    {
	      continue;
	    }
	  if (fortmobs[i].work.compare("IDLE"))
	    {
	      continue;
	    }
	  unassigned->addWidget(new Button(fortmobs[i].name.c_str(), NULL, assignClb, new string(fortorders[index].name + ";" + fortmobs[i].name)));
	  line++;
	}
    }
  unassigned->addWidget(new Label(60, line, ""));
  unassigned->computeSize();
}

/**
 * Add/remove workers from order.
 * @param w Unused.
 * @param data Name of order to edit.
 */
void workersClb(Widget *w, void* data)
{
  string* p = (string*)data;
  int index = find_order(*p);
  state = ORDER;
  ordersmenu->setVisible(false);
  ordermenu->setVisible(true);
  drawWorkers(index);
}

/**
 * Set some options for selected order.
 * @param w Unused.
 * @param data Selected order name and option to change.
 */
void orderClb(Widget *w, void* data)
{
  string* p = (string*)data;
  size_t pos = p->find(';');
  string work = p->substr(0, pos);
  pos++;
  string option = p->substr(pos);
  int index = find_order(work);
  if (option.compare("activate") == 0)
    {
      fortorders[index].active = 'Y';
    }
  else if (option.compare("deactivate") == 0)
    {
      fortorders[index].active = 'N';
    }
  setVisibleOrders(true);
}

/**
 * Set how many times order will be repeated.
 * @param wid Widget which sent callback.
 * @param val New value of widget.
 * @param data Name of order to set.
 */
void timesClb(Widget *wid, char *val, void *data)
{
  string* p = (string*)data;
  string *value = new string(val);
  TextBox *box = (TextBox*)wid;
  if (value->size() == 0)
    {
      box->setText("1");
      return;
    }
  delete value;
  int newval = atoi(val);
  if (newval < 1)
    {
      newval = 1;
    }
  int index = find_order(*p);
  fortorders[index].times = newval;
  char *c = new char[4];
  sprintf(c, "%d", newval);
  box->setText(c);
}

/**
 * Show group of available orders.
 * @param type Type of orders
 * @param line Line on screen to draw
 * @param label Name of orders type
 * @param oldline Old value for line.
 */
void show_orders_group(int type, int *line, const char* label, int *oldline)
{
  if (*line > SCREENHEIGHT - 5)
    {
      return;
    }
  char *c;
  if (*line > 3)
    {
      ordersmenu->addWidget(new Label(2, *line, label));
    }
  *line = *line + 1;
  *oldline = *line;
  int index;
  TCODList<int> *tmplist = new TCODList<int>();
  if (type == CRAFT)
    {
      tmplist->push(WORKSHOP);
      tmplist->push(ARMORER);
      tmplist->push(WEAPONSMITH);
      tmplist->push(CARPENTER);
      tmplist->push(LEATHERWORK);
      tmplist->push(SMELTER);
      tmplist->push(ALCHEMIST);
    }
  else if (type == RESEARCH)
    {
      tmplist->push(LABORATORY);
      tmplist->push(LIBRARY);
    }
  for (int i = 0; i < (int)fortorders.size(); i++)
    {
      if ((fortorders[i].active == 'Y') || (fortorders[i].type != type) || (!fortorders[i].enabled))
	{
	  continue;
	}
      if (*line < 4)
	{
	  *line = *line + 1;
	  continue;
	}
      if (type == CRAFT)
	{
	  index = find_recipe2(fortorders[i].name.substr(5), tmplist);
	  ordersmenu->addWidget(new Button(3, *line, 1, 1, "?", fortrecipes[index].materials.c_str(), NULL, NULL));
	}
      else if (type == RESEARCH)
	{
	  index = find_recipe2(fortorders[i].name.substr(9), tmplist);
	  ordersmenu->addWidget(new Button(3, *line, 1, 1, "?", fortrecipes[index].materials.c_str(), NULL, NULL));
	}
      ordersmenu->addWidget(new Label(7, *line, fortorders[i].name.c_str()));
      c = new char[4];
      sprintf(c, "%c", fortorders[i].active);
      ordersmenu->addWidget(new Button(47, *line, 10, 1, c, "Activate selected order.", orderClb, new string(fortorders[i].name + ";activate")));
      if (fortorders[i].infinite)
	{
	  c = new char[4];
	  sprintf(c, "%d", fortorders[i].times);
	  TextBox *txtbox = new TextBox(62, *line, 3, 3, "", c, "How many times order been repeated.");
	  txtbox->setCallback(timesClb, new string(fortorders[i].name));
	  ordersmenu->addWidget(txtbox);	      
	}
      else
	{
	  ordersmenu->addWidget(new Label(62, *line, "N/A"));
	}
      c = new char[4];
      sprintf(c, "%d", (int)fortorders[i].workers.size());
      ordersmenu->addWidget(new Button(77, *line, 10, 1, c, "Add or remove workers to order.", workersClb, new string(fortorders[i].name)));
      *line = *line + 1;
      if (*line > SCREENHEIGHT - 5)
	{
	  return;
	}
    }
  delete tmplist;
  *line = *line + 1;
  if (*line > SCREENHEIGHT - 5)
    {
      return;
    }
  if (*line == *oldline + 1)
    {
      ordersmenu->addWidget(new Label(7, *oldline, "NONE"));
      *line = *line + 1;
    }
}

/**
 * Show all active orders
 * @param line Line on screen to draw
 * @param oldline Old value of line.
 */
void show_orders_active(int *line, int *oldline)
{
  if (*line > SCREENHEIGHT - 5)
    {
      return;
    }
  char *c;
  if (*line > 3)
    {
      ordersmenu->addWidget(new Label(2, 3, "[Active orders]"));
    }
  for (int i = 0; i < (int)fortorders.size(); i++)
    {
      if (fortorders[i].active == 'N')
	{
	  continue;
	}
      if (*line < 4)
	{
	  *line = *line + 1;
	  continue;
	}
      ordersmenu->addWidget(new Label(7, *line, fortorders[i].name.c_str()));
      ordersmenu->addWidget(new Button(47, *line, 10, 1, "Y", "Deactivate order", orderClb, new string(fortorders[i].name + ";deactivate")));
      if (fortorders[i].infinite)
	{
	  c = new char[4];
	  sprintf(c, "%d", fortorders[i].times);
	  TextBox *txtbox = new TextBox(62, *line, 3, 3, "", c, "How many times order been repeated.");
	  txtbox->setCallback(timesClb, new string(fortorders[i].name));
	  ordersmenu->addWidget(txtbox);	      
	}
      else
	{
	  ordersmenu->addWidget(new Label(62, *line, "N/A"));
	}
      c = new char[4];
      sprintf(c, "%d", (int)fortorders[i].workers.size());
      ordersmenu->addWidget(new Button(77, *line, 10, 1, c, "Add or remove workers to order.", workersClb, new string(fortorders[i].name)));
      *line = *line + 1;
      if (*line > SCREENHEIGHT - 5)
	{
	  return;
	}
    }
  *line = *line + 1;
  if (*line > SCREENHEIGHT - 5)
    {
      return;
    }
  if (*line == *oldline + 1)
    {
      ordersmenu->addWidget(new Label(7, *oldline, "NONE"));
      *line = *line + 1;
    }
}

/**
 * Show all available basic orders.
 * @param line Line on screen to draw
 * @param oldline Old value of line.
 */
void show_orders_basic(int *line, int *oldline)
{
  if (*line > SCREENHEIGHT - 5)
    {
      return;
    }
  char *c;
  if (*line > 3)
    {
      ordersmenu->addWidget(new Label(2, *line, "[Basic orders]"));
    }
  *line = *line + 1;
  for (int i = 0; i < (int)fortorders.size(); i++)
    {
      if ((fortorders[i].active == 'Y') || (fortorders[i].type == OBUILD) || (fortorders[i].type == WORK) || (fortorders[i].type == CRAFT) || (fortorders[i].type == RESEARCH) || (fortorders[i].type == COMBAT) || (!fortorders[i].enabled))
	{
	  continue;
	}
      if (*line < 4)
	{
	  *line = *line + 1;
	  continue;
	}
      ordersmenu->addWidget(new Label(7, *line, fortorders[i].name.c_str()));
      c = new char[4];
      sprintf(c, "%c", fortorders[i].active);
      ordersmenu->addWidget(new Button(47, *line, 10, 1, c, "Activate selected order", orderClb, new string(fortorders[i].name + ";activate")));
      if (fortorders[i].infinite)
	{
	  c = new char[4];
	  sprintf(c, "%d", fortorders[i].times);
	  TextBox *txtbox = new TextBox(62, *line, 3, 3, "", c, "How many times order been repeated.");
	  txtbox->setCallback(timesClb, new string(fortorders[i].name));
	  ordersmenu->addWidget(txtbox);	      
	}
      else
	{
	  ordersmenu->addWidget(new Label(62, *line, "N/A"));
	}
      c = new char[4];
      sprintf(c, "%d", (int)fortorders[i].workers.size());
      ordersmenu->addWidget(new Button(77, *line, 10, 1, c, "Add or remove workers to order.", workersClb, new string(fortorders[i].name)));
      *line = *line + 1;
      if (*line > SCREENHEIGHT - 5)
	{
	  return;
	}
    }
  *line = *line + 1;
}

/**
 * Show earlier page of orders
 * @param w Unused.
 * @param data Start element.
 */
void pageClb(Widget *w, void *data)
{
  int *option = static_cast<int*>(data);
  if (*option < 0)
    {
      *option = 0;
    }
  show_orders(*option);
}

/**
 * Show orders menu.
 * @param start From which element starts. Default 0.
 */
void show_orders(int start)
{
  ordermenu->setVisible(false);
  ordersmenu->clear();
  ordersmenu->addWidget(new Label(7, 2, "Name"));
  ordersmenu->addWidget(new Label(47, 2, "Active"));
  ordersmenu->addWidget(new Label(62, 2, "Repeat"));
  ordersmenu->addWidget(new Label(77, 2, "Workers"));
  int line = 4 - start, oldline = 4 - start;
  //List all active orders
  show_orders_active(&line, &oldline);
  //List all basic orders
  show_orders_basic(&line, &oldline);
  //List all combat orders
  show_orders_group(COMBAT, &line, "[Combat]", &oldline);
  //List all build orders
  show_orders_group(OBUILD, &line, "[Build]", &oldline);
  //List all mining orders
  show_orders_group(WORK, &line, "[Mining]", &oldline);
  //List all crafting orders
  show_orders_group(CRAFT, &line, "[Crafting]", &oldline);
  //List all research orders
  show_orders_group(RESEARCH, &line, "[Researching]", &oldline);
  if (line > SCREENHEIGHT - 5)
    {
      ordersmenu->addWidget(new Button(77, line, 10, 1, "Next>>", "Go to next page.", pageClb, new int(start + line - 1)));
    }
  if (start > 0)
    {
      ordersmenu->addWidget(new Button(7, SCREENHEIGHT - 4, 10, 1, "<<Prev", "Go to previous page.", pageClb, new int(start - SCREENHEIGHT - 5)));
    }
}

/**
 * Show/hide information about fort orders.
 * @param visible If true, show orders, otherwise hide menu.
 */
void setVisibleOrders(bool visible)
{
  if (visible)
    {
      show_orders();
    }
  ordersmenu->setVisible(visible);
  escinfo->setVisible(visible);
}

/**
 * Set all worker information for order with item target.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @param stage Stage of order
 * @return true if everything is properly set, otherwise false.
 */
bool set_order_item(int mindex, int oindex, int stage)
{
  int result = NONE, target = NONE;
  if (fortorders[oindex].type == REFUEL)
    {
      for (int k = 1; k < (int)fortmobs.size(); k++)
	{
	  if ((fortmobs[k].type != ROBOT) || (fortmobs[k].stats[0] > 0) || (k == mindex))
	    {
	      continue;
	    }
	  if ((fortmobs[k].taken == NONE) || (fortmobs[k].taken == mindex))
	    {
	      result = k;
	      fortmobs[k].taken = mindex;
	      break;
	    }
	}
      if (result == NONE)
	{
	  if (mindex == 0)
	    {
	      fortmobs[0].work = "IDLE";
	    }
	  return false;
	}
    }
  TCODList<int> tmptypes;
  tmptypes.push(fortorders[oindex].stages[stage][1]);
  if (fortorders[oindex].stages[stage][10] != NONE)
    {
      tmptypes.push(fortorders[oindex].stages[stage][10]);
    }
  if (fortorders[oindex].stages[stage][11] != NONE)
    {
      tmptypes.push(fortorders[oindex].stages[stage][11]);
    }
  if (fortorders[oindex].type == GATHER)
    {
      target = find_near_obj("", fortmobs[mindex].x, fortmobs[mindex].y, 60, &tmptypes, false);
    }
  else if (fortorders[oindex].type == DIG)
    {
      target = find_near_obj("", fortmobs[mindex].x, fortmobs[mindex].y, 60, &tmptypes, false, true);
    }
  else
    {
      if (fortorders[oindex].type == COLLECT)
	{
	  int type = tmptypes.get(0);
	  if (tmptypes.contains(PLANT))
	    {
	      type = 2;
	    }
	  if (designated[type].size() > 0)
	    {
	      target = designated[type].pop();
	    }
	}
      if (target == NONE)
	{
	  target = find_near_obj("", fortmobs[mindex].x, fortmobs[mindex].y, 60, &tmptypes);
	}
    }
  if (target == NONE)
    {
      return false;
    }
  if (!fortmobs[mindex].generatepath(fortobjs[target].x, fortobjs[target].y))
    {
      return false;
    }
  fortobjs[target].taken = true;
  fortmobs[mindex].targettype = ITEM;
  fortmobs[mindex].target = target;
  return true;
}

/**
 * Set all worker information for order with mob target.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @return true if everything is properly set, otherwise false.
 */
bool set_order_mob(int mindex, int oindex)
{
  switch (fortorders[oindex].type)
    {
    case REFUEL:
      for (int k = 1; k < (int)fortmobs.size(); k++)
	{
	  if (fortmobs[k].taken == mindex)
	    {
	      if (!fortmobs[mindex].generatepath(fortmobs[k].x, fortmobs[k].y))
		{
		  continue;
		}
	      fortmobs[k].taken = mindex;
	      fortmobs[mindex].target = k;
	      fortmobs[mindex].targettype = MOB;
	      break;
	    }
	}
      break;
    case REPAIR:
      for (int k = 1; k < (int)fortmobs.size(); k++)
	{
	  if ((fortmobs[k].type != ROBOT) || (fortmobs[k].stats[7] == fortmobs[k].stats[8]))
	    {
	      continue;
	    }
	  if (!fortmobs[mindex].generatepath(fortmobs[k].x, fortmobs[k].y))
	    {
	      continue;
	    }
	  fortmobs[mindex].target = k;
	  fortmobs[mindex].targettype = MOB;
	  fortorders[oindex].stages[0][6] = fortmobs[k].stats[8] - fortmobs[k].stats[7];
	  fortorders[oindex].stages[0][7] = fortmobs[k].stats[8] - fortmobs[k].stats[7];
	  break;
	}
      break;
    default:
      break;
    }
  if (fortorders[oindex].type == COMBAT)
    {
      TCODList<int> *tmptypes = new TCODList<int>();
      tmptypes->push(fortorders[oindex].stages[0][1]);
      int index = find_near_mob("", fortmobs[mindex].x, fortmobs[mindex].y, 80, tmptypes, 0, false);
      delete tmptypes;
      if (index != NONE)
	{
	  if (!fortmobs[mindex].generatepath(fortmobs[index].x, fortmobs[index].y))
	    {
	      return false;
	    }
	  fortmobs[mindex].target = index;
	  fortmobs[mindex].targettype = MOB;
	}
    }
  if (fortmobs[mindex].target == NONE)
    {
      if (mindex == 0)
	{
	  fortmobs[mindex].work = "IDLE";
	}
      return false;
    }
  return true;
}

/**
 * Set all worker information for order with mob target.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @param stage Stage of order
 * @return true if everything is properly set, otherwise false.
 */
bool set_order_build(int mindex, int oindex, int stage)
{
  for (int k = 0; k < (int)fortbuilds.size(); k++)
    {
      if (fortbuilds[k].type == fortorders[oindex].stages[stage][1])
	{
	  if ((fortorders[oindex].type == OBUILD) && (fortbuilds[k].underconstruction == 'N'))
	    {
	      continue;
	    }
	  if (((fortorders[oindex].type == GATHER) || (fortorders[oindex].type == WORK) || (fortorders[oindex].type == CRAFT) || (fortorders[oindex].type == RESEARCH)) && (!fortbuilds[k].constructed))
	    {
	      continue;
	    }
	  if ((fortbuilds[k].taken != NONE) && (fortbuilds[k].taken != mindex))
	    {
	      continue;
	    }
	  if ((fortmobs[mindex].x != fortbuilds[k].x + fortbuilds[k].enter[0]) && (fortmobs[mindex].y != fortbuilds[k].y + fortbuilds[k].enter[1]))
	    {
	      if (!fortmobs[mindex].generatepath(fortbuilds[k].x + fortbuilds[k].enter[0], fortbuilds[k].y + fortbuilds[k].enter[1]))
		{
		  continue;
		}
	    }
	  fortmobs[mindex].target = k;
	  fortmobs[mindex].targettype = BUILDING;
	  if (fortorders[oindex].type == CRAFT)
	    {
	      fortbuilds[k].taken = mindex;
	    }
	}
    }
  if (fortmobs[mindex].target == NONE)
    {
      fortmobs[mindex].work = "IDLE";
      return false;
    }
  return true;
}

/**
 * Set all worker information for order with land type target.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @param stage Stage of order
 * @return true if everything is properly set, otherwise false.
 */
bool set_order_land(int mindex, int oindex, int stage)
{
  int x = fortmobs[mindex].x;
  int y = fortmobs[mindex].y;
  find_near_land(&x, &y, 50 + fortmobs[mindex].stats[4], fortorders[oindex].stages[stage][1]);
  if ((x == NONE) || (y == NONE))
    {
      return false;
    }
  if (!fortmobs[mindex].generatepath(x, y))
    {
      return false;
    }
  return true;
}

/**
 * Set all worker informations for selected order.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @param stage Stage of order
 */
bool set_order(int mindex, int oindex, int stage)
{
  bool valid;
  switch (fortorders[oindex].stages[stage][0])
    {
    case ITEM:
      if (fortorders[oindex].stages[stage][1] != NONE)
	{
	  valid = set_order_item(mindex, oindex, stage);
	}
      else
	{
	  valid = true;
	}
      break;
    case MOB:
      valid = set_order_mob(mindex, oindex);
      break;
    case BUILDING:
      valid = set_order_build(mindex, oindex, stage);
      break;
    case LAND:
      valid = set_order_land(mindex, oindex, stage);
      break;
    default:
      break;
    }
  if (valid)
    {
      fortmobs[mindex].work = fortorders[oindex].name;
      fortmobs[mindex].setworktime(fortorders[oindex].stages[stage][7], fortorders[oindex].stages[stage][8]);
    }
  return valid;
}

/**
 * Finish selected order with item result.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @param stage Number of stage of order.
 */
void finish_order_item(int mindex, int oindex, int stage)
{
  TCODList<int> tmptypes;
  tmptypes.push(fortorders[oindex].stages[stage][4]);
  if (fortorders[oindex].stages[stage][9] != NONE)
    {
      tmptypes.push(fortorders[oindex].stages[stage][9]);
    }
  if (fortorders[oindex].stages[stage][12] != NONE)
    {
      tmptypes.push(fortorders[oindex].stages[stage][12]);
    }
  int result = find_proto_obj("", &tmptypes);
  tmptypes.clear();
  int x, y;
  int target = fortmobs[mindex].target;
  if ((fortorders[oindex].stages[stage][0] == ITEM) && (fortorders[oindex].stages[stage][1] != NONE))
    {
      x = fortobjs[target].x;
      y = fortobjs[target].y;
      update_map_item(x, y, target, true);
    }
  for (int k = 0; k < fortorders[oindex].stages[stage][5]; k++)
    {
      switch (fortorders[oindex].stages[stage][0])
	{
	case ITEM:
	  if (target != NONE)
	    {
	      protoobjs[result].setitem(fortobjs[target].x, fortobjs[target].y);
	    }
	  else
	    {
	      protoobjs[result].setitem(fortmobs[mindex].x, fortmobs[mindex].y);
	    }
	  break;
	case LAND:
	case BUILDING:
	  protoobjs[result].setitem(fortmobs[mindex].x, fortmobs[mindex].y);
	  break;
	default:
	  break;
	}
    }
  if ((fortorders[oindex].stages[stage][0] == ITEM) && (fortorders[oindex].stages[stage][1] != NONE))
    {
      fortobjs[target].deleteitem();
    }
}

/**
 * Finish selected order with discover result.
 * @param mindex Index of worker
 */
void finish_order_discover(int mindex)
{
  int result = fortmobs[mindex].target;
  fortobjs[result].hidden = false;
  int x = fortobjs[result].x;
  int y = fortobjs[result].y;
  update_map_item(x, y, result);
  fortmap[y][x].hitems.remove(result);
}

/**
 * Finish selected order with none result, item target.
 * @param mindex Index of worker
 */
void finish_order_none_item(int mindex)
{
  int target = fortmobs[mindex].target;
  fortmobs[mindex].carries = target;
  int x = fortobjs[target].x;
  int y = fortobjs[target].y;
  update_map_item(x, y, target, true);
  fortobjs[target].deleteitem();
}

/**
 * Finish selected order with none result, mob target.
 * @param mindex Index of worker
 * @param oindex Index of order
 */
void finish_order_none_mob(int mindex, int oindex)
{
  int target = fortmobs[mindex].target;
  int result = fortmobs[mindex].carries;
  if (result != NONE)
    {
      fortobjs[result].deleteitem();
      fortmobs[mindex].carries = NONE;
    }
  fortmobs[target].taken = NONE;
  switch (fortorders[oindex].type)
    {
    case REFUEL:
      fortmobs[target].stats[0] += 200;
      break;
    case REPAIR:
      fortmobs[target].stats[7] = fortmobs[target].stats[8];
      break;
    case COMBAT:
      fortmobs[mindex].work = "Fighting";
      fortmobs[mindex].worktime = 0;
    default:
      break;
    }
}

/**
 * Finish selected order with none result, building target.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @param stage Stage of order
 * @return New stage of order
 */
int finish_order_none_build(int mindex, int oindex, int stage)
{
  int target = fortmobs[mindex].target, result;
  //Drop carried item
  result = fortmobs[mindex].carries;
  switch (fortorders[oindex].type)
    {
    case GATHER:
      fortobjs[result].x = fortbuilds[target].x + fortbuilds[target].enter[0];
      fortobjs[result].y = fortbuilds[target].y + fortbuilds[target].enter[1];
      fortobjs[result].taken = false;
      fortobjs[result].stored = true;
      fortmobs[mindex].carries = NONE;
      break;
    default:
      if (result == NONE)
	{
	  break;
	}
      fortmobs[mindex].carries = NONE;
      break;
    }
  fortorders[oindex].curamount --;
  if (fortorders[oindex].curamount <= 0)
    {
      stage++;
      if (stage < STEPS)
	{
	  if (fortorders[oindex].stages[stage][0] == BUILDING)
	    {
	      fortorders[oindex].curamount = fortorders[oindex].stages[stage][2];
	      fortorders[oindex].curstage = stage;
	    }
	}
    }
  else if (fortorders[oindex].stages[stage + 1][0] != NONE)
    {
      stage--;
    }
  //Build building
  if ((fortorders[oindex].type == OBUILD) && (fortorders[oindex].stages[stage][0] == NONE))
    {
      fortbuilds[target].underconstruction = 'N';
      fortbuilds[target].constructed = true;
      if (fortbuilds[target].workname.size() > 0)
	{
	  for (int k = 0; k < (int)fortorders.size(); k++)
	    {
	      if (fortorders[k].name.compare(fortbuilds[target].workname) == 0)
		{
		  fortorders[k].enabled = true;
		  break;
		}
	    }
	}
      for (int k = 0; k < (int)protobuilds.size(); k++)
	{
	  if (protobuilds[k].type == fortbuilds[target].type)
	    {
	      if (!protobuilds[k].unique)
		{
		  protobuilds[k].underconstruction = 'N';
		}
	      else
		{
		  protobuilds[k].constructed = true;
		}
	      break;
	    }
	}
      fortorders[oindex].enabled = false;
      //Add recipes (if any) to this building.
      add_recipes(fortbuilds[target].type);
      //Set map cells values
      int l = 0, x, y;
      for (int j = 0; j < fortbuilds[target].height; j++)
	{
	  y = fortbuilds[target].y + j;
	  for (int k = 0; k < fortbuilds[target].width; k++)
	    {
	      x = fortbuilds[target].x + k;
	      if (fortmap[y][x].mob != NONE)
		{
		  continue;
		}
	      fortmap[y][x].ch = fortbuilds[target].chars.get(l + k);
	      fortmap[y][x].color = fortbuilds[target].colors.get(l + k);
	    }
	  l = l + fortbuilds[target].width;
	}
    }
  return stage;
}

/**
 * Finish selected order with none result.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @param stage Stage of order
 */
int finish_order_none(int mindex, int oindex, int stage)
{
  switch (fortorders[oindex].stages[stage][0])
    {
    case ITEM:
      finish_order_none_item(mindex);
      stage++;
      break;
    case MOB:
      finish_order_none_mob(mindex, oindex);
      stage++;
      break;
    case BUILDING:
      stage = finish_order_none_build(mindex, oindex, stage);
      break;
    default:
      break;
    }
  return stage;
}

/**
 * Finish selected order with mob as result.
 * @param mindex Index of worker.
 * @param oindex Index of order.
 */
void finish_order_mob(int mindex, int oindex)
{
  if (fortorders[oindex].type == CRAFT)
    {
      int index = find_mob("", -2, -2);
      fortmobs[index].setmob(fortmobs[mindex].x, fortmobs[mindex].y, index);
      fortmobs[index].work = "IDLE";
      fortorders[oindex].enabled = false;
    }
}

/**
 * Finish selected order with order or spell as result.
 * @param oindex Index of order.
 * @param order If true, result is order, otherwise spell.
 */
void finish_order_research(int oindex, bool order = true)
{
  string name = fortorders[oindex].name.substr(9);
  int index;
  int index2 = NONE;
  if (order)
    {
      index = find_recipe(name, LABORATORY);
      index2 = find_recipe(name, fortrecipes[index].unlocktype[0]);
      fortrecipes[index2].known = true;
      if (find_build("", fortrecipes[index].unlocktype[0], true) != NONE)
	{
	  add_recipes(fortrecipes[index].unlocktype[0]);
	}
    }
  else
    {
      index = find_recipe(name, LIBRARY);
    }
  if (fortrecipes[index].unlocktype[1] != NONE)
    {
      if (fortrecipes[index].unlockname.size() > 0)
	{
	  name = fortrecipes[index].unlockname;
	}
      index2 = find_recipe(name, fortrecipes[index].unlocktype[1]);
      fortrecipes[index2].known = true;
      add_recipes(fortrecipes[index].unlocktype[1]);
    }
  if (order)
    {
      fortorders[oindex].enabled = false;
    }
  else
    {
      index = find_spell(name);
      fortmobs[0].spells.push(index);
    }
}

/**
 * Finish selected order.
 * @param mindex Index of worker
 * @param oindex Index of order
 * @param stage Stage of order
 * @param tmpworkers Temporary list of workers.
 */
void finish_order(int mindex, int oindex, int stage, vector<int> *tmpworkers)
{
  fortmobs[mindex].stats[0] -= fortorders[oindex].stages[stage][6];
  fortmobs[mindex].gainexp(fortorders[oindex].stages[stage][6]);
  switch (fortorders[oindex].stages[stage][3])
    {
    case ITEM:
      finish_order_item(mindex, oindex, stage);
      stage++;
      break;
    case DISCOVER:
      finish_order_discover(mindex);
      stage++;
      break;
    case NONE:
      stage = finish_order_none(mindex, oindex, stage);
      break;
    case MOB:
      finish_order_mob(mindex, oindex);
      stage++;
      break;
    case ORDER1:
      finish_order_research(oindex);
      stage++;
      break;
    case SPELL:
      finish_order_research(oindex, false);
      stage ++;
      break;
    default:
      break;
    }
  if ((fortmobs[mindex].stats[0] <= 0) && (fortmobs[mindex].type == ROBOT))
    {
      addMessage(new Message(fortmobs[mindex].name + " need refueling.", fortmobs[mindex].x, fortmobs[mindex].y));
    }
  if (fortmobs[mindex].work.compare("Fighting") == 0)
    {
      tmpworkers->push_back(mindex);
      return;
    }
  if ((fortorders[oindex].stages[stage][0] == NONE) || (stage == STEPS))
    {
      fortorders[oindex].times--;
      if (fortorders[oindex].times <= 0)
	{
	  fortmobs[mindex].work = "IDLE";
	  if (fortmobs[mindex].targettype == BUILDING)
	    {
	      int target = fortmobs[mindex].target;
	      fortbuilds[target].taken = NONE;
	    }
	  addMessage(new Message("Order '" + fortorders[oindex].name + "' was finished.", fortmobs[mindex].x, fortmobs[mindex].y));
	}
      else
	{
	  fortmobs[mindex].work = "Ready";
	  fortorders[oindex].curamount = fortorders[oindex].stages[0][2];
	  tmpworkers->push_back(mindex);
	}
      fortmobs[mindex].workstage = 0;
      fortorders[oindex].curstage = 0;
    }
  else
    {
      fortmobs[mindex].work = "Ready";
      fortmobs[mindex].workstage = stage;
      tmpworkers->push_back(mindex);
    }
  fortmobs[mindex].target = NONE;
  fortmobs[mindex].targettype = NONE;
}

/**
 * Update all orders during game loop.
 */
void update_orders()
{
  int index;
  vector<int> tmpworkers;
  bool valid;
  for (int i = 0; i < (int)fortorders.size(); i++)
    {
      if (fortorders[i].active == 'N')
	{
	  continue;
	}
      for (int j = 0; j < (int)fortorders[i].workers.size(); j++)
	{
	  valid = true;
	  index = fortorders[i].workers[j];
	  if (fortmobs[index].work.compare("IDLE") == 0)
	    {
	      if (index == 0)
		{
		  fortmobs[index].work = "Ready";
		}
	      else
		{
		  continue;
		}
	    }
	  if (fortorders[i].curstage < fortmobs[index].workstage)
	    {
	      fortorders[i].curstage = fortmobs[index].workstage;
	    }
	  else if (fortmobs[index].workstage < fortorders[i].curstage)
	    {
	      fortmobs[i].workstage = fortorders[index].curstage;
	    }
	  //Set path to target
	  if (fortmobs[index].work.compare("Ready") == 0)
	    {
	      valid = set_order(index, i, fortmobs[index].workstage);
	      if (!valid)
		{
		  fortmobs[index].work = "IDLE";
		}
	    }
	  //Finish work
	  if ((fortmobs[index].work.compare(fortorders[i].name) == 0) && (fortmobs[index].worktime <= 0))
	    {
	      finish_order(index, i, fortmobs[index].workstage, &tmpworkers);
	    }
	  else if (valid)
	    {
	      tmpworkers.push_back(index);
	    }
	}
      fortorders[i].workers = tmpworkers;
      if (fortorders[i].workers.size() == 0)
	{
	  fortorders[i].active = 'N';
	}
      tmpworkers.clear();
    }
}

/**
 * Find selected order by name
 * @param name Name of order
 @ @return Order index or -1
 */
int find_order(std::string name)
{
  int order = NONE;
  for (int i = 0; i < (int)fortorders.size(); i++)
    {
      if (fortorders[i].name.compare(name) == 0)
	{
	  order = i;
	  break;
	}
    }
  return order;
}
