/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#ifndef ORDER_HPP
#define ORDER_HPP
#include <string>
#include <vector>
#include "gui/gui.hpp"

#define STEPLENGTH 13
#define STEPS 11

/**
 * Contains order types.
 */
typedef enum OrderTypes
  {
    COLLECT,
    GATHER,
    OBUILD,
    WORK,
    REFUEL,
    DIG,
    CRAFT,
    WALK,
    RESEARCH,
    REPAIR,
    COMBAT
  } ORDERTYPE;

/**
 * Contains all informations and functions about orders.
 */
class Order
{
public:
  /**
   * Name of order
   */
  std::string name;
  /**
   * Did order can be repeated
   */
  bool infinite;
  /**
   * If Y then order is active, otherwise 'N'
   */
  char active;
  /**
   * List of mobiles assigned to order.
   */
  std::vector<int> workers;
  /**
   * Stages of order.
   */
  int stages[STEPS][STEPLENGTH];
  /**
   * Amount of things needed for finish current state.
   */
  int curamount;
  /**
   * If true, player is automatically added to workers list.
   */
  bool automatic;
  /**
   * If true, orders is available.
   */
  bool enabled;
  /**
   * Type of order.
   */
  int type;
  /**
   * Current stage of order.
   */
  int curstage;
  /**
   * Only player can be assigned to this order.
   */
  bool playeronly;
  /**
   * How many times order will be repeated.
   */
  int times;

  Order();
};

void buildOrderUI();
void setVisibleOrders(bool visible);
void update_orders();
int find_order(std::string name);
void drawWorkers(int index);
void show_orders(int start = 0);

#endif
