/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "libtcod.hpp"
#include "parser.hpp"
#include "main.hpp"
#include "item.hpp"
#include "mobile.hpp"
#include "building.hpp"
#include "order.hpp"
#include "recipe.hpp"
#include <iostream>
#include "help.hpp"
#include <stdio.h>

bool Listener::parserNewStruct(TCODParser *parser, const TCODParserStruct *str, const char *name)
{
  if (!strcmp(str->getName(), "item_type"))
    {
      type = ITEM;
      item.name = name;
      item.walkable = false;
      item.transparent = false;
      item.hidden = false;
      item.description = "";
      item.data = NONE;
      item.damage = "";
      item.range = 1;
      item.use = 0;
    }
  if (!strcmp(str->getName(), "mob_type"))
    {
      type = MOB;
      mob.description = "";
      mob.location = NONE;
      mob.flying = false;
    }
  if (!strcmp(str->getName(), "building_type"))
    {
      type = BUILDING;
      build.name = name;
      build.unique = false;
      build.workname = "";
      build.description = "";
    }
  if (!strcmp(str->getName(), "order_type"))
    {
      type = ORDER1;
      if (find_order(name) > NONE)
	{
	  order.name = "added";
	}
      else
	{
	  order.name = name;
	}
      order.enabled = true;
      order.automatic = false;
      order.infinite = false;
      order.playeronly = false;
      order.type = NONE;
    }
  if (!strcmp(str->getName(), "recipe_type"))
    {
      type = RECIPE;
      recipe.name = name;
      recipe.known = true;
      recipe.fuel = -1;
      recipe.unlocktype[0] = NONE;
      recipe.unlocktype[1] = NONE;
      recipe.unlockname = "";
    }
  if (!strcmp(str->getName(), "mission_type"))
    {
      type = MISSION;
      if (find_mission(name) > NONE)
	{
	  mission.name = "added";
	}
      else
	{
	  mission.name = name;
	}
    }
  if (!strcmp(str->getName(), "help_type"))
    {
      type = HELPENTRY;
      help2.name = name;
      help2.parent = "";
    }
  if (!strcmp(str->getName(), "spell_type"))
    {
      type = SPELL;
      spell.name = name;
      spell.range = 1;
    }
  return true;
}

bool Listener::parserFlag(TCODParser *parser, const char *name)
{
  switch (type)
    {
    case ITEM:
      if (!strcmp(name, "walkable"))
	{
	  item.walkable = true;
	}
      else if (!strcmp(name, "transparent"))
	{
	  item.transparent = true;
	}
      else if (!strcmp(name, "hidden"))
	{
	  item.hidden = true;
	}
      break;
    case MOB:
      if (!strcmp(name, "flying"))
	{
	  mob.flying = true;
	}
      break;
    case BUILDING:
      if (!strcmp(name, "unique"))
	{
	  build.unique = true;
	}
      break;
    case ORDER1:
      if (!strcmp(name, "infinite"))
	{
	  order.infinite = true;
	}
      else if (!strcmp(name, "automatic"))
	{
	  order.automatic = true;
	}
      else if (!strcmp(name, "disabled"))
	{
	  order.enabled = false;
	}
      else if (!strcmp(name, "playeronly"))
	{
	  order.playeronly = true;
	}
      break;
    case RECIPE:
      if (!strcmp(name, "unknown"))
	{
	  recipe.known = false;
	}
      break;
    default:
      break;
    }
  return true;
}

bool Listener::parserProperty(TCODParser *parser, const char *name, TCOD_value_type_t type, TCOD_value_t value) 
{
    switch (Listener::type)
    {
    case ITEM:
      if (!strcmp(name, "types"))
	{
	  item.types = value.list;
	}
      else if (!strcmp(name, "ch"))
	{
	  item.ch = (int)value.c;
	  if (item.ch < 0)
	    {
	      item.ch = 256 + item.ch;
	    }
	}
      else if (!strcmp(name, "col"))
	{
	  item.color = value.col;
	}
      else if (!strcmp(name, "desc"))
	{
	  item.description = value.s;
	}
      else if (!strcmp(name, "data"))
	{
	  item.data = value.i;
	}
      else if (!strcmp(name, "damage"))
	{
	  item.damage = value.s;
	}
      else if (!strcmp(name, "range"))
	{
	  item.range = value.i;
	}
      else if (!strcmp(name, "use"))
	{
	  item.use = value.i;
	}
      break;
    case MOB:
      if (!strcmp(name, "name"))
	{
	  mob.name = value.s;
	}
      else if (!strcmp(name, "type"))
	{
	  mob.type = value.i;
	}
      else if (!strcmp(name, "ch"))
	{
	  mob.ch = value.c;
	}
      else if (!strcmp(name, "col"))
	{
	  mob.color = value.col;
	}
      else if (!strcmp(name, "stats"))
	{
	  mob.stats = value.list;
	}
      else if (!strcmp(name, "desc"))
	{
	  mob.description = value.s;
	}
      else if (!strcmp(name, "equipment"))
	{
	  mob.equip = value.list;
	}
      else if (!strcmp(name, "ais"))
	{
	  mob.ais = value.list;
	}
      else if (!strcmp(name, "inventory"))
	{
	  mob.inv = value.list;
	}
      else if (!strcmp(name, "location"))
	{
	  mob.location = value.i;
	}
      else if (!strcmp(name, "loot"))
	{
	  mob.loot = value.list;
	}
      else if (!strcmp(name, "spells"))
	{
	  mob.spells = value.list;
	}
      break;
    case BUILDING:
      if (!strcmp(name, "type"))
	{
	  build.type = value.i;
	}
      else if (!strcmp(name, "width"))
	{
	  build.width = value.i;
	}
      else if (!strcmp(name, "height"))
	{
	  build.height = value.i;
	}
      else if (!strcmp(name, "enter_x"))
	{
	  build.enter[0] = value.i;
	}
      else if (!strcmp(name, "enter_y"))
	{
	  build.enter[1] = value.i;
	}
      else if (!strcmp(name, "mname"))
	{
	  build.mname = value.s;
	}
      else if (!strcmp(name, "material"))
	{
	  build.material = value.i;
	}
      else if (!strcmp(name, "mamount"))
	{
	  build.mamount = value.i;
	}
      else if (!strcmp(name, "buildtime"))
	{
	  build.buildtime = value.i;
	}
      else if (!strcmp(name, "workname"))
	{
	  build.workname = value.s;
	}
      else if (!strcmp(name, "data"))
	{
	  build.data = value.i;
	}
      else if (!strcmp(name, "chars"))
	{
	  build.chars = value.list;
	}
      else if (!strcmp(name, "colors"))
	{
	  build.colors = value.list;
	}
      else if (!strcmp(name, "neededitems"))
	{
	  build.neededitems = value.list;
	}
      else if (!strcmp(name, "desc"))
	{
	  build.description = value.s;
	}
    case ORDER1:
      if (!strcmp(name, "step1"))
	{
	  order.step1 = value.list;
	}
      else if (!strcmp(name, "step2"))
	{
	  order.step2 = value.list;
	}
      else if (!strcmp(name, "step3"))
	{
	  order.step3 = value.list;
	}
      else if (!strcmp(name, "step4"))
	{
	  order.step4 = value.list;
	}
      else if (!strcmp(name, "step5"))
	{
	  order.step5 = value.list;
	}
      else if (!strcmp(name, "type"))
	{
	  order.type = value.i;
	}
      break;
    case RECIPE:
      if (!strcmp(name, "resource1"))
	{
	  recipe.resource1 = value.list;
	}
      else if (!strcmp(name, "resource2"))
	{
	  recipe.resource2 = value.list;
	}
      else if (!strcmp(name, "resource3"))
	{
	  recipe.resource3 = value.list;
	}
      else if (!strcmp(name, "resource4"))
	{
	  recipe.resource4 = value.list;
	}
      else if (!strcmp(name, "amount"))
	{
	  recipe.amount = value.i;
	}
      else if (!strcmp(name, "worktime"))
	{
	  recipe.worktime = value.i;
	}
      else if (!strcmp(name, "building"))
	{
	  recipe.building = value.i;
	}
      else if (!strcmp(name, "unlocktype"))
	{
	  recipe.unlocktype[0] = value.i;
	}
      else if (!strcmp(name, "unlocktype2"))
	{
	  recipe.unlocktype[1] = value.i;
	}
      else if (!strcmp(name, "unlockname"))
	{
	  recipe.unlockname = value.s;
	}
      else if (!strcmp(name, "fuel"))
	{
	  recipe.fuel = value.i;
	}
      else if (!strcmp(name, "materials"))
	{
	  recipe.materials = value.s;
	}
      break;
    case MISSION:
      if (!strcmp(name, "location"))
	{
	  mission.location = value.i;
	}
      else if (!strcmp(name, "difficult"))
	{
	  mission.difficult = value.i;
	}
      else if (!strcmp(name, "length"))
	{
	  mission.length = value.i;
	}
      else if (!strcmp(name, "targettype"))
	{
	  mission.targettype = value.i;
	}
      else if (!strcmp(name, "target"))
	{
	  mission.target = value.s;
	}
      else if (!strcmp(name, "description"))
	{
	  mission.description = value.s;
	}
      break;
    case HELPENTRY:
      if (!strcmp(name, "body"))
	{
	  help2.body = value.s;
	}
      else if (!strcmp(name, "options"))
	{
	  help2.options = value.list;
	}
      else if (!strcmp(name, "parent"))
	{
	  help2.parent = value.s;
	}
      break;
    case SPELL:
      if (!strcmp(name, "type"))
	{
	  spell.type = value.i;
	}
      else if (!strcmp(name, "target"))
	{
	  spell.target = value.i;
	}
      else if (!strcmp(name, "edice"))
	{
	  spell.dice = value.s;
	}
      else if (!strcmp(name, "effect"))
	{
	  spell.effect = value.i;
	}
      else if (!strcmp(name, "cost"))
	{
	  spell.cost = value.i;
	}
      else if (!strcmp(name, "memory"))
	{
	  spell.memory = value.i;
	}
      else if (!strcmp(name, "range"))
	{
	  spell.range = value.i;
	}
      break;
    default:
      break;
    }
  return true;
}

bool Listener::parserEndStruct(TCODParser *parser, const TCODParserStruct *str, const char *name) 
{
  if (type == ITEM)
    {
      Item *obj = new Item();
      obj->name = item.name;
      int i = 1;
      for (int * iterator = (int*)item.types.begin(); iterator != (int*)item.types.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  obj->types.push(*iterator);
	}
      obj->ch = item.ch;
      obj->color = item.color;
      obj->walkable = item.walkable;
      obj->transparent = item.transparent;
      obj->x = -1;
      obj->y = -1;
      obj->hidden = item.hidden;
      obj->description = item.description;
      obj->data = item.data;
      obj->damage = item.damage;
      obj->range = item.range;
      obj->use = item.use;
      item.types.clear();
      protoobjs.push_back(*obj);
      delete obj;
    }
  else if (type == MOB)
    {
      Mobile *mobile = new Mobile(mob.ch, mob.color, mob.type);
      int i = 1, j = 0;
      for (int * iterator = (int*)mob.stats.begin(); iterator != (int*)mob.stats.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  mobile->stats[j] = *iterator;
	  j++;
	}
      mob.stats.clear();
      if (!mob.equip.isEmpty())
	{
	  for (int i = 0; i < mob.equip.size(); i++)
	    {
	      j = find_proto_obj((char *)mob.equip.get(i));
	      mobile->equipment[i] = j;
	    }
	  mob.equip.clear();
	}
      if (!mob.ais.isEmpty())
	{
	  i = 1, j = 0;
	  for (int * iterator = (int*)mob.ais.begin(); iterator != (int*)mob.ais.end(); iterator ++) 
	    {
	      i++;
	      if ((i % 2) > 0)
		{
		  continue;
		}
	      mobile->ais[j] = *iterator;
	      j++;
	    }
	  mob.ais.clear();
	}
      if (!mob.inv.isEmpty())
	{
	  i = 1, j = 0;
	  for (int i = 0; i < mob.inv.size(); i++)
	    {
	      j = find_proto_obj((char *)mob.inv.get(i));
	      mobile->inventory.push(j);
	    }
	  mob.inv.clear();
	}
      if (!mob.loot.isEmpty())
	{
	  i = 1, j = 0;
	  for (int * iterator = (int*)mob.loot.begin(); iterator != (int*)mob.loot.end(); iterator ++) 
	    {
	      i++;
	      if ((i % 2) > 0)
		{
		  continue;
		}
	      mobile->loot.push(*iterator);
	      j++;
	    }
	  mob.loot.clear();
	}
      if (!mob.spells.isEmpty())
	{
	  i = 1, j = 0;
	  for (int * iterator = (int*)mob.spells.begin(); iterator != (int*)mob.spells.end(); iterator ++) 
	    {
	      i++;
	      if ((i % 2) > 0)
		{
		  continue;
		}
	      mobile->spells.push(*iterator);
	      j++;
	    }
	  mob.spells.clear();
	}
      mobile->name = mob.name;
      mobile->description = mob.description;
      mobile->location = mob.location;
      mobile->flying = mob.flying;
      protomobs.push_back(*mobile);
      delete mobile;
    }
  else if (type == BUILDING)
    {
      Building *build2 = new Building();
      build2->name = build.name;
      build2->type = build.type;
      build2->width = build.width;
      build2->height = build.height;
      build2->enter[0] = build.enter[0];
      build2->enter[1] = build.enter[1];
      build2->mname = build.mname;
      build2->material = build.material;
      build2->mamount = build.mamount;
      build2->buildtime = build.buildtime;
      build2->workname = build.workname;
      build2->data = build.data;
      int i = 1;
      for (int * iterator = (int*)build.chars.begin(); iterator != (int*)build.chars.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  build2->chars.push(*iterator);
	}
      build.chars.clear();
      i = 1;
      for (int * iterator = (int*)build.colors.begin(); iterator != (int*)build.colors.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  build2->colors.push(TCODColor(*(TCOD_color_t *)iterator));
	}
      build.colors.clear();
      i = 1;
      for (int * iterator = (int*)build.neededitems.begin(); iterator != (int*)build.neededitems.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  build2->neededitems.push(*iterator);
	}
      build.neededitems.clear();
      build2->unique = build.unique;
      build2->description = build.description;
      protobuilds.push_back(*build2);
      delete build2;
    }
  else if ((type == ORDER1) && (order.name.compare("added")))
    {
      Order *ord = new Order();
      ord->name = order.name;
      ord->infinite = order.infinite;
      ord->automatic = order.automatic;
      ord->enabled = order.enabled;
      ord->playeronly = order.playeronly;
      ord->type = order.type;
      int i = 1, j = 0;
      for (int * iterator = (int*)order.step1.begin(); iterator != (int*)order.step1.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  ord->stages[0][j] = *iterator;
	  j++;
	}
      i = 1; j = 0;
      for (int * iterator = (int*)order.step2.begin(); iterator != (int*)order.step2.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  ord->stages[1][j] = *iterator;
	  j++;
	}
      i = 1; j = 0;
      for (int * iterator = (int*)order.step3.begin(); iterator != (int*)order.step3.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  ord->stages[2][j] = *iterator;
	  j++;
	}
      i = 1; j = 0;
      for (int * iterator = (int*)order.step4.begin(); iterator != (int*)order.step4.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  ord->stages[3][j] = *iterator;
	  j++;
	}
      i = 1; j = 0;
      for (int * iterator = (int*)order.step4.begin(); iterator != (int*)order.step4.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  ord->stages[3][j] = *iterator;
	  j++;
	}
      order.step1.clear();
      order.step2.clear();
      order.step3.clear();
      order.step4.clear();
      order.step5.clear();
      fortorders.push_back(*ord);
      delete ord;
    }
  if ((type == RECIPE)  && (find_recipe(recipe.name, recipe.building) == NONE))
    {
      Recipe *recip = new Recipe();
      recip->name = recipe.name;
      int i = 1, j = 0;
      for (int * iterator = (int*)recipe.resource1.begin(); iterator != (int*)recipe.resource1.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  recip->resources[0][j] = *iterator;
	  j++;
	}
      recipe.resource1.clear();
      i = 1, j = 0;
      for (int * iterator = (int*)recipe.resource2.begin(); iterator != (int*)recipe.resource2.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  recip->resources[1][j] = *iterator;
	  j++;
	}
      recipe.resource2.clear();
      i = 1, j = 0;
      for (int * iterator = (int*)recipe.resource3.begin(); iterator != (int*)recipe.resource3.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  recip->resources[2][j] = *iterator;
	  j++;
	}
      recipe.resource3.clear();
      i = 1, j = 0;
      for (int * iterator = (int*)recipe.resource4.begin(); iterator != (int*)recipe.resource4.end(); iterator ++) 
	{
	  i++;
	  if ((i % 2) > 0)
	    {
	      continue;
	    }
	  recip->resources[3][j] = *iterator;
	  j++;
	}
      recipe.resource4.clear();
      recip->fuel = recipe.fuel;
      recip->amount = recipe.amount;
      recip->worktime = recipe.worktime;
      recip->building = recipe.building;
      recip->known = recipe.known;
      recip->unlocktype[0] = recipe.unlocktype[0];
      recip->unlocktype[1] = recipe.unlocktype[1];
      recip->unlockname = recipe.unlockname;
      recip->materials = recipe.materials;
      fortrecipes.push_back(*recip);
      delete recip;
    }
  else if ((type == MISSION) && (mission.name.compare("added")))
    {
      Mission *miss = new Mission();
      miss->name = mission.name;
      miss->location = mission.location;
      miss->difficult = mission.difficult;
      miss->length = mission.length;
      miss->targettype = mission.targettype;
      miss->target = mission.target;
      miss->description = mission.description;
      protomissions.push_back(*miss);
      delete miss;
    }
  else if (type == HELPENTRY)
    {
      Help *helpentry = new Help();
      helpentry->name = help2.name;
      helpentry->body = help2.body;
      if (!help2.options.isEmpty())
	{
	  for (int i = 0; i < help2.options.size(); i++)
	    {
	      helpentry->options.push_back((char *)help2.options.get(i));
	    }
	  help2.options.clear();
	}
      helpentry->parent = help2.parent;
      helps.push_back(*helpentry);
      delete helpentry;
    }
  else if (type == SPELL)
    {
      Spell *spell2 = new Spell();
      spell2->name = spell.name;
      spell2->type = spell.type;
      spell2->target = spell.target;
      spell2->dice = spell.dice;
      spell2->effect = spell.effect;
      spell2->cost = spell.cost;
      spell2->memory = spell.memory;
      spell2->range = spell.range;
      protospells.push_back(*spell2);
      delete spell2;
    }
  return true;
}

void Listener::error(const char *msg) 
{
  cout << msg << endl;
  exit(1);
}

/**
 * Define structures and load all proto data from files.
 */
void loadproto()
{
  protoobjs.clear();
  protomobs.clear();
  protobuilds.clear();
  TCODParser *parser = new TCODParser();
  //Load item prototypes
  TCODParserStruct *itemTypeStruct = parser->newStructure("item_type");
  itemTypeStruct->addListProperty("types", TCOD_TYPE_INT, true);
  itemTypeStruct->addProperty("ch", TCOD_TYPE_CHAR, true);
  itemTypeStruct->addProperty("col", TCOD_TYPE_COLOR, true);
  itemTypeStruct->addProperty("desc", TCOD_TYPE_STRING, false);
  itemTypeStruct->addProperty("data", TCOD_TYPE_INT, false);
  itemTypeStruct->addProperty("damage", TCOD_TYPE_STRING, false);
  itemTypeStruct->addProperty("range", TCOD_TYPE_INT, false);
  itemTypeStruct->addProperty("use", TCOD_TYPE_INT, false);
  itemTypeStruct->addFlag("walkable");
  itemTypeStruct->addFlag("transparent");
  itemTypeStruct->addFlag("hidden");
  parser->run("data/items.dat", new Listener());
  delete itemTypeStruct;
  //Load mobile prototypes
  TCODParserStruct *mobTypeStruct = parser->newStructure("mob_type");
  mobTypeStruct->addProperty("name", TCOD_TYPE_STRING, true);
  mobTypeStruct->addProperty("type", TCOD_TYPE_INT, true);
  mobTypeStruct->addProperty("ch", TCOD_TYPE_CHAR, true);
  mobTypeStruct->addProperty("col", TCOD_TYPE_COLOR, true);
  mobTypeStruct->addProperty("desc", TCOD_TYPE_STRING, false);
  mobTypeStruct->addListProperty("stats", TCOD_TYPE_INT, true);
  mobTypeStruct->addListProperty("equipment", TCOD_TYPE_STRING, false);
  mobTypeStruct->addListProperty("ais", TCOD_TYPE_INT, false);
  mobTypeStruct->addListProperty("inventory", TCOD_TYPE_STRING, false);
  mobTypeStruct->addProperty("location", TCOD_TYPE_INT, true);
  mobTypeStruct->addListProperty("loot", TCOD_TYPE_INT, false);
  mobTypeStruct->addListProperty("spells", TCOD_TYPE_INT, false);
  mobTypeStruct->addFlag("flying");
  parser->run("data/mobs.dat", new Listener());
  delete mobTypeStruct;
  //Load building prototypes
  TCODParserStruct *buildTypeStruct = parser->newStructure("building_type");
  buildTypeStruct->addProperty("type", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("width", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("height", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("enter_x", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("enter_y", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("mname", TCOD_TYPE_STRING, true);
  buildTypeStruct->addProperty("material", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("mamount", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("buildtime", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("workname", TCOD_TYPE_STRING, false);
  buildTypeStruct->addProperty("data", TCOD_TYPE_INT, true);
  buildTypeStruct->addProperty("desc", TCOD_TYPE_STRING, false);
  buildTypeStruct->addListProperty("chars", TCOD_TYPE_INT, true);
  buildTypeStruct->addListProperty("colors", TCOD_TYPE_COLOR, true);
  buildTypeStruct->addListProperty("neededitems", TCOD_TYPE_INT, false);
  buildTypeStruct->addFlag("unique");
  parser->run("data/buildings.dat", new Listener());
  delete buildTypeStruct;
  //Load help system
  TCODParserStruct *helpTypeStruct = parser->newStructure("help_type");
  helpTypeStruct->addProperty("body", TCOD_TYPE_STRING, true);
  helpTypeStruct->addProperty("parent", TCOD_TYPE_STRING, false);
  helpTypeStruct->addListProperty("options", TCOD_TYPE_STRING, false);
  parser->run("data/helps.dat", new Listener());
  delete helpTypeStruct;
  //Load spells
  TCODParserStruct *spellTypeStruct = parser->newStructure("spell_type");
  spellTypeStruct->addProperty("type", TCOD_TYPE_INT, true);
  spellTypeStruct->addProperty("target", TCOD_TYPE_INT, true);
  spellTypeStruct->addProperty("edice", TCOD_TYPE_STRING, true);
  spellTypeStruct->addProperty("effect", TCOD_TYPE_INT, true);
  spellTypeStruct->addProperty("cost", TCOD_TYPE_INT, true);
  spellTypeStruct->addProperty("memory", TCOD_TYPE_INT, true);
  spellTypeStruct->addProperty("range", TCOD_TYPE_INT, false);
  parser->run("data/spells.dat", new Listener());
  delete parser;
  delete spellTypeStruct;
}

/**
 * Define structures and load orders from files.
 */
void loadorders()
{
  TCODParser *parser = new TCODParser();
  TCODParserStruct *orderTypeStruct = parser->newStructure("order_type");
  orderTypeStruct->addFlag("infinite");
  orderTypeStruct->addFlag("automatic");
  orderTypeStruct->addFlag("disabled");
  orderTypeStruct->addFlag("playeronly");
  orderTypeStruct->addProperty("type", TCOD_TYPE_INT, false);
  orderTypeStruct->addListProperty("step1", TCOD_TYPE_INT, true);
  orderTypeStruct->addListProperty("step2", TCOD_TYPE_INT, false);
  orderTypeStruct->addListProperty("step3", TCOD_TYPE_INT, false);
  orderTypeStruct->addListProperty("step4", TCOD_TYPE_INT, false);
  orderTypeStruct->addListProperty("step5", TCOD_TYPE_INT, false);
  parser->run("data/orders.dat", new Listener());
  delete orderTypeStruct;
  TCODParserStruct *recipeTypeStruct = parser->newStructure("recipe_type");
  recipeTypeStruct->addListProperty("resource1", TCOD_TYPE_INT, true);
  recipeTypeStruct->addListProperty("resource2", TCOD_TYPE_INT, false);
  recipeTypeStruct->addListProperty("resource3", TCOD_TYPE_INT, false);
  recipeTypeStruct->addListProperty("resource4", TCOD_TYPE_INT, false);
  recipeTypeStruct->addFlag("unknown");
  recipeTypeStruct->addProperty("amount", TCOD_TYPE_INT, true);
  recipeTypeStruct->addProperty("worktime", TCOD_TYPE_INT, true);
  recipeTypeStruct->addProperty("building", TCOD_TYPE_INT, true);
  recipeTypeStruct->addProperty("unlocktype", TCOD_TYPE_INT, false);
  recipeTypeStruct->addProperty("unlocktype2", TCOD_TYPE_INT, false);
  recipeTypeStruct->addProperty("unlockname", TCOD_TYPE_STRING, false);
  recipeTypeStruct->addProperty("fuel", TCOD_TYPE_INT, false);
  recipeTypeStruct->addProperty("materials", TCOD_TYPE_STRING, true);
  parser->run("data/recipes.dat", new Listener());
  delete recipeTypeStruct;
  TCODParserStruct *missionTypeStruct = parser->newStructure("mission_type");
  missionTypeStruct->addProperty("location", TCOD_TYPE_INT, true);
  missionTypeStruct->addProperty("difficult", TCOD_TYPE_INT, true);
  missionTypeStruct->addProperty("length", TCOD_TYPE_INT, true);
  missionTypeStruct->addProperty("targettype", TCOD_TYPE_INT, true);
  missionTypeStruct->addProperty("target", TCOD_TYPE_STRING, true);
  missionTypeStruct->addProperty("description", TCOD_TYPE_STRING, true);
  parser->run("data/missions.dat", new Listener());
  delete missionTypeStruct;
  delete parser;
}
