/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#ifndef PARSER_HPP
#define PARSER_HPP
#include <string>

struct ProtoItem
{
  std::string name;
  TCODList<void *> types;
  int ch;
  TCODColor color;
  bool walkable;
  bool transparent;
  bool hidden;
  std::string description;
  int data;
  std::string damage;
  int range;
  int use;
};

struct ProtoMob
{
  std::string name;
  int type;
  char ch;
  TCODColor color;
  TCODList<void *> stats;
  std::string description;
  TCODList<void *> equip;
  TCODList<void *> ais;
  TCODList<void *> inv;
  int location;
  TCODList<void *>loot;
  bool flying;
  TCODList<void *>spells;
};

struct ProtoBuild
{
  std::string name;
  int type;
  int width;
  int height;
  int enter[2];
  std::string mname;
  int material;
  int mamount;
  int buildtime;
  std::string workname;
  int data;
  TCODList<void *> chars;
  TCODList<void *> colors;
  TCODList<void *> neededitems;
  bool unique;
  std::string description;
};

struct ProtoOrder
{
  std::string name;
  bool infinite;
  bool automatic;
  bool enabled;
  TCODList<void *> step1;
  TCODList<void *> step2;
  TCODList<void *> step3;
  TCODList<void *> step4;
  TCODList<void *> step5;
  bool playeronly;
  int type;
};

struct ProtoRecipe
{
  std::string name;
  TCODList<void *> resource1;
  TCODList<void *> resource2;
  TCODList<void *> resource3;
  TCODList<void *> resource4;
  int fuel;
  int amount;
  int worktime;
  int building;
  bool known;
  int unlocktype[2];
  std::string unlockname;
  std::string materials;
};

struct ProtoMission
{
  std::string name;
  int location;
  int difficult;
  int length;
  int targettype;
  std::string target;
  std::string description;
};

struct ProtoHelp
{
  std::string name;
  std::string body;
  TCODList<void *> options;
  std::string parent;
};

struct ProtoSpell
{
  std::string name;
  int type;
  int target;
  std::string dice;
  int effect;
  int cost;
  int memory;
  int range;
};

class Listener : public ITCODParserListener 
{
  int type;
  ProtoItem item;
  ProtoMob mob;
  ProtoBuild build;
  ProtoOrder order;
  ProtoRecipe recipe;
  ProtoMission mission;
  ProtoHelp help2;
  ProtoSpell spell;

  bool parserNewStruct(TCODParser *parser, const TCODParserStruct *str, const char *name);
  bool parserFlag(TCODParser *parser, const char *name);
  bool parserProperty(TCODParser *parser, const char *name, TCOD_value_type_t type, TCOD_value_t value);
  bool parserEndStruct(TCODParser *parser, const TCODParserStruct *str, const char *name);
  void error(const char *msg);
};

void loadproto();
void loadorders();

#endif
