/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "recipe.hpp"
#include "main.hpp"
#include "building.hpp"
#include "libtcod.hpp"
#include <iostream>

/**
 * Standard class constructor.
 */
Recipe::Recipe()
{
  for (int i = 0; i < RESOURCES; i++)
    {
      for (int j = 0; j < 3; j++)
	{
	  resources[i][j] = NONE;
	}
    }
  fuel = -1;
  known = true;
  unlocktype[0] = NONE;
  unlocktype[1] = NONE;
  unlockname = "";
}

/**
 * Find selected recipe by name
 * @param name Name of recipe to find
 * @param building Building to which recipes is assigned. Default -1.
 * @return Index of recipe or -1.
 */
int find_recipe(std::string name, int building)
{
  int recipe = NONE;
  for (int i = 0; i < (int)fortrecipes.size(); i++)
    {
      //Search by building
      if ((building != NONE) && (fortrecipes[i].building != building))
	{
	  continue;
	}
      //Search by name
      if (fortrecipes[i].name.compare(name))
	{
	  continue;
	}
      recipe = i;
      break;
    }
  return recipe;
}

/**
 * Find selected recipe by name and buildings.
 * @param name Name of recipe to find
 * @param buildings Buildings to which search recipe.
 * @return Index of recipe or -1.
 */
int find_recipe2(std::string name, TCODList<int> *buildings)
{
  int recipe = NONE;
  for (int i = 0; i < (int)fortrecipes.size(); i++)
    {
      //Search by building
      if (!buildings->isEmpty())
	{
	  if (!buildings->contains(fortrecipes[i].building))
	    {
	      continue;
	    }
	}
      //Search by name
      if (fortrecipes[i].name.compare(name))
	{
	  continue;
	}
      recipe = i;
      break;
    }
  return recipe;
}

/**
 * Add orders for selected building
 * @param build Type of building.
 */
void add_recipes(int build)
{
  int stage, item, exists;
  Order *order;
  for (int i = 0; i < (int)fortrecipes.size(); i++)
    {
      if ((fortrecipes[i].building == build) && (fortrecipes[i].known))
	{
	  stage = 0;
	  order = new Order();
	  if ((fortrecipes[i].building != LABORATORY) && (fortrecipes[i].building != LIBRARY))
	    {
	      order->name = "Make " + fortrecipes[i].name;
	      order->type = CRAFT;
	      order->infinite = true;
	    }
	  else
	    {
	      order->name = "Research " + fortrecipes[i].name;
	      order->type = RESEARCH;
	      order->playeronly = true;
	      order->infinite = false;
	    }
	  for (int j = 0; j < RESOURCES; j++)
	    {
	      if (fortrecipes[i].resources[j][0] == NONE)
		{
		  break;
		}
	      order->stages[stage][0] = ITEM;
	      order->stages[stage][1] = fortrecipes[i].resources[j][0];
	      order->stages[stage][2] = 1;
	      order->stages[stage][6] = 0;
	      order->stages[stage][7] = 1;
	      order->stages[stage][10] = fortrecipes[i].resources[j][1];
	      stage++;
	      order->stages[stage][0] = BUILDING;
	      order->stages[stage][1] = fortrecipes[i].building;
	      order->stages[stage][2] = fortrecipes[i].resources[j][2];
	      order->stages[stage][6] = 0;
	      order->stages[stage][7] = 1;
	      stage++;
	    }
	  if (fortrecipes[i].fuel != NONE)
	    {
	      order->stages[stage][0] = ITEM;
	      order->stages[stage][1] = FUEL;
	      order->stages[stage][2] = 1;
	      order->stages[stage][6] = 0;
	      order->stages[stage][7] = 1;
	      stage ++;
	      order->stages[stage][0] = BUILDING;
	      order->stages[stage][1] = fortrecipes[i].building;
	      order->stages[stage][2] = fortrecipes[i].fuel;
	      order->stages[stage][6] = 0;
	      order->stages[stage][7] = 1;
	      stage ++;
	    }
	  order->stages[stage][0] = BUILDING;
	  order->stages[stage][1] = fortrecipes[i].building;
	  order->stages[stage][2] = fortrecipes[i].worktime;
	  if (order->type == CRAFT)
	    {
	      order->stages[stage][3] = ITEM;
	      item = find_proto_obj(fortrecipes[i].name);
	      order->stages[stage][4] = protoobjs[item].types.get(1);
	      order->stages[stage][5] = fortrecipes[i].amount;
	      order->stages[stage][6] = 1;
	      order->stages[stage][7] = 1;
	      order->stages[stage][8] = 3;
	      if (protoobjs[item].types.size() > 2)
		{
		  order->stages[stage][9] = protoobjs[item].types.get(2);
		}
	      if (protoobjs[item].types.size() > 3)
		{
		  order->stages[stage][12] = protoobjs[item].types.get(3);
		}
	    }
	  else
	    {
	      if (fortrecipes[i].building == LABORATORY)
		{
		  order->stages[stage][3] = ORDER1;
		}
	      else
		{
		  order->stages[stage][4] = SPELL;
		}
	      order->stages[stage][4] = fortrecipes[i].unlocktype[0];
	      order->stages[stage][8] = 6;
	      order->stages[stage][9] = fortrecipes[i].unlocktype[1];
	    }
	  order->automatic = true;
	  order->enabled = true;
	  order->curamount = fortrecipes[i].resources[0][2];
	  exists = find_order(order->name);
	  if (exists == NONE)
	    {
	      fortorders.push_back(*order);
	    }
	  delete order;
	}
    }
}
