/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#ifndef RECIPE_HPP
#define RECIPE_HPP
#include <string>
#include "libtcod.hpp"

#define RESOURCES 4

/**
 * Contains all information about recipe.
 */
class Recipe
{
public:
  /**
   * Recipe name - must be that same like produced item
   */
  std::string name;
  /**
   * Resources needed for this recipe
   */
  int resources[RESOURCES][3];
  /**
   * Amount of fuel needed by recipe.
   */
  int fuel;
  /**
   * Amount of produced items.
   */
  int amount;
  /**
   * Time needed for made selected item
   */
  int worktime;
  /**
   * Building type in which work will be do.
   */
  int building;
  /**
   * Did recipe is known.
   */
  bool known;
  /**
   * What will be unlocked after finishing this recipe
   */
  int unlocktype[2];
  /**
   * Name of recipe to unlock (if other than this name recipe)
   */
  std::string unlockname;
  /**
   * String information about needed materials.
   */
  std::string materials;

  Recipe();
};

int find_recipe(std::string name, int building = -1);
int find_recipe2(std::string name, TCODList<int> *buildings);
void add_recipes(int build);

#endif
