/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "gui/gui.hpp"
#include "main.hpp"
#include "ui.hpp"
#include "mainmenu.hpp"
#include "game.hpp"
#include "designation.hpp"
#include "design.hpp"
#include "message.hpp"
#include "mission.hpp"
#include <iostream>
#include <stdio.h>
#include "adventure.hpp"
#include "help.hpp"
#include "mobileui.hpp"

/**
 * Main menu container
 */
VBox *mainmenu = new VBox(42, 20, 1);
/**
 * Label with copyright information
 */
Label *credits = new Label(35, 74, "(C)2011 Bartek thindil Jasicki");
/**
 * Game menu container
 */
ToolBar *gamemenu = new ToolBar(80, 2, 1, "Menu");
/**
 * Label with information about Escape key
 */
Label *escinfo = new Label(40, SCREENHEIGHT - 3, "Hit ESC to quit");
/**
 * Popup with information about object
 */
ToolBar *objinfo = new ToolBar(30, 20, 1, "Info");
/**
 * Label with name of object - Popup
 */
Label *objname = new Label(0, 0, "");
/**
 * Label with description of object - Popup
 */
Label *objdesc = new Label(0, 0, "");
/**
 * Status bar with tips.
 */
StatusBar *tips = new StatusBar(0, SCREENHEIGHT - 1, SCREENWIDTH - 1, 1);
/**
 * Label with notices
 */
Label *lblnotice = new Label(0, 0, "");
/**
 * Label with time
 */
Label *lbltime = new Label(76, 0, "");

/**
 * Set visibility of game menu
 * @param visible If true, menu is visible, otherwise false.
 */
void setVisibleGamemenu(bool visible)
{
  lblnotice->setVisible(visible);
  lbltime->setVisible(visible);
  gamemenu->setVisible(visible);
  if (!visible)
    {
      setInfo(visible);
      lblnotice->setValue("");
      lblnotice->computeSize();
    }
}

/**
 * Show informations about game
 * @param w Unused
 * @param data Unused
 */
void aboutClb(Widget *w, void* data)
{
  mainmenu->setVisible(false);
  state = ABOUT;
  about();
}

/**
 * Quit from game.
 * @param w Unused
 * @param data Unused
 */
void quitClb(Widget *w, void* data)
{
  state = ENDGAME;
}

/**
 * Continue saved game.
 * @param w Unused
 * @param data Unused
 */
void continueClb(Widget *w, void* data)
{
  if (load_game())
    {
      mainmenu->setVisible(false);
      credits->setVisible(false);
      if (state == FORT)
	{
	  setVisibleGamemenu(true);
	  tips->setVisible(true);
	}
      else
	{
	  setVisibleAdvmenu(true);
	}
      Notice("Game loaded!");
    }
}

/**
 * Start new game.
 * @param w Unused
 * @param data Unused
 */
void newClb(Widget *w, void* data)
{
  state = FORT;
  mainmenu->setVisible(false);
  credits->setVisible(false);
  new_game();
  setTime();
  setVisibleGamemenu(true);
  tips->setVisible(true);
}

/**
 * Game menu options
 * @param w Unused.
 * @param data Selected option.
 */
void menuClb(Widget *w, void *data)
{
  int *option = static_cast<int*>(data);
  switch (*option)
    {
    case ORDERS:
      setVisibleOrders(true);
      break;
    case CREW:
      setVisibleCrew(true);
      break;
    case OBJECTS:
      setVisibleItems(true);
      break;
    case BUILD:
      setVisibleBuild(true);
      break;
    case MARKS:
      setVisibleDesignation(true);
      break;
    case DESIGN:
      if (find_build("", WORKSHOP, true) == NONE)
	{
	  Notice("You need workshop to enable design steambots.");
	  return;
	}
      setVisibleDesign(true);
      break;
    case MESSAGES:
      setVisibleMessage(true);
      break;
    case MISSIONS:
      if (find_build("", INN, true) == NONE)
	{
	  Notice("You need Inn to enable this option.");
	  return;
	}
      setVisibleMission(true);
      break;
    case CAST:
      if (fortmobs[0].spells.isEmpty())
	{
	  Notice("You don't know any spells!");
	  return;
	}
      state = CAST;
      setVisibleSpells(true);
      break;
    default:
      break;
    }
  if (*option != FORT)
    {
      Widget::setForegroundColor(TCODColor::white, TCODColor::red);
      state = (GameState)*option;
      setVisibleGamemenu(false);
    }
}

/**
 * Save game and quit to main menu.
 * @param w Unused
 * @param data Unused
 */
void saveClb(Widget *w, void* data)
{
  save_game();
  Widget::setForegroundColor(TCODColor::white, TCODColor::red);
}

/**
 * Make screenshot
 * @param w Unused.
 * @param data Unused.
 */
void screenshotClb(Widget *w, void* data)
{
  TCODSystem::saveScreenshot(NULL);
}

/**
 * Open help.
 * @param w Unused.
 * @param data Unused.
 */
void helpClb(Widget *w, void *data)
{
  if (state == FORT)
    {
      state = HELP;
      setVisibleGamemenu(false);
    }
  else
    {
      state = HELP2;
      setVisibleAdvmenu(false);
    }
  tips->setVisible(false);
  Widget::setForegroundColor(TCODColor::white, TCODColor::red);
  showHelp(0);
}

/**
 * Use items.
 * @param w Unused.
 * @param data Unused.
 */
void useClb(Widget *w, void *data)
{
  bool have = false;
  if (state == FORT)
    {
      if (fortmobs[0].inventory.isEmpty())
	{
	  Notice("You don't have any items in your inventory");
	  //return;
	}
      for (int i = 0; i < fortmobs[0].inventory.size(); i++)
	{
	  if (fortobjs[fortmobs[0].inventory.get(i)].types.contains(USEABLE))
	    {
	      have = true;
	      break;
	    }
	}
      if (!have)
	{
	  Notice("You don't have any items to use.");
	  //return;
	}
      state = USE;
      setVisibleGamemenu(false);
    }
  else
    {
      if (advmobs[0].inventory.isEmpty())
	{
	  Notice("You don't have any items in your inventory");
	  return;
	}
      for (int i = 0; i < advmobs[0].inventory.size(); i++)
	{
	  if (advobjs[advmobs[0].inventory.get(i)].types.contains(USEABLE))
	    {
	      have = true;
	      break;
	    }
	}
      if (!have)
	{
	  Notice("You don't have any items to use.");
	  return;
	}
      state = USE2;
      setVisibleAdvmenu(false);
    }
  Widget::setForegroundColor(TCODColor::white, TCODColor::red);
  setVisibleUse(true);
}

/**
 * Show informations about selected object.
 * @param visible If true, show info box.
 * @param x X-axis position of object. Default is -1.
 * @param y Y-axis position of object. Default is -1.
 */
void setInfo(bool visible, int x, int y)
{
  if (visible)
    {
      string text;
      string desc;
      int index = find_mob("", x + screenx, y + screeny);
      if (index == NONE)
	{
	  index = find_obj("", x + screenx, y + screeny);
	  if (index == NONE)
	    {
	      index = find_build("", NONE, true, x + screenx, y + screeny);
	      if (index == NONE)
		{
		  index = find_build("", NONE, false, x + screenx, y + screeny);
		  if (index == NONE)
		    {
		      objinfo->setVisible(false);
		      return;
		    }
		}
	      text = fortbuilds[index].name;
	      desc = fortbuilds[index].description;
	    }
	  else
	    {
	      text = fortobjs[index].name;
	      desc = fortobjs[index].description;
	    }
	}
      else
	{
	  text = fortmobs[index].name;
	  desc = fortmobs[index].description;
	}
      objname->setValue(text.c_str());
      objdesc->setValue(desc.c_str());
      objinfo->computeSize();
    }
  objinfo->setVisible(visible);
}

/**
 * Show message for player.
 * @param message Message to show.
 */
void Notice(const char* message)
{
  lblnotice->setValue(message);
  lblnotice->computeSize();
}

/**
 * Set value for clock.
 */
void setTime()
{
  char *c = new char[15];
  string season;
  if (gtime[2] <= 3)
    {
      season = "Winter";
    }
  else if ((gtime[2] > 3) && (gtime[2] <= 6))
    {
      season = "Spring";
    }
  else if ((gtime[2] > 6) && (gtime[2] <= 9))
    {
      season = "Summer";
    }
  else if (gtime[2] > 9)
    {
      season = "Autumn";
    }
  if (gtime[0] - 12 < 0)
    {
      sprintf(c, "%dam %dd %dm %dy %s", gtime[0], gtime[1], gtime[2], gtime[3], season.c_str());
    }
  else
    {
      int hour = gtime[0] - 12;
      if (hour == 0)
	{
	  hour = 12;
	}
      sprintf(c, "%dpm %dd %dm %dy %s", hour, gtime[1], gtime[2], gtime[3], season.c_str());
    }
  lbltime->setValue(c);
}

/**
 * Create all user interface.
 */
void createUI()
{
  //Create main menu
  mainmenu->addWidget(new Button("New game", "", newClb, NULL));
  mainmenu->addWidget(new Button("Continue game", "", continueClb, NULL));
  mainmenu->addWidget(new Button("Quit game", "", quitClb, NULL));
  mainmenu->addWidget(new Button("About game", "", aboutClb, NULL));
  //Create game menu
  gamemenu->addWidget(new Button("Orders", "Set orders for robots.", menuClb, new int(ORDERS)));
  gamemenu->addWidget(new Button("Crew", "Informations about you and your robots.", menuClb, new int(CREW)));
  gamemenu->addWidget(new Button("Resources", "Available resources/items.", menuClb, new int(OBJECTS)));
  gamemenu->addWidget(new Button("Build", "Build buildings.", menuClb, new int(BUILD)));
  gamemenu->addWidget(new Button("Designate", "Designate resources for orders, build walls.", menuClb, new int(MARKS)));
  gamemenu->addWidget(new Button("Design robots", "Design robots. You need workshop for this.", menuClb, new int(DESIGN)));
  gamemenu->addWidget(new Button("Cast spell", "Cast spell. You need library for develop spells.", menuClb, new int(CAST)));
  gamemenu->addWidget(new Button("Use item", "Use item from your inventory.", useClb, NULL));
  gamemenu->addWidget(new Button("Messages", "Display messages.", menuClb, new int(MESSAGES)));
  gamemenu->addWidget(new Button("Adventure", "Go on adventure.", menuClb, new int(MISSIONS)));
  gamemenu->addSeparator("");
  gamemenu->addWidget(new Button("Help", "Usefull information", helpClb, NULL));
  gamemenu->addWidget(new Button("Make screenshot", "Make screenshot and save it in main game directory.", screenshotClb, NULL));
  gamemenu->addWidget(new Button("Save and quit", "Save game and quit to main menu.", saveClb, NULL));
  objinfo->addWidget(objname);
  objinfo->addWidget(objdesc);
  objinfo->setVisible(false);
  escinfo->setVisible(false);
  setVisibleGamemenu(false);
  tips->setVisible(false);
  buildMobileUI();
  buildOrderUI();
  buildItemsUI();
  buildBuildUI();
  buildDesignationUI();
  buildDesignUI();
  buildMessageUI();
  buildMissionUI();
  buildAdventureUI();
  buildSpellsUI();
}
