/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "libtcod.hpp"
#include "main.hpp"

/**
 * Check if selected position is valid
 * @param x X-axis position
 * @param y Y-axis position
 * @param walkable Did position is walkable? Default true.
 * @return true if position is valid, otherwise false.
 */
bool valid_pos(int x, int y, bool walkable = true)
{
  if ((x < 0) || (x >= MAPWIDTH))
    {
      return false;
    }
  if ((y < 0) || (y >= MAPHEIGHT))
    {
      return false;
    }
  if ((!fovmap->isWalkable(x, y)) && (walkable))
    {
      return false;
    }
  return true;
}

/**
 * Check if file of saved game exists.
 * @return True if exists, otherwise false.
 */
bool save_exists()
{
  TCODList<char*> files = TCODSystem::getDirectoryContent("data", "savegame.dat");
  bool retval = files.isEmpty();
  files.clear();
  return !retval;
}

/**
 * Find nearest land of selected type.
 * @param x X-axis position to start. Return X-axis position of land or -1.
 * @param y Y-axis position to start. Return Y-axis position of land or -1.
 * @param radius Size of radius to search.
 * @param type Type of land to search.
 */
void find_near_land(int *x, int *y, int radius, int type)
{
  int sum, oldsum = 1000, tmpx = NONE, tmpy = NONE;
  for (int y1 = *y - (radius + 1); y1 < *y + (radius + 1); y1++)
    {
      for (int x1 = *x - (radius + 1); x1 < *x + (radius + 1); x1++)
	{
	  if ((y1 < 0) || (x1 < 0))
	    {
	      continue;
	    }
	  sum = abs(*x - x1) + abs(*y - y1);
	  if (sum > oldsum)
	    {
	      continue;
	    }
	  if (fortmap[y1][x1].landtype != type)
	    {
	      continue;
	    }
	  tmpx = x1;
	  tmpy = y1;
	  oldsum  = sum;
	}
    }
  *x = tmpx;
  *y = tmpy;
}
